-- ATE Tool database upgrade script 
-- From version     : 1.10.x
-- To version       : 1.11.0
-- Target DB engine : MySQL 5.5.x

-- Adds placeholder flag to the Task table
ALTER TABLE `tool`.`Task` ADD COLUMN `Fake` TINYINT NOT NULL DEFAULT '0';
-- Adds new column into the table Setting
ALTER TABLE `Setting` ADD COLUMN `ShowFakeTasks` TINYINT(1) NOT NULL DEFAULT '1';
-- Adds Task Fake flag for placeholder tasks
UPDATE `Task` SET `Fake` = 1 WHERE CHAR_LENGTH(`SAP_ID`) = 10;
-- Update placehodler task SAP ID to correct pattern
UPDATE `Task` SET `SAP_ID` = CONCAT(`Task`.`SAP_ID`, ' 0010') WHERE CHAR_LENGTH(`SAP_ID`) = 10;
-- Adds Task import flag for placeholder tasks 
ALTER TABLE `tool`.`Task` ADD COLUMN `ActualsImportBySAP_ID` TINYINT NOT NULL DEFAULT '0';
-- Adds ActualsCost flag for actuals import from iASAP
ALTER TABLE `tool`.`ActualsCost` ADD COLUMN `Source` TINYINT NOT NULL DEFAULT '0';
-- Drop trigger DecideLabourTask from TaskPlan table. The decision is moved to SAP import
DROP TRIGGER IF EXISTS `DecideLabourTask`;
-- Replace old workpackage structure for placeholder task with new one
-- For every root place holder WP creates new lvl 2 WP
INSERT INTO `tool`.`WorkPackage` (`SAP_ID`, `Project_ID`, `Parent_ID`, `Name`, `Date`)
     SELECT CONCAT(`WorkPackage`.`SAP_ID`, '-Placeholder'), NULL, `WorkPackage`.`WorkPackage_ID`, `WorkPackage`.`Name`, `WorkPackage`.`Date`
         FROM `tool`.`WorkPackage` WHERE `WorkPackage`.`SAP_ID` LIKE '%PH%'  AND CHAR_LENGTH(`WorkPackage`.`SAP_ID`) = 9;
-- Update placeholder task WP ID
UPDATE `Task` SET `Task`.`WorkPackage_ID` = 
  (SELECT `WorkPackage`.`WorkPackage_ID` FROM `WorkPackage` WHERE `WorkPackage`.`SAP_ID` = 
    CONCAT((SELECT `Project`.`SAP_ID` FROM `Project` WHERE `Project`.`Project_ID` = `Task`.`Project_ID`), '-Placeholder'))
     WHERE `Task`.`Fake` = 1; 
-- Deletes old WPs
DELETE FROM `tool`.`WorkPackage` WHERE `WorkPackage`.`SAP_ID` LIKE '%PH%' AND CHAR_LENGTH(`WorkPackage`.`SAP_ID`) = 11;


-- Update displayed content {D&S, HTS} > {AERO}
UPDATE IGNORE `SettingDisplayedContent` SET `OrganizationUnit_ID` = 121 WHERE `OrganizationUnit_ID` IN (3, 87);
DELETE FROM `SettingDisplayedContent` WHERE `OrganizationUnit_ID` IN (3, 87);

-- Update responsible units for projects {D&S, HTS} > {AERO}
UPDATE `Project` SET `ResponsibleUnit` = 121 WHERE `ResponsibleUnit` IN (3, 87);

-- Remove phase Pilot from set of project phases
UPDATE `Project` SET `Phase` = 'Active' WHERE `Phase` = 'Pilot';
ALTER TABLE `Project` CHANGE `Phase` `Phase` ENUM('Planned','Closed','Active','Cancelled') NOT NULL DEFAULT 'Planned';

-- Set SBEs D&S and HTS as virtual
UPDATE `tool`.`OrganizationUnit` SET `Virtual` = '1' WHERE `OrganizationUnit`.`OrganizationUnit_ID` = 3;
UPDATE `tool`.`OrganizationUnit` SET `Virtual` = '1' WHERE `OrganizationUnit`.`OrganizationUnit_ID` = 87; 