<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for Actuals cost import.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ActualsCostImportPresenter extends SecuredPresenter
{

    /**
     * Nette default render method 
     */
    public function renderDefault()
    {
        $formDefaultReport = array();
        $importForm = $this['importForm'];
        $importForm->setDefaults($formDefaultReport);
        $this->template->importForm = $importForm;
    }

    /**
     * Creates ImportForm for upload file with actuals cost.
     * 
     * @param string $name ImportForm.
     * @return AppForm ImportForm.
     */
    public function createComponentImportForm($name)
    {
        $form = new AppForm($this, $name);

        $years = array();
		for ($i = 2012; $i <= (date('Y')); $i++)
			$years[$i] = $i;        
        $form->addSelect('year', 'Year:', $years)->setDefaultValue(date('Y'));
		
        $form->addUpload('file', 'Select csv file:');

		$form->addHidden('import_token_value_id');
		
        $submit_button = $form->addSubmit('update', 'Import');
		if (!$this->testUserRoles('PMO'))
			$submit_button->setDisabled(true);

        $form->onSuccess[] = array($this, 'importFormSubmitted');

        return $form;
    }
    
    /**
     * ImportForm submitted.
     * 
     * @param AppForm $form ImportForm submitted.
     */
    public function importFormSubmitted(AppForm $form)
    {
        $entry = $form->getValues();
		//Only PMO user can run import
		if ($this->testUserRoles('PMO'))
		{
			if ($form->isValid())
			{
				//check if imported file is csv type
				if(!preg_match('#.csv$#', $entry['file']->name))
				{
					$this->flashMessage("Imported file has to be in csv format!", 'error');
					setcookie('importToken', $entry['import_token_value_id']);
					$this->redirect('default');
				}
				//extending time limit for import.
				$exec_time = 600;
				if (!ini_set("max_execution_time", $exec_time)) 
				{
					$this->flashMessage('Setup of maximum execution time failed. Contact administrator.', 'error');
					trigger_error('Unable to setup maximum execution time to '.$exec_time.'.', E_USER_ERROR);
				}
				//log initialization
				$multilog = new MultiLogger();
				$batch_id = 'ACI-'.time().'-'. rand(1, 9);
				$multilog->add($this->context->createLogger($batch_id, "Actuals Cost Import"));
				$multilog->log('Actuals Cost Import started.', ILogger::INFO);
				// drop data for selected year
				if(!is_null($this->context->actualsCostRepository->deleteDataFromYear($entry['year'], ActualsCost::SOURCE_SAP)))
				{
					$multilog->log('Actuals Cost deleted for year: ' . $entry['year'], ILogger::INFO);
				}
				else
				{
					$multilog->log('Deleting of Actuals Cost for year: ' . $entry['year'] . ' failed.', ILogger::ERROR);
				}
				//variables initialization
				$importResult = true;
				$delimiter = ";";
				$enclosure = '"';				
				$header = array();
				$actualsCostData = array();
				$totalRecords = 1;
				// cache SAP_ID => Tasks_ID array
				$taskSAPIDs = $this->context->taskRepository->findAllProjectTasksSAPIDs(TRUE);
				
				//open csv file
				$file_handle = fopen($entry['file'], "r");
				if($file_handle)
				{
					// process file
					while (!feof($file_handle))
					{
						$counter = 0;
						while (!feof($file_handle) && ($counter++ < 1000))
						{
							//gets row from csv file
							$rowData = fgetcsv($file_handle, 0, $delimiter, $enclosure);
							//gets header
							if(empty($header))
							{
								$header = $rowData;
							}
							//gets data
							else
							{
								if($rowData)
								{
									$actualsCostData[] = $rowData;
								}
							}
						}
						$actualsCostArray = array();
						// 
						foreach($actualsCostData as $rowNumber => $row)
						{
							foreach($row as $key => $value)
							{
								$actualsCostData[$rowNumber][trim($header[$key])] = $value;
								unset($actualsCostData[$rowNumber][$key]);
							}
							//creates ActualsCost object and saves it to $actualsCostArray for import
							$actualsCost = new ActualsCost();
							$totalRecords++;
							try
							{
								$chargeNumber = str_replace(' ', '', $actualsCostData[$rowNumber]['Object']);
								
								// we don't have a Task in DB, which has this Charge Number
								if (!isset($taskSAPIDs[$chargeNumber]))
									continue;
								
								$actualsCost->Task_id = $taskSAPIDs[$chargeNumber];
								$actualsCost->Project_sap_id = $actualsCostData[$rowNumber]['Project Def.'];
								$actualsCost->Activity_name = $actualsCostData[$rowNumber]['CO object name'];
								$actualsCost->Creation_date = date('Y-m-d', strtotime($actualsCostData[$rowNumber]['Created on']));
								$actualsCost->Posting_date = date('Y-m-d', strtotime($actualsCostData[$rowNumber]['Postg Date']));
								$actualsCost->Description = $actualsCostData[$rowNumber]['Name'];
								$postDate = DateTime::createFromFormat('d.m.Y', $actualsCostData[$rowNumber]['Postg Date']);
								$actualsCost->Year = FiscalCalendar::getFiscalMonthByTimestamp($postDate->getTimestamp())->year;
								$actualsCost->Fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp($postDate->getTimestamp())->index;
								$actualsCost->Category = $actualsCostData[$rowNumber]['Category'];
								$actualsCost->Quantity = str_replace(',', '.', $actualsCostData[$rowNumber]['Quantity']);
								$actualsCost->Quantity = floatval(str_replace(' ', '', $actualsCost->Quantity));
								$actualsCost->Value_usd = str_replace(',', '.', $actualsCostData[$rowNumber]['USD']);
								$actualsCost->Value_usd = floatval(str_replace(' ', '', $actualsCost->Value_usd));
								$actualsCost->Value_czk = str_replace(',', '.', $actualsCostData[$rowNumber]['Vbl. value/Obj. curr']);
								$actualsCost->Value_czk = floatval(str_replace(' ', '', $actualsCost->Value_czk));
								$actualsCost->Value = str_replace(',', '.', $actualsCostData[$rowNumber]['Vbl. value/TranCurr.']);
								$actualsCost->Value = floatval(str_replace(' ', '', $actualsCost->Value));
								$actualsCost->Currency = $actualsCostData[$rowNumber]['TCurr'];
								$actualsCost->Source = ActualsCost::SOURCE_SAP;
							}
							catch (InvalidArgumentException $e) 
							{
								Debugger::log('Actuals Cost Import: ' . $e->getMessage() . ' Row: ' . $totalRecords, Debugger::ERROR);
								$multilog->log('Actuals Cost Import: ' . $e->getMessage() . ' Row: ' . $totalRecords, ILogger::ERROR);
								$this->flashMessage("Import of row: " . $totalRecords . " failed. See log.", 'error');
								// redirect?
								// setcookie('importToken', $entry['import_token_value_id']);
								//$this->redirect('Maintenance:log');
								//$this->redirect('default');
							}
							$actualsCostArray[] = $actualsCost;
						}
						$actualsCostData = array();

						// run import for prepared part of data
						if($this->context->actualsCostRepository->importActualsCost($actualsCostArray))
						{
							$importResult = true;
						}
						else
						{
							$importResult = false;
						}
					}
					fclose($file_handle);
					if($importResult)
					{
						$multilog->log('Actuals Cost Imported successfully.', ILogger::NOTICE);
						$this->flashMessage("Actuals Cost were imported.", 'success');
					}
					else 
					{
						$this->flashMessage("Actuals Cost import failed.", 'error');
					}
				}
				else
				{
					$multilog->log('Actuals Cost Import: Cant access uploaded file.', ILogger::ERROR);
					$this->flashMessage("Can't access uploaded file.", 'error');
				}
			}
			else
			{
				$this->flashMessage("Invalid form values.", 'error');
			}
		}
		else
		{
			$this->flashMessage("You are not authorized to import actuals cost. PMO role required.", 'error');
		}
		setcookie('importToken', $entry['import_token_value_id']);
        $this->redirect('default');
    }
}