<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for assignment vs workable hours report.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class AssignmentWHReportPresenter extends SecuredPresenter
{        
    /** @const Key to form state */
    const SELECT_FORM_KEY = 'assignment_workable_hours_report_select_form';

    /**
     * Shows default page
     * @param int $year Selected year
     */
    public function renderDefault($year = NULL)
    {
        if ($year)
        {
            if ($this->context->formStateRepository->exists(self::SELECT_FORM_KEY))
                $this->context->formStateRepository->load(self::SELECT_FORM_KEY, $this['selectForm']);
        }
        else
        {
            if ($this->context->formStateRepository->exists(self::SELECT_FORM_KEY))
                $this->context->formStateRepository->drop(self::SELECT_FORM_KEY);
        }
    }
    
    /**
     * Creates form for download.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentSelectForm($name)
    {
        $form = new AppForm();

        $years = array();
        for ($i = 2011; $i <= (date('Y')+1); $i++)
            $years[$i] = $i;
        $form->addSelect('Year', 'Year', $years)
                ->setDefaultValue(date('Y'));

        $form['OrganizationUnitSelector'] = new OrganizationUnitSelectorContainer(380, FALSE);
		$form['OrganizationUnitSelector']['SBE']->addRule(Form::FILLED, 'Please select SBE at least!');
            $form['OrganizationUnitSelector']['ajaxHandle']->setValue($this->link('OrganizationUnitSelector'));

        $form['OrganizationUnitSelector']['SBE']
                ->setPrompt('*');
        $form['OrganizationUnitSelector']['Department']
                ->setPrompt('*');
        $form['OrganizationUnitSelector']['Cluster']
                ->setPrompt('*');
        $form['OrganizationUnitSelector']['TechnicalGroup']
                ->setPrompt('*');

        $form->addSubmit('send', 'Report');

        $form->onSuccess[] = callback($this, 'selectFormSubmitted');
		
        return $form;
    }    
    
    /**
     * Handle for send button of selectForm
     * 
     * @param AppForm $form
     */
    public function selectFormSubmitted(AppForm $form)
    {
		$values = $form->getValues();
		
        $orgUnit_id = NULL;
        if ($values->OrganizationUnitSelector->TechnicalGroup)
            $orgUnit_id = $values->OrganizationUnitSelector->TechnicalGroup;
        elseif ($values->OrganizationUnitSelector->Cluster)
            $orgUnit_id = $values->OrganizationUnitSelector->Cluster;
        elseif ($values->OrganizationUnitSelector->Department)
            $orgUnit_id = $values->OrganizationUnitSelector->Department;
        elseif ($values->OrganizationUnitSelector->SBE)
            $orgUnit_id = $values->OrganizationUnitSelector->SBE;

		$multilog = new MultiLogger();
		$batch_id = 'ASSIGNMENTvWH-'.time().'-'. rand(1, 9);
		$multilog->add($this->context->createLogger($batch_id, "ASSIGNMENT vs WORKABLE HOURS generation"));
		$assignmentWHReportService = new AssignmentWHReportService(
            $this->context,
            $multilog,
            $values->Year,
            $orgUnit_id,
            time()
        );
		$importResult = true;
		
        try
		{
			$assignmentWHReportService->runAssignmentWHReportGeneration();
		}
		catch (EAssignmentWHReportServiceException $e)
		{
			$importResult = false;
			Debugger::log('Assignment vs Workable hours manual generation failed: ' . $e->getMessage(), Debugger::ERROR);
			$multilog->log('Assignment vs Workable hours manual generation failed: ' . $e->getMessage(), ILogger::ERROR);
			$this->flashMessage('Assignment vs Workable hours manual generation failed: ' . $e->getMessage(), 'error');
		}

		if($importResult)
		{
			$multilog->log('Assignment vs Workable hours manual generation completed.', ILogger::NOTICE);
			$this->flashMessage('Report generation was successful.', 'success');
		}

        $this->context->formStateRepository->save(self::SELECT_FORM_KEY, $form);

		$this->redirect('this', $values->Year);
    }
}