<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * DashboardPresenter presents first information
 * for user after login. It is a combination of useful information
 * that should be available after login.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class DashboardPresenter extends SecuredPresenter
{
	/**
	 * Shows default page
	 */
	public function renderDefault()
	{
		$this->template->latestNews = $this->context->newsRepository->findLatestNews(time(), 10);
		$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp(time());	
		if ($fiscal_month === null)
			throw new ApplicationException('Fiscal month is not defined current date.');
		else
		{
			$report_service = new UserMonthlyReportService($this->context);
			$report_service->prepareUserMonthlyTaskReport(Environment::getUser()->getId(), $fiscal_month);
			$this->template->report = $report_service->getReport();
			$this->template->totals = $report_service->getTotals();
		}

		// Fiscal month
		$this->template->fiscalMonth = $fiscal_month->From . ' - ' . $fiscal_month->To;

        // Charge numbers
        $this->template->charge_numbers = array(
 	            1 => array(0 => 'Training', 'aero' => '501261', 'corp' => '502501'),
				2 => array(0 => 'Meetings', 'aero' => '501263', 'corp' => '502503'),
				3 => array(0 => 'Work Direction/Group Lead', 'aero' => '501273', 'corp' => '502513'),
				4 => array(0 => 'Equipment Maintenance', 'aero' => '501262', 'corp' => '502502'),
				5 => array(0 => 'Process Improvement', 'aero' => '501269', 'corp' => '502509'),
				6 => array(0 => 'Holiday', 'aero' => '501276', 'corp' => '502516'),
				7 => array(0 => 'Vacation', 'aero' => '501277', 'corp' => '502517'),
				8 => array(0 => 'Compensatory vacation', 'aero' => 'no charging', 'corp' => 'no charging'),
				9 => array(0 => 'Indir:Absence (Ilness, wedding, funeral, birth of child)', 'aero' => '501278', 'corp' => '502518'),
				10 => array(0 => 'General Admin Labor', 'aero' => '501260', 'corp' => '502500'),
				11 => array(0 => 'Health, Safety & Environm', 'aero' => '501266', 'corp' => '502506')
        );
        
        /* Project charter where user is involved */
        $projectCharterReportService = new ProjectCharterDashboardService($this->context);
        $projectCharterReportService->prepareProjectCharterDashboardService(Environment::getUser()->getId());
        $this->template->projectCharterReportService = $projectCharterReportService->getProjectCharterDashboardService();
        
        // Projects where user is involved
		$this->template->user_projects = $this->getUserProjects();
	}

	/**
	 * Show detail of news
	 */
	public function renderShowNews($id)
	{
		$news = $this->context->newsRepository->find($id);
		if ($news != null)
			$this->template->news = $this->context->newsRepository->find($id);
		else
			$this->flashMessage ('Invalid news ID. Cannot render.', 'error');
	}


    /**
     * Returns list of projects where current user is involved
     *
     * @return array
     */
	protected function getUserProjects()
	{
		$projects = $this->context->projectRepository->findByPerson($this->getUser()->getId());

		foreach ($projects as $project)
		{
			if (($project->Phase == Project::PROJECT_PHASE_CLOSED) || ($project->Phase == Project::PROJECT_PHASE_CANCELLED))
				unset($projects[$project->ID]);
		}

		return $projects;
	}
	
}