<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * EChargeImportPresenter presents interface for importing data from eCharge.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 * @todo Load mail addresses dynamically from DB
 */
class EChargeImportPresenter extends SecuredPresenter
{
    
    /**
     * eCharge service for importing actuals
     * @var EChargeImportService 
     */
    private $eChargeImportService;
    
    /**
     * Path to logfile
     * @var string
     */
    private $logfileName = 'echarge_import.html';
    
    /**
     * Nette default startup method 
     */
    public function startup()
    {
        parent::startup();
        
        if (!$this->isAllowed(self::RESOURCE_IMPORT, self::ACTION_VIEW))
        {
            $this->flashMessage('You don\'t have permission to use eCharge Import. You were redirected.');
            $this->redirect('Dashboard:');
        }        
    }
    
    /**
     * Nette default render method 
     */
	public function renderDefault()
	{
        $this->template->lastUpdated = $this->context->actualsRepository->getLastImportDate(Actuals::SOURCE_ECHARGE);
        $this->template->maxExecTime = EChargeImportService::MAXIMUM_EXECUTION_TIME;
		$lock = $this->context->createLock('ECHARGE_IMPORT');
		$this->template->running = $lock->locked();
	}
    
    /**
     * Run method 
     */
    public function renderRun()
    {
		if (!$this->isAllowed(self::RESOURCE_IMPORT, self::ACTION_RUN))
        {
            $this->flashMessage('You don\'t have permission to use eCharge Import. You were redirected.');
            $this->redirect('Dashboard:');
        }
		
		/* init logging */
		$logs_suffix = time().'-'. rand(1, 9);
		$current_log_file_name = 'echarge_import-'.$logs_suffix.'.html';
		$filelog_path = TEMP_DIR . DIRECTORY_SEPARATOR . $current_log_file_name;
		$batch_id = 'echarge-'.$logs_suffix;
        $filelog = $this->context->createFileLogger($filelog_path, $batch_id, "ECHARGE ACTUALS") ;
		$multilog = new MultiLogger();
		$multilog->add($filelog);
		$multilog->add($this->context->createLogger($batch_id, "ECHARGE ACTUALS"));

		$import_result = FALSE;
		$import_ex_error = '';
		
        $this->eChargeImportService = new EChargeImportService($this->context, $multilog);
        
        try
        {
            $import_result = $this->eChargeImportService->runImport();
        }
        catch (EChargeImportServiceException $e)
        {
            Debugger::log('eCharge import failed: ' . $e->getMessage(), Debugger::ERROR);
			$multilog->log('eCharge import failed: ' . $e->getMessage(), ILogger::ERROR);
            $import_result = FALSE;
            $import_ex_error = $e->getMessage();
        }
		
		$import_result = FALSE;
		$import_ex_error = '';
		
        $this->eChargeImportService = new EChargeIASAPImportService($this->context, $multilog);
        
        try
        {
            $import_result = $this->eChargeImportService->runImport();
        }
        catch (EChargeIASAPImportServiceException $e)
        {
            Debugger::log('eCharge import failed: ' . $e->getMessage(), Debugger::ERROR);
			$multilog->log('eCharge import failed: ' . $e->getMessage(), ILogger::ERROR);
            $import_result = FALSE;
            $import_ex_error = $e->getMessage();
        }
		
        $filelog->close();
		
        $this->template->lastUpdated    = $this->context->actualsRepository->getLastImportDate(Actuals::SOURCE_ECHARGE);
        $this->template->logfilePath    = basename(TEMP_DIR) . '/' . $current_log_file_name;
		$this->template->batchNumber	= $batch_id;
		$this->template->result			= $import_result;
		$this->template->error			= $import_ex_error;
    }
	
}
