<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for export control month check.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ExportControlMonthCheckPresenter extends TokenAccessPresenter
{
	/* authentication token setup */
	const AUTH_TOKEN_NAME = 'token',
		  AUTH_TOKEN_KEY = 'exportControlMonthCheck.credentials.cronToken';
	
    /**
     * Returns http parameter name with authentication token.
     * 
     * @return string
     */
	protected function getAuthTokenName()
	{
		return self::AUTH_TOKEN_NAME;
	}

    /**
     * Returns http parameter name with authentication token.
	 * 
     * @return string
     */
	protected function getAuthTokenSettingsKey()
	{
		return self::AUTH_TOKEN_KEY;
	}
	
    /**
     * Nette default render method 
     */
	public function renderDefault()
	{
	}
    
	/**
     * Export control check method 
     */
    public function renderExportControlCheck()
    {
		$counter = 0;
		$year = Date('Y');
		//log ini
		$multilog = new MultiLogger();
		$batch_id = 'EC-'.time().'-'. rand(1, 9);
		$multilog->add($this->context->createLogger($batch_id, "Excport control auto check"));
		$multilog->log('EC auto control check starts.', ILogger::NOTICE);
		//Licence control mechanism
		$persons = $this->context->personRepository->findAll();
		foreach($persons as $person)
		{
			if($person->Status == 1)
			{
				$personLicences = $this->context->personECLicenceRepository->findByEID($person->ID);
				foreach($personLicences as $licence)
				{
					$removeLicence = true;
					$allPersonProjects = $this->context->projectRepository->findPersonAssignedAtProjectsInYear($person->ID, $year);
					foreach($allPersonProjects as $personProject)
					{
						if($personProject->ECLicenceID == $licence->ECLicenceID)//check if person has any other assignment under this licence. If other assignment doesn't exist, remove licence from preson.
						{
							$projectTasks = $this->context->taskRepository->findPersonTasksInYear($person->ID, $personProject->ID, $year);
							foreach($projectTasks as $task)
							{
								$assignment = 0;
								$assigns = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task->ID, $year, $person->ID);
								foreach($assigns as $assign)
								{
									if($assign->Month >= Date('m'))
										$assignment += $assign->Hours;
								}
								if($assignment > 0)
								{
									$removeLicence = false;
									break 2;
								}
							}
						}
					}
					if($removeLicence)
					{
						$counter++;
						$multilog->log('EC auto control check removes licence: ' . $licence->ECLicenceID . ' from person: ' . $person->ID . '.', ILogger::NOTICE);
						$this->context->personECLicenceRepository->delete($person->ID, $licence->ECLicenceID);
					}
				}
			}
		}
		$this->template->counter = $counter;
		$multilog->log('EC auto control check ends.', ILogger::NOTICE);
    }
}