<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter started by cron process to run person import.
 * Authentication is done by url (known token).
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PersonAutoImportPresenter extends TokenAccessPresenter
{
	/* authentication token setup */
	const AUTH_TOKEN_NAME = 'token',
		AUTH_TOKEN_KEY = 'Person.import.credentials.cronToken';
	
    /**
     * Service for Person import.
     * @var PersonImportService
     */
	private $personImportService;
	
    /**
     * Returns http parameter name with authentication token.
     * 
     * @return string
     */
	protected function getAuthTokenName()
	{
		return self::AUTH_TOKEN_NAME;
	}

    /**
     * Returns http parameter name with authentication token.
	 * 
     * @return string
     */
	protected function getAuthTokenSettingsKey()
	{
		return self::AUTH_TOKEN_KEY;
	}
	
	/**
     * Run method 
     */
    public function renderRun()
    {
		// init logging
		$logs_suffix = time().'-'. rand(1, 9);
		$current_log_file_name = 'person_import-'.$logs_suffix.'.html';
		$batch_id = 'person_import-'.$logs_suffix;
		$filelog_path = TEMP_DIR . DIRECTORY_SEPARATOR . $current_log_file_name;
        $filelog = $this->context->createFileLogger($filelog_path, $batch_id, "PERSON IMPORT") ;
		$multilog = new MultiLogger();
		$multilog->add($filelog);
		$multilog->add($this->context->createLogger($batch_id, "PERSON IMPORT"));

		//  Person import
        $this->personImportService = new PersonImportService($this->context, $multilog);
        try
        {
			switch ($this->personImportService->runPersonImport())
			{
				case PersonImportService::RESULT_SUCCESS:
					$this->flashMessage('Person import was successful.', 'success');
					break;
				case PersonImportService::RESULT_NO_DATA:
					$this->flashMessage('Person import was successful but no person to import was found.', 'warning');
					break;
				case PersonImportService::RESULT_WARNING:
					$this->flashMessage('Person import was successful with warnings.', 'warning');
					break;
				case PersonImportService::RESULT_ERROR:
					$this->flashMessage('Person import failed!', 'error');
					break;
				default: break;
			}
        }
        catch (IASAPActualsImportServiceTLCException $e)
        {
			$this->flashMessage("Person import failed: " . $e->getMessage(), 'error');
			$multilog->log('Person import failed: ' . $e->getMessage(), ILogger::ERROR);
			Debugger::log('Person import failed: ' . $e->getMessage(), Debugger::ERROR);
		}

        $filelog->close();
		
        $this->template->logfilePath    = basename(TEMP_DIR) . '/' . $current_log_file_name;
		$this->template->batchNumber	= $batch_id;
    }
}
