<?php

/**
 * This file is part of the HON Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for auto generation of tracking table.
 *
 * @author Leos Navratil <leos.navratil@honeywell.com>
 */
class PersonReportAutoGenerationPresenter extends TokenAccessPresenter
{
	/* authentication token setup */
	const AUTH_TOKEN_NAME = 'token',
		  AUTH_TOKEN_KEY = 'PersonReport.credentials.cronToken';
	
    /**
     * Returns http parameter name with authentication token.
     * 
     * @return string
     */
	protected function getAuthTokenName()
	{
		return self::AUTH_TOKEN_NAME;
	}

    /**
     * Returns http parameter name with authentication token key.
	 * 
     * @return string
     */
	protected function getAuthTokenSettingsKey()
	{
		return self::AUTH_TOKEN_KEY;
	}
	
	public function renderDefault()
	{
	}
    
	/**
     * Generate method 
     */
    public function renderGenerateReport()
    {
		$year = date('Y');
		$sbe = $this->context->organizationUnitRepository->findAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE);
		
		$multilog = new MultiLogger();
		$batch_id = 'Person-' . time() . '-' . rand(1, 9);
		$multilog->add($this->context->createLogger($batch_id, "Person auto generation"));
        $personReportService = new PersonReportService($this->context, $multilog, $year, $sbe);
		$importResult = true;
        try
        {
			$personReportService->runPersonReportGeneration();
		}
		catch (InvalidStateException $e)
		{
			$importResult = false;
            Debugger::log('Person auto generation failed: ' . $e->getMessage(), Debugger::ERROR);
			$multilog->log('Person auto generation failed: ' . $e->getMessage(), ILogger::ERROR);
		}
        catch (EPersonReportServiceException $e)
        {
			$importResult = false;
            Debugger::log('Person auto generation failed: ' . $e->getMessage(), Debugger::ERROR);
			$multilog->log('Person auto generation failed: ' . $e->getMessage(), ILogger::ERROR);
		}
		$this->template->result = $importResult;
		if ($importResult)
		{
			$multilog->log('Person auto generation completed.', ILogger::NOTICE);
		}
    }
}

