<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for exporting project charter row data.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ProjectCharterRowDataReportPresenter extends SecuredPresenter
{
	
    /**
     * Nette default render method.
     */
    public function renderDefault()
    {
        $formDefaultReport = array();
		$lock = $this->context->createLock('ProjectCharter_RowDataReport');
        $downloadReport = $this['downloadForm'];
		if($lock->locked())
		{
			$downloadReport['download']->setDisabled(TRUE);
			$this->flashMessage('Report is generated right now.', 'notice');
		}
        $downloadReport->setDefaults($formDefaultReport);
        $this->template->downloadReport = $downloadReport;
		
		$dir = null;
        if (file_exists(Environment::getConfig('projectCharterRowDataReport_storage_dir'))) 
			$dir = opendir(Environment::getConfig('projectCharterRowDataReport_storage_dir'));
		$files = array();
		//List files in directory
		if($dir != null)
		{
			while (($file = readdir($dir)) !== false)
			{
				if(preg_match('#.xlsx#', $file))
				{
					$files[$file]['file'] = $file;
					$files[$file]['modDate'] = date("Y-d-m H:i:s.", filemtime(Environment::getConfig('projectCharterRowDataReport_storage_dir') . $file));
				}
			}
			closedir($dir); 
		}
		
		$this->template->path = Environment::getConfig('projectCharterRowDataReport_storage_dir');
		$this->template->files = $files;
    }

    /**
     * Creates form for download.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentDownloadForm($name)
    {
        $form = new AppForm($this, $name);
        $years = array();
		for ($i = 2013; $i <= (date('Y')+1); $i++)
			$years[$i] = $i;
        $form->addHidden('download_token_value_id');
		$form->addSelect('Year', 'Year', $years)->setDefaultValue(date('Y')+1);
        $form->addSubmit('download', 'Regenerate Row Data Report');
        $form->onSuccess[] = array($this, 'downloadFormSubmitted');
    } 

    /**
     * Component downloadForm submit download row data report.
     * 
     * @param AppForm $form 
     */
    public function downloadFormSubmitted(AppForm $form)       
    {
        $values = $form->getValues();
		$year = $values['Year'];
		$multilog = new MultiLogger();
		$batch_id = 'PCH-RDR-'.time().'-'. rand(1, 9);
		$multilog->add($this->context->createLogger($batch_id, "Project Charter row data report generation"));
        $projectCahrterRowDataReportService = new ProjectCharterRowDataReportService($this->context, $multilog, $year, $values['download_token_value_id']);
		$importResult = true;
        try
        {
			$projectCahrterRowDataReportService->runProjectCharterRowDataReportGeneration();
		}
        catch (Exception $e)
        {
			$importResult = false;
            Debugger::log('Project Charter row data report generation failed: ' . $e->getMessage(), Debugger::ERROR);
			$multilog->log('Project Charter row data report generation failed: ' . $e->getMessage(), ILogger::ERROR);
			$this->flashMessage('Project Charter row data report generation failed: ' . $e->getMessage(), 'error');
		}
		if($importResult)
		{
			$multilog->log('Project Charter row data report generation completed.', ILogger::NOTICE);
			$this->flashMessage('Report regeneration was successful.', 'success');
		}
		$this->redirect('default');
    }
}

