<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for administration of persons.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCoverageReportPresenter extends SecuredPresenter
{    
    /** @const Key to form state */
    const SELECT_FORM_KEY = 'project_coverage_report_select_form';

    /**
     * Shows default page
     * @param int $year Selected year
     * @param int $id ID of selected organization unit
     */
    public function renderDefault($year = NULL, $id = NULL)
    {
        if ($year)
        {
            $projects = $this->context->projectRepository->findAllProjects();

            if ($id)
            {
                $selected_org_unit = $this->context->organizationUnitRepository->find($id);
                $all_org_units = $this->context->organizationUnitRepository->findAll();
                
                $cluster_leaders = array();
                foreach ($all_org_units as $org_unit)
                {
                    if (strpos($org_unit->path, $selected_org_unit->path) === 0)
                        $cluster_leaders[] = $org_unit->leader;
                }

                foreach ($projects as $project)
                {
                    if (!in_array($project->TM, $cluster_leaders))
                        unset($projects[$project->ID]);
                }
            }

            $project_reporters = array();
            foreach ($projects as $project)
            {
                if ($project->phase == Project::PROJECT_PHASE_CLOSED)
                    continue;

                $available_years = $this->context->projectRepository->findProjectYears($project->ID);
                if (in_array($year, $available_years))
                {
                    $project_reporters[] = new ProjectEACReporter($this->context, $project, $year);
                }
            }
            
            setcookie('fileDownloadToken', time());

            if ($project_reporters == NULL)
            {
                $this->flashMessage('No data to export.', 'info');
                $this->context->formStateRepository->drop(self::SELECT_FORM_KEY);
            }

            if ($this->context->formStateRepository->exists(self::SELECT_FORM_KEY))
                $this->context->formStateRepository->load(self::SELECT_FORM_KEY, $this['selectForm']);

            $this->template->projects = $project_reporters;
        }
        else
        {
            if ($this->context->formStateRepository->exists(self::SELECT_FORM_KEY))
                $this->context->formStateRepository->drop(self::SELECT_FORM_KEY);
        }
    }
    
    /**
     * Creates form for select part of organization to report.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentSelectForm($name)
    {
        $form = new AppForm();

        $years = array();
		for ($i = 2011; $i <= (date('Y')+1); $i++)
			$years[$i] = $i;
        $form->addSelect('Year', 'Year', $years)
                ->setDefaultValue(date('Y'));

        $form['OrganizationUnitSelector'] = new OrganizationUnitSelectorContainer(380, FALSE);
		$form['OrganizationUnitSelector']['SBE']->addRule(Form::FILLED, 'Please select SBE at least!');
            $form['OrganizationUnitSelector']['ajaxHandle']->setValue($this->link('OrganizationUnitSelector'));

        $form['OrganizationUnitSelector']['SBE']
                ->setPrompt('*');
        $form['OrganizationUnitSelector']['Department']
                ->setPrompt('*');
        $form['OrganizationUnitSelector']['Cluster']
                ->setPrompt('*');

        $form->addSubmit('send', 'Report');

        $form->onSuccess[] = callback($this, 'selectFormSubmitted');

        return $form;
    }
    
    /**
     * Handle for send button of selectForm
     * @param AppForm $form
     */
    public function selectFormSubmitted(AppForm $form)
    {
        $values = $form->getValues();

        $id = NULL;
        if ($values->OrganizationUnitSelector->Cluster)
            $id = $values->OrganizationUnitSelector->Cluster;
        elseif ($values->OrganizationUnitSelector->Department)
            $id = $values->OrganizationUnitSelector->Department;
        elseif ($values->OrganizationUnitSelector->SBE)
            $id = $values->OrganizationUnitSelector->SBE;

        $this->context->formStateRepository->save(self::SELECT_FORM_KEY, $form);

        $this->redirect('this', $values->Year, $id);
    }
}
