<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for Resource assignment report.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ResourceAssignmentReportPresenter extends SecuredPresenter
{
	/** @const Key to form state */
    const SELECT_FORM_KEY = 'resource_assignment_report_select_form';
    
    /**
     * Shows default page
     * @param int $year Selected year
     * @param int $id ID of selected organization unit
     */
    public function renderDefault($year = NULL, $id = NULL)
    {
        $personOutput = $finalTotal = $finalWorkableHours = array();
		$actualMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index+1;

        if ($year)
        {
            $allProjects = $this->context->projectRepository->findAllProjects();                        
            foreach ($allProjects as $project)
			{
                for ($i = 1; $i < 13; $i++)
				{
                    $finalTotal[$project->name]['values'][$i] = 0;
					$finalTotal[$project->name]['valuesAbs'][$i] = 0;
				}
				$finalTotal[$project->name]['fake'] = $project->Fake;
			}

            for ($i = 1; $i < 13; $i++)
                $finalWorkableHours[$i] = 0;
            
            $persons = $this->context->personRepository->findAll();
            $reportedPersons = array();
            if ($id)
            {
                $orgUnit = $this->context->organizationUnitRepository->find($id);
                foreach ($persons as $person)
                {
                    if ($person->OrganizationUnitID != null)
                    {
                        if (strpos($this->context->organizationUnitRepository->find($person->OrganizationUnitID)->path, $orgUnit->path) === 0)
                            $reportedPersons[$person->ID] = $person;
                    }
                }
            }
            else
            {
                $orgUnit = 0;
                $reportedPersons = $persons;
            }

			foreach ($reportedPersons as $person)
            {
				$projects = array();
                $projects = $this->context->projectRepository->findPersonAssignedAtProjectsInYear($person->ID, $year); 
				// lest find projects where persons are not assigned
				$notAssignedProjects = $this->context->projectRepository->findPersonNotAssignedAtProjectsInYear($person->ID, $year);
				foreach ($notAssignedProjects as $project)
				{
					if (!isset($projects[$project->ID]))
						$projects[$project->ID] = $project;  
				}

                $assignedAndActualHours = array();
                $assignedAndActualHoursYearTotal = $actualHoursProjectMonthTotal = array();
                for ($i = 1; $i < 13; $i++)
                    $assignedAndActualHoursYearTotal[$i] = 0;

                foreach ($projects as $project)
                {			
					
                    $personOutput[$person->ID]['projectNames'][$project->Name] = $project->Name;
					$personOutput[$person->ID]['projectNames'][$project->Name . 'Fake'] = $project->Fake;
                    $tasks = $this->context->taskRepository->findPersonTasksInYear($person->ID, $project->ID, $year);
					$activeTasks = $this->context->taskRepository->findPersonActiveTasksInYear($person->ID, $project->ID, $year);
					foreach ($activeTasks as $task)
					{
						if (!isset($tasks[$task->ID]))
						{
							$tasks[$task->ID] = $task;
						}
					}	

                    foreach ($tasks as $task)
                    {
                        $assigned = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task->ID, $year, $person->ID);
                        $actual = $this->context->workRepository->findPersonTaskYearWork($person->ID, $task->ID, $year);
                        $personOutput[$person->ID]['taskNames'][$task->ID] = $task->Name;
						$personOutput[$person->ID]['taskNames'][$task->ID . 'Fake'] = $task->Fake;
                        $personOutput[$person->ID]['taskSapID'][$task->ID] = $task->SAPID;
                        $personOutput[$person->ID]['actualHours'][$task->ID] = $actual;
                        foreach ($assigned as $month => $hour)
                        {
							if ($year == Date('Y'))
							{
								if ($month >= $actualMonth)
								{                                
									$assignedAndActualHours[$project->Name][$task->ID][$month] = $hour->Hours;
									$finalTotal[$project->Name]['values'][$month] += $hour->Hours;
									$finalTotal[$project->Name]['valuesAbs'][$month] += abs($hour->Hours);
									$assignedAndActualHoursYearTotal[$month] += $hour->Hours;
								}
								else
								{
									$assignedAndActualHours[$project->Name][$task->ID][$month] = $actual[$month];
									$finalTotal[$project->Name]['values'][$month] += $actual[$month];
									$finalTotal[$project->Name]['valuesAbs'][$month] += abs($actual[$month]);
									$assignedAndActualHoursYearTotal[$month] += $actual[$month];
								}
							}
							elseif ($year > Date('Y'))
							{
								$assignedAndActualHours[$project->Name][$task->ID][$month] = $hour->Hours;
								$finalTotal[$project->Name]['values'][$month] += $hour->Hours;
								$finalTotal[$project->Name]['valuesAbs'][$month] += abs($hour->Hours);
								$assignedAndActualHoursYearTotal[$month] += $hour->Hours;
							}
							else
							{
								$assignedAndActualHours[$project->Name][$task->ID][$month] = $actual[$month];
								$finalTotal[$project->Name]['values'][$month] += $actual[$month];
								$finalTotal[$project->Name]['valuesAbs'][$month] += abs($actual[$month]);
								$assignedAndActualHoursYearTotal[$month] += $actual[$month];
							}							
                        }
                    }
					
					$actualHoursProjectMonthTotal[$project->Name][$person->ID] = $this->context->workRepository->findPersonProjectMonthWork($person->ID, $project->ID, $year);
                }				
                /* --- Total task hours sum --- */
                $assignedAndActualHoursTaskTotal = array();

                foreach ($assignedAndActualHours as $projectName => $project)
                {
                    $assignedAndActualHoursTaskTotal[$projectName] = array();
                    for ($i = 1; $i < 13; $i++)
                    {
                        $assignedAndActualHoursTaskTotal[$projectName][$i] = 0;
                    }

                    foreach ($project as $task)
                    {
                        foreach ($task as $month => $hour)
                        {
                            $assignedAndActualHoursTaskTotal[$projectName][$month] += $hour;
                        }
                    }
                }

                $personOutput[$person->ID]['resources'] = ($assignedAndActualHours == null && $person->Status == false) ? "none" : "ok";
                $personOutput[$person->ID]['personName'] = $person->getReversedFullNameWithEID();
                $personOutput[$person->ID]['assignedAndActualHours'] = $assignedAndActualHours;
                $personOutput[$person->ID]['assignedAndActualHoursTaskTotal'] = $assignedAndActualHoursTaskTotal;
                $personOutput[$person->ID]['assignedAndActualHoursYearTotal'] = $assignedAndActualHoursYearTotal;
				$personOutput[$person->ID]['actualHoursProjectMonthTotal'] = $actualHoursProjectMonthTotal;
                $workableHours = $this->context->aopRepository->findForPerson($year, $person->ID);
                $personOutput[$person->ID]['workableHours'] = $workableHours;

                for ($i = 1; $i < 13; $i++)
                {
                    if ($personOutput[$person->ID]['resources'] == 'ok')
                        $finalWorkableHours[$i] += $workableHours->getMonthAOPHours($i);
                }
			}

            setcookie('fileDownloadToken', time());

            if ($personOutput == NULL)
            {
                $this->flashMessage('No data to export', 'info');
                $this->context->formStateRepository->drop(self::SELECT_FORM_KEY);
            }

            if ($this->context->formStateRepository->exists(self::SELECT_FORM_KEY))
                $this->context->formStateRepository->load(self::SELECT_FORM_KEY, $this['selectForm']);


            $this->template->personOutput = $personOutput;
            $this->template->finalTotal = $finalTotal;
            $this->template->finalWorkableHours = $finalWorkableHours;
            $this->template->year = $year;
            $this->template->currentYear = date('Y');
            $this->template->currentMonth = $actualMonth;
        }
        else
        {
            if ($this->context->formStateRepository->exists(self::SELECT_FORM_KEY))
                $this->context->formStateRepository->drop(self::SELECT_FORM_KEY);
        }
    }

    /**
     * Creates form for select part of organization to report.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    protected function createComponentSelectForm($name)
    {
        $form = new AppForm();

        $years = array();
        for ($i = 2011; $i <= (date('Y')+1); $i++)
            $years[$i] = $i;
        $form->addSelect('Year', 'Year', $years)
				->setRequired('Please select the year!')
                ->setDefaultValue(date('Y'));        

        $form['OrganizationUnitSelector'] = new OrganizationUnitSelectorContainer(380, FALSE);
		$form['OrganizationUnitSelector']['SBE']->addRule(Form::FILLED, 'Please select SBE at least!');
            $form['OrganizationUnitSelector']['ajaxHandle']->setValue($this->link('OrganizationUnitSelector'));

        $form['OrganizationUnitSelector']['SBE']
                ->setPrompt('*');
        $form['OrganizationUnitSelector']['Department']
                ->setPrompt('*');
        $form['OrganizationUnitSelector']['Cluster']
                ->setPrompt('*');
        $form['OrganizationUnitSelector']['TechnicalGroup']
                ->setPrompt('*');
		$form->addHidden('IE');
        $form->addButton('toggle', 'Toggle All');
        $form->addSubmit('send', 'Report Here');
		$form->addSubmit('reportToExcel', 'Report to Excel');

        $form->onSuccess[] = callback($this, 'selectFormSubmitted');
        
        return $form;
    }
    
    /**
     * Handle for send button of selectForm
     * @param AppForm $form
     */
    public function selectFormSubmitted(AppForm $form)
    {
        $values = $form->getValues();

		if ($form->isValid())
		{
			$orgUnitID = NULL;
			if ($values->OrganizationUnitSelector->TechnicalGroup)
				$orgUnitID = $values->OrganizationUnitSelector->TechnicalGroup;
			elseif ($values->OrganizationUnitSelector->Cluster)
				$orgUnitID = $values->OrganizationUnitSelector->Cluster;
			elseif ($values->OrganizationUnitSelector->Department)
				$orgUnitID = $values->OrganizationUnitSelector->Department;
			elseif ($values->OrganizationUnitSelector->SBE)
				$orgUnitID = $values->OrganizationUnitSelector->SBE;

			$this->context->formStateRepository->save(self::SELECT_FORM_KEY, $form);

			if($values['IE'] == 'send')
			{
				$this->redirect('this', $values->Year, $orgUnitID);
			}
			elseif($values['IE'] == 'reportToExcel')
			{
				try
				{
					// MS Excel file as output
					$objPHPExcel = new PHPExcel();
					$this->generateReportToExcel($values->Year, $orgUnitID, $objPHPExcel);
					$this->redirect('default');
				}
				catch (IOException $e)
				{
					$this->flashMessage('Error while generating excel report.', 'error');
					$this->redirect('default');
				}
			}
		}
		else
		{
			$this->flashMessage('Invalid input. Report was not generated!', 'error');
			$this->redirect('this');
		}
    }
	
     /**
     * Generates resouce assignment/actual report as a XLSX (MS-EXCEL file)
	 * 
     * @param int $year Year
     * @param int $orgUnitID ID of reported org unit
     * @param PHHPExcel $objPHPExcel Output file.
	 * @throws IOException if there is problem with opening output file.
     */
    public function generateReportToExcel($year, $orgUnitID, $objPHPExcel)
    {
		if(is_null($year))
		{
			throw new InvalidArgumentException;
		}
		//increase executon time limit
        $exec_time = 600;
	    if (!ini_set("max_execution_time", $exec_time)) 
		{
			$this->flashMessage('Setup of maximum execution time failed. Contact administrator.', 'error');
			trigger_error('Unable to setup maximum execution time to '.$exec_time.'.', E_USER_ERROR);
		}

        $sheet = $objPHPExcel->getActiveSheet();
		$sheet->setTitle('Report');
		// Formatting settings
        //colors
		$bluePerson = '91B4D7';
        $blueTask = 'DBE5F1';
        $blueProject = 'B8CCE4';
        $grayTotal = 'F2F2F2';
        $graySum = 'D8D8D8';
		$yellowPlaceholderTask = 'FFF9D7';
		$brownPlanningTask = 'B6A5C2';
		$yellowPlaceholderProject = 'FFF08C';
        $borderPerson = array(
        	'borders' => array(
                'right' => array(
                    'style' => PHPExcel_Style_Border::BORDER_THIN,
                    'color' => array('rgb' => '000000')
                ),
				'left' => array(
                    'style' => PHPExcel_Style_Border::BORDER_THIN,
                    'color' => array('rgb' => '000000')
                ),
				'top' => array(
                    'style' => PHPExcel_Style_Border::BORDER_THIN,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderProjectBotton = array(
        	'borders' => array(
					'top' => array(
						'style' => PHPExcel_Style_Border::BORDER_THIN,
						'color' => array('rgb' => '000000')
                ),
            ),
        );
		$borderProject = array(
        	'borders' => array(
                'right' => array(
                    'style' => PHPExcel_Style_Border::BORDER_THIN,
                    'color' => array('rgb' => '000000')
                ),
				'left' => array(
                    'style' => PHPExcel_Style_Border::BORDER_THIN,
                    'color' => array('rgb' => '000000')
                ),
				'top' => array(
                    'style' => PHPExcel_Style_Border::BORDER_THIN,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
		$warningColorRed = 'E6B8B8';
		$warningColorYellow = 'FFFF00';
		$warningColorGreen = 'D8E4BC';
		//set conditional formatting for resource coverage
		$objConditional1Less = new PHPExcel_Style_Conditional();
		$objConditional1Less->setConditionType(PHPExcel_Style_Conditional::CONDITION_CELLIS);
		$objConditional1Less->setOperatorType(PHPExcel_Style_Conditional::OPERATOR_LESSTHAN);
		$objConditional1Less->addCondition(50);
		$objConditional1Less->getStyle()->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getEndColor()->setARGB($warningColorRed);
		
		$objConditional1Greater = new PHPExcel_Style_Conditional();
		$objConditional1Greater->setConditionType(PHPExcel_Style_Conditional::CONDITION_CELLIS);
		$objConditional1Greater->setOperatorType(PHPExcel_Style_Conditional::OPERATOR_GREATERTHAN);
		$objConditional1Greater->addCondition(150);
		$objConditional1Greater->getStyle()->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getEndColor()->setARGB($warningColorRed);
		$objConditionalStyles = array();
		
		$objConditional2Less = new PHPExcel_Style_Conditional();
		$objConditional2Less->setConditionType(PHPExcel_Style_Conditional::CONDITION_CELLIS);
		$objConditional2Less->setOperatorType(PHPExcel_Style_Conditional::OPERATOR_LESSTHAN);
		$objConditional2Less->addCondition(85);
		$objConditional2Less->getStyle()->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getEndColor()->setARGB($warningColorYellow);
		
		$objConditional2Greater = new PHPExcel_Style_Conditional();
		$objConditional2Greater->setConditionType(PHPExcel_Style_Conditional::CONDITION_CELLIS);
		$objConditional2Greater->setOperatorType(PHPExcel_Style_Conditional::OPERATOR_GREATERTHAN);
		$objConditional2Greater->addCondition(115);
		$objConditional2Greater->getStyle()->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getEndColor()->setARGB($warningColorYellow);
		
		$objConditional3Less = new PHPExcel_Style_Conditional();
		$objConditional3Less->setConditionType(PHPExcel_Style_Conditional::CONDITION_CELLIS);
		$objConditional3Less->setOperatorType(PHPExcel_Style_Conditional::OPERATOR_LESSTHANOREQUAL);
		$objConditional3Less->addCondition(100);
		$objConditional3Less->getStyle()->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getEndColor()->setARGB($warningColorGreen);
		
		$objConditional3Greater = new PHPExcel_Style_Conditional();
		$objConditional3Greater->setConditionType(PHPExcel_Style_Conditional::CONDITION_CELLIS);
		$objConditional3Greater->setOperatorType(PHPExcel_Style_Conditional::OPERATOR_GREATERTHANOREQUAL);
		$objConditional3Greater->addCondition(100);
		$objConditional3Greater->getStyle()->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getEndColor()->setARGB($warningColorGreen);
		
		array_push($objConditionalStyles, $objConditional1Greater);
		array_push($objConditionalStyles, $objConditional1Less);
		array_push($objConditionalStyles, $objConditional2Greater);
		array_push($objConditionalStyles, $objConditional2Less);
		array_push($objConditionalStyles, $objConditional3Greater);
		array_push($objConditionalStyles, $objConditional3Less);
		
		$sheet->getColumnDimension('A')->setWidth(40);
        $rowIterator = $sheet->getRowIterator();
        $cellIterator = $rowIterator->current()->getCellIterator();

		// Prepare basic data
		$actualMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index+1;
		// get all presons
		$persons = $this->context->personRepository->findAll();
		$reportedPersons = $reportedProjects = $workableHoursSum = array();
		if ($orgUnitID)
		{
			$orgUnit = $this->context->organizationUnitRepository->find($orgUnitID);
			// get person only from selected Org Unit
			foreach ($persons as $person)
			{
				if ($person->OrganizationUnitID != null)
				{
					if (strpos($this->context->organizationUnitRepository->find($person->OrganizationUnitID)->path, $orgUnit->path) === 0)
						$reportedPersons[$person->ID] = $person;
				}
			}
		}
		else
		{
			$orgUnit = 0;
			$reportedPersons = $persons;
		}

		//LEGEND
		$cellIterator->current()->setValue('Legend:');
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('Person name (EID)');
		$sheet->getRowDimension($cellIterator->current()->getRow())->setOutlineLevel(1);
		$sheet->getRowDimension($cellIterator->current()->getRow())->setVisible(false);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($bluePerson);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('Real project name');
		$sheet->getRowDimension($cellIterator->current()->getRow())->setOutlineLevel(1);
		$sheet->getRowDimension($cellIterator->current()->getRow())->setVisible(false);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($blueProject);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('Real task name');
		$sheet->getRowDimension($cellIterator->current()->getRow())->setOutlineLevel(1);
		$sheet->getRowDimension($cellIterator->current()->getRow())->setVisible(false);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($blueTask);
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('Placeholder project name');
		$sheet->getRowDimension($cellIterator->current()->getRow())->setOutlineLevel(1);
		$sheet->getRowDimension($cellIterator->current()->getRow())->setVisible(false);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($yellowPlaceholderProject);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('Placeholder task name');
		$sheet->getRowDimension($cellIterator->current()->getRow())->setOutlineLevel(1);
		$sheet->getRowDimension($cellIterator->current()->getRow())->setVisible(false);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($yellowPlaceholderTask);
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('Planning task name');
		$sheet->getRowDimension($cellIterator->current()->getRow())->setOutlineLevel(1);
		$sheet->getRowDimension($cellIterator->current()->getRow())->setVisible(false);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($brownPlanningTask);
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($grayTotal);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$sheet->getRowDimension($cellIterator->current()->getRow())->setOutlineLevel(1);
		$sheet->getRowDimension($cellIterator->current()->getRow())->setVisible(false);
		$cellIterator->current()->setValue('Totals');
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('Actuals');
		$sheet->getRowDimension($cellIterator->current()->getRow())->setOutlineLevel(1);
		$sheet->getRowDimension($cellIterator->current()->getRow())->setVisible(false);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('Assignment');
		$sheet->getRowDimension($cellIterator->current()->getRow())->setOutlineLevel(1);
		$sheet->getRowDimension($cellIterator->current()->getRow())->setVisible(false);
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('Resource coverage:');
		$sheet->getRowDimension($cellIterator->current()->getRow())->setOutlineLevel(1);
		$sheet->getRowDimension($cellIterator->current()->getRow())->setVisible(false);
		$cellIterator->next();
		$cellIterator->current()->setValue('bellow 50%');
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($warningColorRed);
		$cellIterator->next();
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($warningColorYellow);
		$cellIterator->next();
		$cellIterator->current()->setValue('85% - 115%');
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($warningColorGreen);
		$cellIterator->next();
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($warningColorYellow);
		$cellIterator->next();
		$cellIterator->current()->setValue('above 150%');
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($warningColorRed);
		$rowIterator->next();
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Report data for every person
		foreach ($reportedPersons as $person)
		{
			//Person Header
			$cellIterator->current()->setValue($person->getReversedFullNameWithEID());
			//HARDCODED for person Header
			$row = $cellIterator->current()->getRow();
			$sheet->mergeCells('A' . $row . ':N' . $row);
			$sheet->getStyle('A' . $row)->getFont()->setBold(true);
			$sheet->getStyle('A' . $row . ':N' . $row)->applyFromArray($borderPerson);
			$sheet->getStyle('A' . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($bluePerson);
			$sheet->getStyle('A'. $row)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
			
			$rowIterator->next();
			$cellIterator = $rowIterator->current()->getCellIterator();
			
			$projects = $personTotalValueRows = array();
			$workableHours = $this->context->aopRepository->findForPerson($year, $person->ID);
			// get person's project with assignemnt
			$projects = $this->context->projectRepository->findPersonAssignedAtProjectsInYear($person->ID, $year);
			// lest find projects where persons are not assigned, but has some actuals
			$notAssignedProjects = $this->context->projectRepository->findPersonNotAssignedAtProjectsInYear($person->ID, $year);
			foreach ($notAssignedProjects as $project)
			{
				if (!isset($projects[$project->ID]))
					$projects[$project->ID] = $project;
			}

			// Report data for every project
			foreach ($projects as $project)
			{
				$reportedProjects[$project->Name]['Fake'] = $project->Fake;
				$row = $cellIterator->current()->getRow();
				$sheet->getStyle('A' . $row. ':N' . $row)->getFont()->setBold(true);
				$sheet->getStyle('A' . $row . ':N' . $row)->applyFromArray($borderProject);
				$sheet->getStyle('A' . ($row + 1) . ':N' . ($row + 1))->applyFromArray($borderProjectBotton);
				$sheet->getRowDimension($row)->setOutlineLevel(1);
				$sheet->getRowDimension($row)->setVisible(false);
				if($project->Fake)
					$sheet->getStyle('A' . $row . ':N' . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($yellowPlaceholderProject);
				else
					$sheet->getStyle('A' . $row . ':N' . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($blueProject);
				$sheet->getStyle('A'. $row . ':N' . $row)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				// Project Header
				$cellIterator->current()->setValue($project->Name);
				$cellIterator->next();
				for($i = 0; $i < 12; $i++)
				{
					$cellIterator->current()->setValue(FiscalCalendar::fiscalMonthIndexToName($i, false));
					if($i < ($actualMonth - 1))
						$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
					$cellIterator->next();
				}
				$cellIterator->current()->setValue('Total');
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
				
				// Get all task
				$tasks = $this->context->taskRepository->findPersonTasksInYear($person->ID, $project->ID, $year);
				$activeTasks = $this->context->taskRepository->findPersonActiveTasksInYear($person->ID, $project->ID, $year);
				foreach ($activeTasks as $task)
				{
					if (!isset($tasks[$task->ID]))
					{
						$tasks[$task->ID] = $task;
					}
				}	
				// Report data for every task
				$projectTotalFirstValueRow = $cellIterator->current()->getRow();
				foreach ($tasks as $task)
				{
					// Get all task's data
					$assigned = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task->ID, $year, $person->ID);
					$actual = $this->context->workRepository->findPersonTaskYearWork($person->ID, $task->ID, $year);
					
					// Task cells
					$cellIterator->current()->setValue($task->Name);
					if($task->Fake == Task::TASK_TYPE_PLACEHOLDER)
						$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($yellowPlaceholderTask);
					elseif($task->Fake == Task::TASK_TYPE_SAP)
						$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($blueTask);
					else
						$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($brownPlanningTask);
					$row = $cellIterator->current()->getRow();
					$sheet->getRowDimension($row)->setOutlineLevel(1);
					$sheet->getRowDimension($row)->setVisible(false);
					$cellIterator->next();
					$taskTotalFirstValueCol = $projectTotalFirstValueCol = $personTotalFirstValueCol = $cellIterator->current()->getColumn();
					for($i = 1; $i < 13; $i++)
					{
						if($year == Date('Y'))
						{
							//assigned hours
							if($i >= $actualMonth)
							{
								$cellIterator->current()->setValue($assigned[$i]->Hours);
							}
							//actuals hours
							else
							{
								$cellIterator->current()->setValue($actual[$i]);
								$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
							}
						}
						elseif ($year > Date('Y'))
						{
							$cellIterator->current()->setValue($assigned[$i]->Hours);
						}
						else
						{
							$cellIterator->current()->setValue($actual[$i]);
							$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
						}
						$cellIterator->next();
					}
					$taskTotalLastValueCol = chr(ord($cellIterator->current()->getColumn())-1);
					$taskTotalValueRow = $cellIterator->current()->getRow();
					$cellIterator->current()->setValue('=SUM(' . $taskTotalFirstValueCol . $taskTotalValueRow . ':' . $taskTotalLastValueCol . $taskTotalValueRow . ')');
					$projectTotalLastValueCol = $personTotalLastValueCol = $cellIterator->current()->getColumn();
					$projectTotalLastValueRow = $cellIterator->current()->getRow();
					$rowIterator->next();
					$cellIterator = $rowIterator->current()->getCellIterator();
				}
				$cellIterator->current()->setValue('Total');
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($grayTotal);
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$row = $cellIterator->current()->getRow();
				$sheet->getRowDimension($row)->setOutlineLevel(1);
				$sheet->getRowDimension($row)->setVisible(false);
				$cellIterator->next();
				$reportedProjects[$project->Name]['Data'][] = $cellIterator->current()->getRow();
				$actualMonthTotal = $actualMonth + ord($projectTotalFirstValueCol) - 1;
				for($i = ord($projectTotalFirstValueCol); $i <= ord($projectTotalLastValueCol); $i++)
				{
					$cellIterator->current()->setValue('=SUM(' . chr($i) . $projectTotalFirstValueRow . ':' . chr($i) . $projectTotalLastValueRow . ')');
					//$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($grayTotal);
					$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
					if($i < $actualMonthTotal)
						$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
					$cellIterator->next();
				}
				$personTotalValueRows[] = $cellIterator->current()->getRow();
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
			}
			if(!empty($projects) && $person->Status)
			{
				// Person total sum
				$cellIterator->current()->setValue('SUM');
				$row = $cellIterator->current()->getRow();
				$sheet->mergeCells('A' . $row . ':N' . $row);
				$sheet->getStyle('A' . $row. ':N' . $row)->getFont()->setBold(true);
				$sheet->getStyle('A' . $row . ':N' . $row)->applyFromArray($borderProject);
				$sheet->getStyle('A' . ($row + 1) . ':N' . ($row + 1))->applyFromArray($borderProjectBotton);
				$sheet->getStyle('A' . $row . ':N' . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($graySum);
				$sheet->getStyle('A'. $row . ':N' . $row)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
				$row = $cellIterator->current()->getRow();
				$sheet->getStyle('A' . $row. ':N' . $row)->getFont()->setBold(true);
				$sheet->getStyle('A' . $row . ':N' . $row)->applyFromArray($borderProject);
				$sheet->getStyle('A' . ($row + 1) . ':N' . ($row + 1))->applyFromArray($borderProjectBotton);
				$sheet->getStyle('A' . $row . ':N' . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($blueTask);
				$sheet->getStyle('A'. $row . ':N' . $row)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$cellIterator->next();
				for($i = 0; $i < 12; $i++)
				{
					$cellIterator->current()->setValue(FiscalCalendar::fiscalMonthIndexToName($i, false));
					if($i < ($actualMonth - 1))
						$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
					$cellIterator->next();
				}
				$cellIterator->current()->setValue('Total');
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
				$cellIterator->current()->setValue('ACTUALS/ASSIGNED IN TOTAL');
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($grayTotal);
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$cellIterator->next();
				for($i = ord($personTotalFirstValueCol); $i <= ord($personTotalLastValueCol); $i++)
				{
					$personTotalSumRows = array();
					foreach($personTotalValueRows as $rowNumber)
					{
						$personTotalSumRows[] = chr($i) . $rowNumber;
					}
					if(!empty($personTotalValueRows))
						$cellIterator->current()->setValue('=' . join('+', $personTotalSumRows));
					else
						$cellIterator->current()->setValue(0);
					if($i < $actualMonthTotal)
						$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
					$cellIterator->next();
				}
				$rowIterator->next();
				// Worable hours
				$cellIterator = $rowIterator->current()->getCellIterator();
				$cellIterator->current()->setValue('Workable hours');
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($grayTotal);
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$cellIterator->next();
				$x = 1;
				for($i = ord($personTotalFirstValueCol); $i < ord($personTotalLastValueCol); $i++)
				{
					$cellIterator->current()->setValue($workableHours->getMonthAOPHours($x));
					if($i < $actualMonthTotal)
						$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
					$cellIterator->next();
					$x++;
				}
				$workableHoursSum[] = $cellIterator->current()->getRow();
				$row = $cellIterator->current()->getRow();
				$cellIterator->current()->setValue('=SUM(' . $personTotalFirstValueCol . $row . ':' . chr(ord($personTotalLastValueCol)-1) . $row . ')');
				$rowIterator->next();
				// Resouce coverage
				$cellIterator = $rowIterator->current()->getCellIterator();
				$cellIterator->current()->setValue('Resource coverage');
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($grayTotal);
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$cellIterator->next();
				for($i = ord($personTotalFirstValueCol); $i <= ord($personTotalLastValueCol); $i++)
				{
					$row = $cellIterator->current()->getRow();
					$cellIterator->current()->setValue('=ROUND((' . chr($i) . ($row - 2) . '/' . chr($i) . ($row - 1) . ')*100,0)');
					$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->setConditionalStyles($objConditionalStyles);
					$cellIterator->next();
				}
			}
			else
			{
				$cellIterator->current()->setValue('No data to Export');
				$row = $cellIterator->current()->getRow();
				$sheet->mergeCells('A' . $row . ':N' . $row);
				$sheet->getStyle('A' . $row . ':N' . $row)->applyFromArray($borderProjectBotton);
				$sheet->getStyle('A'. $row)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$cellIterator->next();
			}
			$rowIterator->next();
			$cellIterator = $rowIterator->current()->getCellIterator();
			// Empty row
			$rowIterator->next();
			$cellIterator = $rowIterator->current()->getCellIterator();
        }
		// Final projects sum
		$cellIterator->current()->setValue('FINAL SUM');
		$row = $cellIterator->current()->getRow();
		$sheet->mergeCells('A' . $row . ':N' . $row);
		$sheet->getStyle('A' . $row. ':N' . $row)->getFont()->setBold(true);
		$sheet->getStyle('A' . $row . ':N' . $row)->applyFromArray($borderProject);
		$sheet->getStyle('A' . ($row + 1) . ':N' . ($row + 1))->applyFromArray($borderProjectBotton);
		$sheet->getStyle('A' . $row . ':N' . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($graySum);
		$sheet->getStyle('A'. $row . ':N' . $row)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$row = $cellIterator->current()->getRow();
		$sheet->getStyle('A' . $row. ':N' . $row)->getFont()->setBold(true);
		$sheet->getStyle('A' . $row . ':N' . $row)->applyFromArray($borderProject);
		$sheet->getStyle('A' . ($row + 1) . ':N' . ($row + 1))->applyFromArray($borderProjectBotton);
		$sheet->getStyle('A' . $row . ':N' . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($blueProject);
		$sheet->getStyle('A'. $row . ':N' . $row)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$cellIterator->next();
		for($i = 0; $i < 12; $i++)
		{
			$cellIterator->current()->setValue(FiscalCalendar::fiscalMonthIndexToName($i, false));
			if($i < ($actualMonth - 1))
						$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
			$cellIterator->next();
		}
		$cellIterator->current()->setValue('Total');
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$firstProjectTotalRow = $cellIterator->current()->getRow();
		foreach($reportedProjects as $projectname => $projectRowsArray)
		{
			$cellIterator->current()->setValue($projectname);
			if($projectRowsArray['Fake'])
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($yellowPlaceholderTask);
			else
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($blueTask);
			$cellIterator->next();
			for($i = ord($personTotalFirstValueCol); $i <= ord($personTotalLastValueCol); $i++)
			{
				$projectColsAndRows = array();
				foreach($projectRowsArray['Data'] as $rowNumber)
				{
					$projectColsAndRows[$projectname][] = chr($i) . $rowNumber;
				}
				$row = $cellIterator->current()->getRow();
				$cellIterator->current()->setValue('=' . join('+', $projectColsAndRows[$projectname]));
				if($i < $actualMonthTotal)
					$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
				$cellIterator->next();
			}
			$rowIterator->next();
			$cellIterator = $rowIterator->current()->getCellIterator();
		}
		$row = $cellIterator->current()->getRow();
		$sheet->getStyle('A' . $row. ':N' . $row)->getFont()->setBold(true);
		$sheet->getStyle('A' . $row . ':N' . $row)->applyFromArray($borderProject);
		$sheet->getStyle('A' . ($row + 1) . ':N' . ($row + 1))->applyFromArray($borderProjectBotton);
		$sheet->getStyle('A' . $row . ':N' . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($bluePerson);
		$sheet->getStyle('A'. $row . ':N' . $row)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$cellIterator->next();
		for($i = 0; $i < 12; $i++)
		{
			$cellIterator->current()->setValue(FiscalCalendar::fiscalMonthIndexToName($i, false));
			if($i < ($actualMonth - 1))
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
			$cellIterator->next();
		}
		$cellIterator->current()->setValue('Total');
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// ACTUALS/ASSIGNED IN TOTAL of all projects
		$lastProjectTotalRow = ($cellIterator->current()->getRow() - 2);
		$cellIterator->current()->setValue('ACTUALS/ASSIGNED IN TOTAL');
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($grayTotal);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$cellIterator->next();
		for($i = ord($personTotalFirstValueCol); $i <= ord($personTotalLastValueCol); $i++)
		{
			$cellIterator->current()->setValue('=SUM(' . chr($i) . $firstProjectTotalRow . ':' . chr($i) . $lastProjectTotalRow . ')');
			if($i < $actualMonthTotal)
				$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
			$cellIterator->next();
		}
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		
		// Workable hours of all persons
		$cellIterator->current()->setValue('Workable Hours');
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($grayTotal);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$cellIterator->next();
		for($i = ord($personTotalFirstValueCol); $i <= ord($personTotalLastValueCol); $i++)
		{
			$workableHoursRows = array();
			foreach($workableHoursSum as $rowNumber)
			{
				$workableHoursRows[] = chr($i) . $rowNumber;
			}
			$cellIterator->current()->setValue('=' . join('+', $workableHoursRows));
			if($i < $actualMonthTotal)
					$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setItalic(true);
			$cellIterator->next();
		}
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		
		//Resource coverage
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('Resource coverage');
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($grayTotal);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$cellIterator->next();
		for($i = ord($personTotalFirstValueCol); $i <= ord($personTotalLastValueCol); $i++)
		{
			$row = $cellIterator->current()->getRow();
			$cellIterator->current()->setValue('=ROUND((' . chr($i) . ($row - 2) . '/' . chr($i) . ($row - 1) . ')*100,0)');
			$sheet->getStyle($cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->setConditionalStyles($objConditionalStyles);
			$cellIterator->next();
		}
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		
        setcookie('fileDownloadToken', time());
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment; filename=ResourceAssignmentActualsReport.xlsx');
        header('Cache-Control: max-age=0');
        $objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
		ob_end_clean();
        $objWriter->save("php://output");
		exit;
    }
}