<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for administration of Tasks and TaskPlans.
 * 
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class TaskPlanPresenter extends SecuredPresenter
{
    /** @var session */
	private $session;
	/** @persistent */
    public $year;
	/** @persistent */
	public $prevPresenter = '';

    /**
	 * Initialization
	 */
	public function startup()
	{
		parent::startup();
		
		$this->session = $this->getSession('TaskPlan');
        $this->session->setExpiration(0);
        
        $this->setYear($this->year);
	}
	
	public function shutdown($response)
	{
		parent::shutdown($response);
		
		$taskPlanID = (int)$this->getParameter('id');
		$taskPlanYear = (int)$this->getParameter('year');
		$taskPlanAction = $this->getParameter('action');
		
		$this->prevPresenter = 'taskPlan' . '+' . $taskPlanID . '+' . $taskPlanYear . '+' . $taskPlanAction;
	}

	/**
     * Global setting of templates
     */
    public function beforeRender()
    {
        parent::beforeRender();
        
        $this->template->year = $this->year;
    }

    /**
     * Nette default render method.
     * Prepares and stores data in $this->template for use in [view].latte
     */
    public function renderDefault()
    {
    }

	/**
	 * Nette labor render method.
	 * @param int $id labor task id.
	 * @param int $year Year.
	 */
	public function renderLabor($id = NULL, $year = NULL)
	{
		// get task
		$task = $this->context->taskRepository->find($id);
		if(!$task)
		{
			$this->flashMessage('Task with ID = ' . $id . ' not found.', 'notice');
			$this->redirect('Project:default');
		}

        if ($year && is_numeric($year))
			$this->year = $year;

		$project = $this->context->projectRepository->findProjectByTaskId($task->ID);
		
		//get TaskPlan
		$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year);
		if($task_plan === NULL)
		{
			/* If there is nobody with actual(s) for this SAP_ID and year,
			 * redirect. Otherwise create an empty TaskPlan.
			 */
			$task_plan = $this->createNewTaskPlanOrRedirect($task, $this->year);
		}
		//get TaskPlanner
		$taskPlanner = new TaskResourceYearPlanner($this->context, $task_plan);
		$first_updateable_month = FiscalCalendar::currentFiscalMonth() + 1;
		
		// get task plan editing control object
		$taskPlanEditing = new TaskPlanEditing($task, $year);
		$this->template->taskPlanEditing = $taskPlanEditing;
		
		/* --- Task section --- */// start
		$remarkForm = $this['remarkForm'];
		$poNumber = $this->context->taskPONumberRepository->findByTaskIDandYear($task->ID, $this->year);
		$poNumberID = null;
		if(!$poNumber)
		{
			$remarkForm['poNumber']->setAttribute('placeholder', 'Undefined');
		}
		else
		{
			$poNumberID = $poNumber->ID;
			$poNumber = $poNumber->PONumber;
		}

		$remarkFormDefaults = array(
			'task_id' => $task->ID,
			'poNumber_id' => $poNumberID,
			'remark' => $task->Remark,
			'year' => $this->year,
			'poNumber' => $poNumber
		);
		$remarkForm->setDefaults($remarkFormDefaults);
		$this->template->remarkForm = $remarkForm;

		//Task assignment lock form
		$taskLockForm = $this['taskLockForm'];
		$taskLockForm['save']->setDisabled(!$this->isAllowed(self::RESOURCE_TASK_ASSIGNMENT_LOCK, self::ACTION_EDIT));
		if($task->Locked)
			$taskLockFormDefaults = array('AssignmentLock' => 1);
		else
			$taskLockFormDefaults = array('AssignmentLock' => 0);
		$taskLockFormDefaults['task_id'] = $task->ID;
		$taskLockForm->setDefaults($taskLockFormDefaults);
		$this->template->taskLockForm = $taskLockForm;
		
		//Task assignment transfer form
		$taskAssignmentTransferForm = $this['taskAssignmentTransferForm'];
		$this->template->taskAssignmentTransferAllowed = $this->isAllowed(self::RESOURCE_TASK_ASSIGNMENT_TRANSFER, self::ACTION_EDIT);
		$this->template->taskAssignmentTransferForm = $taskAssignmentTransferForm;

		$taskInfo = array();
		$taskInfo['name'] = $task->Name;
		$taskInfo['workPackageStructure'] = '';
		foreach ($this->context->workPackageRepository->getWPTreePathByTaskID($task->ID) as $workPackage)
			$taskInfo['workPackageStructure'][] = $workPackage->Name;
		$taskInfo['workPackageStructure'] = implode(' » ', $taskInfo['workPackageStructure']);
		$taskInfo['costShare'] = $task->CostSharePercentage;
		$this->template->taskInfo = $taskInfo;
		/* --- Task section --- */// end

		/* --- SAP section --- */
		$this->template->sapInfo = array();
		$this->template->sapInfo['sapID'] = $task->SAPID;
		if($task->CostCenterID)
			$this->template->sapInfo['costCenter'] = $this->context->costCenterRepository->findCostCenter($task->CostCenterID)->Name . ' (' . $task->CostCenterID . ')';
		else
			$this->template->sapInfo['costCenter'] = 'Unknown';
		$this->template->sapInfo['effort'] = 0;
		foreach ($this->context->taskPlanRepository->findTaskPlans($task->ID) as $task_plan)
		{
			$task_planner = new TaskResourceYearPlanner($this->context, $task_plan);
			$this->template->sapInfo['effort'] += $task_planner->getTaskYearPlanHours()->sum();
		}
		/* --- SAP section --- END*/

		/* --- Current res section ---*/
		// get planned years            
		$task_plan_years = $actuals_years = array();                
		$task_plan_years = $this->context->taskPlanRepository->findTaskYears($task->ID);
		$actuals_years = $this->context->actualsRepository->findTaskYears($task->SAPID);

		$availableYears = array_unique(array_merge($task_plan_years, $actuals_years));

		$this->template->AvailableYears = $availableYears;
		if (!in_array($this->year, $availableYears))
		{
			$alternative_year = 0;
			if (in_array($this->year + 1, $availableYears))
				$alternative_year = $this->year + 1;
			elseif (in_array($this->year - 1, $availableYears))
				$alternative_year = $this->year - 1;
			elseif (count($availableYears))
				$alternative_year = $availableYears[0]; // first year of project
			if ($alternative_year != 0)
			{
				$this->flashMessage("There is no plan for current year $this->year. Year $alternative_year selected.", 'info');
				$this->redirect('labor', 
					array(	'id' => $task->ID, 
							'year' => $alternative_year
						));
			}
			else
			{
				$this->flashMessage("There is no plan for task: " . $task->Name . ".", 'info');
				$this->redirect('Project:detail', $project->ID, $this->year);
			}
		}

		//get total resources
		$resourcesInfo = array();
		$resourcesInfo['sapHours'] = $taskPlanner->getTaskYearPlanHours();
		$resourcesInfo['eacHours'] = $taskPlanner->getTaskYearEACHours();
		$this->template->resourcesInfo = $resourcesInfo;

		// get assigned workers
		$assignedResources = array();
		$assignedWorkers = $this->context->personRepository->findAllPersonsAssignedToTaskInYear($task->ID, $this->year);
		$anyLockedWorker = false;
		$lockedWorkerList = array();
		foreach($assignedWorkers as $person)
		{
			// Person objet
			$assignedResources[$person->ID]['person'] = $person;
			// workable hours
			$assignedResources[$person->ID]['workableHours'] = $this->context->aopRepository->findForPerson($this->year, $person->ID);
			// available hours
			// total assigned hours for person
			$total_assignments = $this->context->personTaskAssignmentRepository->findPersonTotalYearAssignments($person->ID, $this->year);
			// initialize array
			$aop_available = new MonthArray((float) 0.0, false);
			// delta => available
			for ($i = 1; $i <= 12; $i++)
			{
				//if($first_updateable_month <= $i && $this->year >= Date('Y')) // past months
					$aop_available[$i] = $assignedResources[$person->ID]['workableHours']->getMonthAOPHours($i) - $total_assignments[$i];
			}
			$assignedResources[$person->ID]['available'] = $aop_available;
			// gets effort spent in current month
			$assignedResources[$person->ID]['actualMonthHours'] = $this->context->workRepository->findPersonTaskMonthWork($person->ID, $task->ID, $first_updateable_month - 1, $this->year);
			// Info message about why cant be person assigned
			$assignmentControl = new AssignmentControl($person, $task);
			$assignedResources[$person->ID]['assignableControl'] = $assignmentControl;
			if($person->Locked)
			{
				$anyLockedWorker = true;
				$lockedWorkerList[] = $person->getReversedFullNameWithEID();
			}
		}
		$this->template->assignedResources = $assignedResources;
		// current resources form
		$curResForm = $this['currentResourcesForm'];
		$curResForm['discard']->setDisabled(!$taskPlanEditing->editable);
		$curResForm['save']->setDisabled(!$taskPlanEditing->editable);
		if(!$taskPlanEditing->editable)
			$curResForm['save']->setAttribute('data-form-button-tooltip', $taskPlanEditing->Message);
		$curResForm['delete']->setDisabled(!$taskPlanEditing->editable);
		if($anyLockedWorker && $this->isAllowed(self::RESOURCE_RESOURCE_LOCK, self::ACTION_RUN))
			$curResForm['save']->setAttribute('onclick', '
				return confirm("There is/are locked worker(s): '. implode(', ', $lockedWorkerList) . '. Are you sure to update task assignments?");
				');
		$this->template->currentResourcesForm = $curResForm;
		/* --- Current res section ---*///end
		if($anyLockedWorker && $this->isAllowed(self::RESOURCE_RESOURCE_LOCK, self::ACTION_RUN))
			$this->template->assignmentTrasferMessage = "The worker(s): ". implode(', ', $lockedWorkerList) . '. to be tranferred is/are locked by resource owner. Are you sure you will not violate resource owner´s intentions?'; // inform PMO users is any worker is locked
		else
			$this->template->assignmentTrasferMessage = null;

		
		/* --- Non assigned workers section --- */
		// assign worker form
		$assignWorkerForm = $this['assignWorkerForm'];
		$notAssignedWorkersEAC = $taskPlanner->getUnassignedWorkersEACArray();
		$nonAssignedWorkers = array();
		foreach($notAssignedWorkersEAC as $eid => $monthArray)
		{
				$unassigned_worker = $this->context->personRepository->find($eid);
				if ($unassigned_worker !== NULL)
				{
					$nonAssignedWorkers[$eid]['person'] = $unassigned_worker;
					$assignmentControl = new AssignmentControl($unassigned_worker, $task);
					$nonAssignedWorkers[$eid]['assignmentControl'] = $assignmentControl;
					if(!$assignmentControl->assignable)
						$assignWorkerForm['assign_' . $eid]->setDisabled(true);
					if($assignmentControl->LockedPersonPMOWarning)
						$assignWorkerForm['assign_' . $eid]->setAttribute('onclick', '
							return confirm("Worker ' . $unassigned_worker->ReversedFullNameWithEID . ' is locked. Do you really want to assign worker?");
							');
				}
				else
				{
					$nonAssignedWorkers[$eid]['person'] = "!Unknown resource! (" . $eid . ")";
					$this->flashMessage("There is an unknown person with EID ".$eid." working on the task. This resource needs to be added to person list!", 'warning');
				}
			$nonAssignedWorkers[$eid]['eac'] = $monthArray;
		}
		$this->template->nonAssignedWorkers = $nonAssignedWorkers;
		$this->template->assignWorkerForm = $assignWorkerForm;
		/* --- Non assigned workers section --- *///end


		/* -- NavigationForm --- */
		$navigationForm = $this['navigationForm'];
		$navigationForm['discard']->setDisabled(!$taskPlanEditing->editable);
		$navFromDefaultValues = array('task_id' => $task->ID);
		$navigationForm->setDefaults($navFromDefaultValues);
		$this->template->navigationForm = $this['navigationForm'];
		/* -- NavigationForm --- *///end

		/* -- AssignNewWorkerForm --- */
		$assignNewWorkerForm = $this['assignNewWorkerForm'];
		$this->template->assignNewWorkerForm = $assignNewWorkerForm;
		/* -- AssignNewWorkerForm --- *///end

		if($this->year < Date('Y'))
			$this->template->firstUpdateableMonth = $first_updateable_month + $this->year; // past year -> nothing is updateble
		elseif($this->year == Date('Y'))
			$this->template->firstUpdateableMonth = $first_updateable_month; // current year -> depends on fiscal month
		else
			$this->template->firstUpdateableMonth = 1; // future year -> every month is updateable

		$this->template->project = $project;
		$this->template->costAsLabour = $task->getCostAsLabour();
		$urlPO = str_replace('?', '?do=POYear&', $this->link('TaskPlan:labor'));
		$this->template->linkToSignalPO = $urlPO;
		$urlNewWorker = str_replace('?', '?do=AssignNewWorkerByOrgUnit&', $this->link('TaskPlan:labor'));
		$this->template->linkToSignalNewWorker = $urlNewWorker;
		$urlTaskAssignmentTransferSelect = str_replace('?', '?do=TaskAssignmentTransferSelect&', $this->link('TaskPlan:labor'));
		$this->template->linkToTaskAssignmentTrasfer = $urlTaskAssignmentTransferSelect;
		$this->template->task = $task;

		/* PMB form */
		$planHoursDefaults = array();
		foreach($resourcesInfo['sapHours'] as $monthNumber => $hours)
		{
			$planHoursDefaults['pmb' . $monthNumber] = $hours;
		}
		foreach($resourcesInfo['eacHours'] as $monthNumber => $hours)
		{
			$planHoursDefaults['actuals' . $monthNumber] = $hours;
		}
		$planHoursDefaults['year'] = $this->year;
		$planHoursDefaults['taskID'] = $task->ID;
		$planHoursForm = $this['planHoursForm'];
		$planHoursForm->setDefaults($planHoursDefaults);
		$planHoursForm['save']->setDisabled(!$taskPlanEditing->editable);
		if(!$taskPlanEditing->editable)
			$planHoursForm['save']->setAttribute('data-form-button-tooltip', $taskPlanEditing->Message);
		$planHoursForm['discard']->setDisabled(!$taskPlanEditing->editable);
		$this->template->planHoursForm = $planHoursForm;
		$this->template->eacLink = $this->link('eAC:AjaxCall!');
	}
	
	/**
     * Creates component PlanHoursForm.
	 * Labour Actuals added to the form.
     * 
     * @param string $name Name of the form.
     * @return \AppForm Instance of the form.
     */
	public function createComponentPlanHoursForm($name)
	{
		$form = new AppForm($this, $name);
        for($i = 1; $i <= 12; $i++)
        {
			$form->addText('pmb' . $i, '')
				->addCondition(Form::FILLED, TRUE)
				->addRule(Form::INTEGER, 'SAP(h) must be integer.');
            $form['pmb' . $i]->setAttribute('size', '7');
			$form->addText('actuals' . $i, '')
				->addCondition(Form::FILLED, TRUE)
				->addRule(Form::INTEGER, 'Actuals must be integer.');
            $form['actuals' . $i]->setAttribute('size', '7');
        }
		$form->addHidden('taskID');
		$form->addHidden('year');
		
        $form->addSubmit('save', 'Save Values');
		
		if (!$this->testUserRoles('PMO'))
            $form['save']->setDisabled(TRUE);
        
		$form->addSubmit('discard', 'Discard changes')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'discardClicked');
		if (!$this->testUserRoles('PMO'))
			$form['discard']->setDisabled(TRUE);

        $form->onSuccess[] = callback($this, 'planHoursFormSubmitted');
		
        return $form;
	}
	
    /**
     * Plan Hours form submitted.
     * 
     * @param AppForm $form 
     */
    public function planHoursFormSubmitted(AppForm $form)
	{
		if($form->isValid())
		{
			$values = $form->getValues();
			$task = $this->context->taskRepository->find($values['taskID']);
			$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($values['taskID'], $values['year']);
			$project = $this->context->projectRepository->find($task->ProjectID);
			$success = true;
			//process plan values
			for($i = 1; $i <= 12; $i++)
			{
				/* Workaround for sending empty string or any number of spaces only
				 * 
				 * I was unable to handle this in createComponentPlanHoursForm() -
				 * addRule(Form::FILLED, TRUE) fails there, because if we're submitting
				 * Plan then EAC form isn't FILLED and vice-versa.
				 * 
				 * And if we use addCondition(Form::FILLED, TRUE), then it isn't strict
				 * enough as it allows for instance empty string to be entered and then
				 * we get Server error 500 as a response.
				 * 
				 * This solution isn't uniform, because it shows user the red bar upon entering
				 * wrong value. It should highlight the cell with wrong value.
				 */
				if(!is_numeric($values['pmb' . $i]))
				{
					$success = false;
					break;
				}
				$task_plan->setMonthPlan($i, new MonthPlan($values['pmb' . $i]), 0);
			}
			if(!$this->context->taskPlanRepository->update($task_plan))
				$success = false;
			
			if(($task->Fake == Task::TASK_TYPE_PLACEHOLDER) && $task->ActualsImportDataSource == TASK::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL)
			{
				try
				{
					$this->context->actualsRepository->deleteActualsByTaskSAPIDAndYear($task->SAPID, $values['year']);
				}
				catch (Exception $e)
				{
					Debugger::log("Deleting actuals labour for year = " . $values['year'] . " and task " . $task->SAPID . " failed: " . $e->getMessage(), Debugger::ERROR);
					$this->flashMessage('Labour actuals wasn\'t deleted before inserting.', 'error');
					$this->redirect('this');
				}
				//proccess actuals values
				for($i = 1; $i <= 12; $i++)
				{
					if(is_numeric($values['actuals' . $i]) && $values['actuals' . $i] != 0)
					{
						$actuals = new Actuals();
						$actuals->Eid = 'ENN0000';// TODO: more info needed!!!
						$actuals->Charge_number = $task->SAPID;
						$actuals->Year = $values['year'];
						$actuals->Fiscal_month = ($i - 1);
						$actuals->Hours = $values['actuals' . $i];
						$actuals->Project_sap_id = $project->SAPID;
						$actuals->Source = Actuals::SOURCE_MANUAL;
						if (!$this->context->actualsRepository->add($actuals))
							$success = false;
					}
				}
			}
			if($success)
				$this->flashMessage('Task values updated.', 'success');
			else
				$this->flashMessage('Task values updated unsuccessfully.', 'error');
			$this->redirect('this');
		}
		else
		{
			$this->flashMessage('Invalid form values', 'error');
			$this->redirect('this');
		}
	}
    
    /**
     * Creates EAC table
     * @return \EACControl
     */
    public function createComponentEAC()
    {
        $task = $this->context->taskRepository->find($this->getParameter('id'));        
        $control = new EACControl($task, $this->year, $this->getParameter('expandMonths'));
        return $control;
    }
	
	/**
	 * Signal unassign person from task for given year.
	 * 
	 * @param int $taskID Task id.
	 * @param string $eid Person eid.
	 * @param int $year Year.
	 */
	public function handleUnassign($eid, $taskID, $year)
	{
		$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($taskID, $year);
        if($task_plan === NULL)
		{
			$task_plan = new TaskYearPlan($year);
            $task_plan->TaskID = $taskID;
		}
		$task_planner = new TaskResourceYearPlanner($this->context, $task_plan);
		try
		{
			$task = $this->context->taskRepository->find($taskID);
			if(is_null($task))
			{
				$this->flashMessage("Task with ID " . $taskID . " was not found. ", 'error');
				$this->redirect('this');
			}
			$person = $this->context->personRepository->find($eid);
			if(is_null($person))
			{
				$this->flashMessage("Person with EID " . $eid . " was not found. ", 'error');
				$this->redirect('this');
			}
			$assignmentControl = new AssignmentControl($person, $task);
			if(!$assignmentControl->assignable)
			{
				$this->flashMessage("Person with EID " . $eid . " was not unassigned. " . $assignmentControl->Message, 'error');
				$this->redirect('this', $taskID, $year);
			}
			// unassign worker
			$task_planner->unassignWorker($eid);
			$this->flashMessage("Worker with EID " . $eid . " was unassigned.", 'success');
			//Licence control mechanism
			$project = $this->context->projectRepository->findProjectByTaskId($taskID);
			$removeLicence = true;
			$allPersonProjects = $this->context->projectRepository->findPersonAssignedAtProjectsInYear($eid, $year);
			foreach($allPersonProjects as $personProject)
			{
				if($personProject->ECLicenceID == $project->ECLicenceID)//check if person has any other assignment under this licence. If other assignment doesn't exist, remove licence from preson.
				{
					$projectTasks = $this->context->taskRepository->findPersonTasksInYear($eid, $personProject->ID, $year);
					foreach($projectTasks as $task)
					{
						$assignment = 0;
						$assigns = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task->ID, $year, $eid);
						foreach($assigns as $assign)
						{
							if($assign->Month >= Date('m'))
								$assignment += $assign->Hours;
						}
						if($assignment > 0)
						{
							$removeLicence = false;
							break 2;
						}
					}
				}
			}
			if($removeLicence)
				$this->context->personECLicenceRepository->delete($eid, $project->ECLicenceID);
		}
		catch (InvalidStateException $e)
		{
			$this->flashMessage("Worker with EID " . $eid . " cannot be unassigned because he already worked on the task.", 'error');
		}
		$this->redirect('this', $taskID, $year);
	}
	
	/**
	 * Signal assign person to task for given year.
	 * 
	 * @param int $taskID Task id.
	 * @param string $eid Person eid.
	 * @param int $year Year.
	 */
	public function handleAssign($eid, $taskID, $year)
	{
		$task = $this->context->taskRepository->find($taskID);
		if(is_null($task))
		{
			$this->flashMessage("Task with ID " . $taskID . " was not found. ", 'error');
			$this->redirect('this');
		}
		$person = $this->context->personRepository->find($eid);
		if(is_null($person))
		{
			$this->flashMessage("Person with EID " . $eid . " was not found. ", 'error');
			$this->redirect('this');
		}
		$assignmentControl = new AssignmentControl($person, $task);
		if(!$assignmentControl->assignable)
		{
			$this->flashMessage("Person with EID " . $eid . " was not assigned. " . $assignmentControl->Message, 'error');
			$this->redirect('this');
		}
		$assignment = new PersonTaskMonthAssignment();
		$assignment->TaskID = $taskID;
		$assignment->Hours = 0;
		$assignment->Month = 1;
		$assignment->Year = $year;
		$assignment->PersonID = $eid;
		if($this->context->personTaskAssignmentRepository->save($assignment))
		{
			//Licence control mechanism
			$project = $this->context->projectRepository->findProjectByTaskId($taskID);
			if(is_null($this->context->personECLicenceRepository->find($eid, $project->ECLicenceID)))// check if person has already other assignment under this licence. If other assignment doesnt exist, add licence to preson.
			{
				$personLicence = new PersonECLicence();
				$personLicence->EID = $eid;
				$personLicence->ECLicenceID = $project->ECLicenceID;
				$this->context->personECLicenceRepository->save($personLicence);
			}
			$this->flashMessage("Person with EID " . $assignment->PersonID . " was assigned.", 'success');
		}
		else
			$this->flashMessage("Person with EID " . $assignment->PersonID . " was not assigned.", 'error');
		$this->redirect('this');
	}
	
    /**
     * Load PO value for selected year
     * @param int $id Task ID
	 * @param int $year Year
     */
    public function handlePOYear($id, $year)
    {
		$this->invalidateControl('poNumberSnippet');
    }
    
	/**
	 * Nette non labor render method.
	 * @param int $id Non labor task id.
	 */
	public function renderNonLabor($id = null)
	{
		// get task
		$task = $this->context->taskRepository->find($id);
		if($task == null)
		{
			$this->flashMessage('Task with ID = ' . $id . ' not found.', 'notice');
			$this->redirect('Project:default');
		}
		$project = $this->context->projectRepository->findProjectByTaskId($task->ID);

		//get TaskPlan
		$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year);
		if($task_plan === NULL)
		{
			/* If there is nobody with actual(s) for this SAP_ID and year,
			 * redirect. Otherwise create an empty TaskPlan.
			 */
			$task_plan = $this->createNewTaskPlanOrRedirect($task, $this->year);
		}
		
		// EAC reporter preparation
		$first_updateable_month = FiscalCalendar::currentFiscalMonth() + 1;
        $project = $this->context->projectRepository->findProjectByTaskId($id);
        $project_reporter = new ProjectEACReporter($this->context, $project, $this->year);
		
		// get task plan editing control object
		$taskPlanEditing = new TaskPlanEditing($task, $this->year);
		$this->template->taskPlanEditing = $taskPlanEditing;
		
		/* --- Task section --- */// start
		$remarkForm = $this['remarkForm'];
		$poNumber = $this->context->taskPONumberRepository->findByTaskIDandYear($task->ID, $this->year);
		$poNumberID = null;
		if(!$poNumber)
		{
			$remarkForm['poNumber']->setAttribute('placeholder', 'Undefined');
		}
		else
		{
			$poNumberID = $poNumber->ID;
			$poNumber = $poNumber->PONumber;
		}

		$remarkFormDefaults = array(
			'task_id' => $task->ID,
			'poNumber_id' => $poNumberID,
			'remark' => $task->Remark,
			'year' => $this->year,
			'poNumber' => $poNumber
		);
		$remarkForm->setDefaults($remarkFormDefaults);
		$this->template->remarkForm = $remarkForm;
		
		//Task assignment lock form
		$taskLockForm = $this['taskLockForm'];
		$taskLockForm['save']->setDisabled(!$this->isAllowed(self::RESOURCE_TASK_ASSIGNMENT_LOCK, self::ACTION_EDIT));
		if($task->Locked)
			$taskLockFormDefaults = array('AssignmentLock' => 1);
		else
			$taskLockFormDefaults = array('AssignmentLock' => 0);
		$taskLockFormDefaults['task_id'] = $task->ID;
		$taskLockForm->setDefaults($taskLockFormDefaults);
		$this->template->taskLockForm = $taskLockForm;
		
		$taskInfo = array();
		$taskInfo['name'] = $task->Name;
		$taskInfo['workPackageStructure'] = '';
		foreach ($this->context->workPackageRepository->getWPTreePathByTaskID($task->ID) as $workPackage)
			$taskInfo['workPackageStructure'][] = $workPackage->Name;
		$taskInfo['workPackageStructure'] = implode(' » ', $taskInfo['workPackageStructure']);
		$taskInfo['costShare'] = $task->CostSharePercentage;
		$this->template->taskInfo = $taskInfo;
        $url = str_replace('?', '?do=POYear&', $this->link('TaskPlan:nonLabor'));
        $this->template->linkToSignal = $url;
		/* --- Task section --- */// end


		/* --- SAP section --- */
		$this->template->sapInfo = array();
		$this->template->sapInfo['sapID'] = $task->SAPID;
		$this->template->sapInfo['effort'] = 0;
//        foreach ($this->context->taskPlanRepository->findTaskPlans($task->ID) as $task_plan)
//        {
//		      $task_planner = new TaskResourceYearPlanner($this->context, $task_plan);
//            $this->template->sapInfo['effort'] += $task_planner->getTaskYearPlanHours()->sum();
//        }

		/* --- Project EAC section --- */
		$this->template->projectEac = array();
		$this->template->projectEac['ready'] = true;
		try
		{
			$this->template->projectEac['labourEACCosts'] = $project_reporter->getProjectLabourEACCostsMonthTotals();
			$this->template->projectEac['nonLabourEACCosts'] = $project_reporter->getProjectNonLabourEACCostsMonthTotals();
			$this->template->projectEac['labourPlannedHours'] = $project_reporter->getProjectLabourPlannedHoursMonthTotals();
			$this->template->projectEac['labourEACHours'] = $project_reporter->getProjectLabourEACHoursMonthTotals();
			$this->template->projectEac['labourPlanEACDelta'] = $project_reporter->getProjectLabourPlanEACMonthDelta();
			$this->template->projectEac['labourPlannedCostsTotals'] = $project_reporter->getProjectLabourPlannedCostsMonthTotals();
			$this->template->projectEac['labourEACCostsTotals'] = $project_reporter->getProjectLabourEACCostsMonthTotals();
			$this->template->projectEac['labourPlanEACCostsTotalsDelta'] = $project_reporter->getProjectLabourPlanEACCostsMonthTotalsDelta();
			/**/
			$this->template->projectEac['nonLabourPlannedCostsTotals'] = $project_reporter->getProjectNonLabourPlannedCostsMonthTotals();
			$this->template->projectEac['nonLabourEACCostsTotals'] = $project_reporter->getProjectNonLabourEACCostsMonthTotals();
			$this->template->projectEac['nonLabourCostsTotalsDelta'] = $project_reporter->getProjectNonLabourCostsMonthTotalsDelta();
		}
		catch (InvalidStateException $e)
		{
			$this->template->projectEac['ready'] = false;
			Debugger::log($e->getMessage(), Debugger::ERROR);
			$this->flashMessage($e->getMessage() . " Project EAC section unavailable.", 'error');
		}
		
		/* --- Resources section --- */
		// get planned years            
		$task_plan_years = $actuals_years = array();                
		$task_plan_years = $this->context->taskPlanRepository->findTaskYears($task->ID);
		$actuals_years = $this->context->actualsCostRepository->findTaskYears($task->ID);

		$availableYears = array_unique(array_merge($task_plan_years, $actuals_years));

		$this->template->AvailableYears = $availableYears;
		if (!in_array($this->year, $availableYears))
		{
			$alternative_year = 0;
			if (in_array($this->year + 1, $availableYears))
				$alternative_year = $this->year + 1;
			elseif (in_array($this->year - 1, $availableYears))
				$alternative_year = $this->year - 1;
			elseif (count($availableYears))
				$alternative_year = $availableYears[0]; // first year of project
			if ($alternative_year != 0)
			{
				$this->flashMessage("There is no plan for current year $this->year. Year $alternative_year selected.", 'info');
				$this->redirect('nonLabor', 
					array(	'id' => $task->ID, 
							'year' => $alternative_year
						));
			}
			else
			{
				$this->flashMessage("There is no plan for task: " . $task->Name . ".", 'info');
				$this->redirect('Project:detail', $project->ID, $this->year);
			}
		}
		
		$costAssignmentDefaults = $costPlanDefaults = $costActualsDefaults = array();

		$plan_values = $task_actuals = $task_deltas = array();
		$monthly_task_costs = $this->context->costCenterRepository->findActualCostsYearSummary($task->ID, $this->year, 'USD'); // sap plan values
		$assignmentsArray = $this->context->assignmentsCostRepository->findTaskYearAssignments($task->ID, $this->year); // ATE Tool assignmet values
		for ($i = 1; $i <= 12; $i++)
		{
			$plan_values[$i] = $task_plan->getMonthPlan($i)->Budget;
			$actual_values[$i] = $costActualsDefaults['actuals' . $i] = round($monthly_task_costs[$i]);
			if($this->year < Date('Y')) // past years
			{
				$costAssignmentDefaults[$i] = $actual_values[$i];
				$costPlanDefaults['pmb' . $i] = $plan_values[$i];
				$delta_values[$i] = $plan_values[$i] - $actual_values[$i];
			}
			elseif($this->year == Date('Y')) // current year
			{
				if($i < $first_updateable_month) // past months
				{
					$costAssignmentDefaults[$i] = $actual_values[$i];
					$costPlanDefaults['pmb' . $i] = $plan_values[$i];
					$delta_values[$i] = $plan_values[$i] - $actual_values[$i];
				}
				else //future months
				{
					$costPlanDefaults['pmb' . $i] = $plan_values[$i];
					if(isset($assignmentsArray[$i-1])) // assignment exists
					{
						$costAssignmentDefaults[$i] = $assignmentsArray[$i-1]->Cost;
						$delta_values[$i] = $plan_values[$i] - $assignmentsArray[$i-1]->Cost;
					}
					else // no assignment exists -> plan is default value
					{
						$costAssignmentDefaults[$i] = $plan_values[$i];
						$delta_values[$i] = 0;
					}
				}
			}
			else // future years
			{
				$costPlanDefaults['pmb' . $i] = $plan_values[$i];
				if(isset($assignmentsArray[$i-1])) // assignment exists
				{
					$costAssignmentDefaults[$i] = $assignmentsArray[$i-1]->Cost;
					$delta_values[$i] = $plan_values[$i] - $assignmentsArray[$i-1]->Cost;
				}
				else // no assignment exists -> plan is default value
				{
					$costAssignmentDefaults[$i] = $plan_values[$i];
					$delta_values[$i] = 0;
				}
			}
		}
		
		if($this->year < Date('Y'))
			$this->template->eac_values_sum = array_sum($actual_values);
		else
			$this->template->eac_values_sum = array_sum($costAssignmentDefaults);
		$this->template->plan_values = $plan_values;
		$this->template->actual_values = $actual_values;
		$this->template->delta_values = $delta_values;
		if($this->year < Date('Y'))
			$this->template->firstUpdateableMonth = $first_updateable_month + $this->year; // past year -> nothing is updateble
		elseif($this->year == Date('Y'))
			$this->template->firstUpdateableMonth = $first_updateable_month; // current year -> depends on fiscal month
		else
			$this->template->firstUpdateableMonth = 1; // future year -> every month is updateable
		$costAssignmentDefaults['year'] = $this->year;
		$costAssignmentDefaults['taskID'] = $task->ID;
		
		/* --- year navigation --- */
		$this->template->previousYear = Html::el('button')->setText('<--');
		if (($previousTaskPlan = $this->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year - 1)))
		{
			$this->template->previousYear->addAttributes(array('id' => 'prevYear'))
				->addAttributes(array('title' => 'Go to previous year task plan'))
				->addAttributes(array('class' => 'navbtnOn'));
			$this->template->previousTask = $previousTaskPlan->TaskID;
		}
		else
		{
			$this->template->previousYear->addAttributes(array('disabled' => 'disabled'))
				->addAttributes(array('title' => 'No task plan for previous year'))
				->addAttributes(array('class' => 'navbtnOff'));
		}
		$this->template->nextYear = Html::el('button')->setText('-->');
		if (($nextTaskPlan = $this->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year + 1)))
		{
			$this->template->nextYear->addAttributes(array('id' => 'nextYear'))
				->addAttributes(array('title' => 'Go to next year task plan'))
				->addAttributes(array('class' => 'navbtnOn'));
			$this->template->nextTask = $nextTaskPlan->TaskID;
		}
		else
			$this->template->nextYear->addAttributes(array('disabled' => 'disabled'))
				->addAttributes(array('title' => 'No task plan for next year'))
				->addAttributes(array('class' => 'navbtnOff'));
		
		/* -- NavigationForm --- */
		$navigationForm = $this['navigationForm'];
		$navigationForm['discard']->setDisabled(!$taskPlanEditing->editable);
		$navFromDefaultValues = array('task_id' => $task->ID);
		$navigationForm->setDefaults($navFromDefaultValues);
        $this->template->navigationForm = $this['navigationForm'];
		
		/* -- costAssignmentsForm --- */
		$costAssignmentsForm = $this['costAssignmentsForm'];
        // set unavailable months
		for($i = 1; $i <= 12; $i++)
        {
			if($this->year == Date('Y'))
			{
				if($i < $first_updateable_month)
				{
					$costAssignmentsForm[$i]->setAttribute('readonly', 'readonly');
				}
			}
			elseif($this->year < Date('Y'))
			{
				$costAssignmentsForm[$i]->setAttribute('readonly', 'readonly');
				$costAssignmentsForm['save']->setDisabled(true);
			}
        }
		$costAssignmentsForm->setDefaults($costAssignmentDefaults);
		$costAssignmentsForm->setDefaults($costPlanDefaults);
		$costAssignmentsForm->setDefaults($costActualsDefaults);
		$costAssignmentsForm['save']->setDisabled(!$taskPlanEditing->editable);
		if(!$taskPlanEditing->editable)
			$costAssignmentsForm['save']->setAttribute('data-form-button-tooltip', $taskPlanEditing->Message);
		$costAssignmentsForm['discard']->setDisabled(!$taskPlanEditing->editable);
        for($i = 1; $i <= 12; $i++)
        {
            $costAssignmentsForm[$i]->setDisabled(!$taskPlanEditing->editable);
        }
		$this->template->taskPlanType = 'NonLabor';
		$this->template->project = $project;
		$this->template->task = $task;
        $this->template->costAssignmentsForm = $costAssignmentsForm;
		$this->template->eacLink = $this->link('eAC:AjaxCall!');
	}
	
    /**
     * Nette render method for when TaskPlan_ID not found.
     * 
     * @param int $id TaskPlan_ID.
     */
    public function renderTaskPlanNotFound($id)
    {
        $this->template->id = $id;
    }

    /**
     * Creates component RemarkForm for editing Remark.
     * 
     * @param string $name Name of the form.
     * @return \AppForm Instance of the form.
     */
    public function createComponentRemarkForm($name)
    {
        $form = new AppForm($this, $name);
        
        $form->addHidden('task_id');
		
		$form->addHidden('poNumber_id');
        
        $form->addTextArea('remark', 'Remark', 40, 3);
		
		$plannedYears = $this->context->taskPlanRepository->findTaskPlans($this->getHttpRequest()->getQuery('id'));
		$formYear = array();
		foreach ($plannedYears as $key => $value)
		{
			$formYear[$key] = $key;
		}
        $form->addSelect('year', 'Year', $formYear)
				->setDisabled(TRUE);
		
		$form->addText('poNumber', 'PO Number')
			->setDisabled(TRUE)
			->addConditionOn($form['poNumber'], Form::FILLED)
				->addRule(Form::MIN_LENGTH, 'PO number must be positive integer and must have 10 numbers at least!', 10)
				->addRule(Form::INTEGER, 'PO number must be positive integer and must have 10 numbers at least!');
        
        $form->addSubmit('save', 'Save');
        
        $form->onSuccess[] = callback($this, 'remarkFormSubmitted');
        
        return $form;
    }
    
    /**
     * Component RemarkForm submit method.
     * Saves task Remark information.
     * 
     * @param AppForm $form RemarkForm.
     */
    public function remarkFormSubmitted(AppForm $form)
    {
        $values = $form->getValues();
		$task = $this->context->taskRepository->find($values['task_id']);
//		$poCreated = false;
//		if($task->getCostAsLabour()) // if Cost as Labour task then process PO Number
//		{
//			if($values['poNumber_id']) // updates existing PO number
//			{
//				$taskPONumber = $this->context->taskPONumberRepository->findByTaskIDandYear($values['task_id'], $values['year']);
//				if($values['poNumber'])// update
//				{
//					$taskPONumber->PONumber = $values['poNumber'];
//					if($this->context->taskPONumberRepository->update($taskPONumber))
//						$this->flashMessage('Task PO number updated successfully.', 'success');
//					else
//						$this->flashMessage('Task PO number updated unsuccessfully.', 'error');
//				}
//				else
//				{
//					if($this->context->taskPONumberRepository->delete($taskPONumber->ID))
//						$this->flashMessage('Task PO number deleted successfully.', 'success');
//					else
//						$this->flashMessage('Task PO number deleted unsuccessfully.', 'error');
//				}
//			}
//			else
//			{
//				if($values['poNumber']) // creates new PO number
//				{
//					$poCreated = true;
//					$taskPONumber = new TaskPONumber();
//					$taskPONumber->Year = $values['year'];
//					$taskPONumber->TaskID = $values['task_id'];
//					$taskPONumber->PONumber = $values['poNumber'];
//					if($this->context->taskPONumberRepository->save($taskPONumber))
//						$this->flashMessage('Task PO number inserted successfully.', 'success');
//					else
//						$this->flashMessage('Task PO number inserted unsuccessfully.', 'error');
//				}
//			}
//			// Import actuals from IASAP database
//			if($values['poNumber'])
//			{
//				// init logging
//				$logs_suffix = time().'-'. rand(1, 9);
//				$current_log_file_name = 'iasap_actuals_import-'.$logs_suffix.'.html';
//				$batch_id = 'iasap_actuals-'.$logs_suffix;
//				$filelog_path = TEMP_DIR . DIRECTORY_SEPARATOR . $current_log_file_name;
//				$filelog = $this->context->createFileLogger($filelog_path, $batch_id, "IASAP ACTUALS") ;
//				$multilog = new MultiLogger();
//				$multilog->add($filelog);
//				$multilog->add($this->context->createLogger($batch_id, "IASAP ACTUALS"));
//
//				$import_result = FALSE;
//				$import_ex_error = '';
//
//				$po_numbers = array();
//				$po_numbers[] = $taskPONumber->PONumber;
//				
//				$iASAPActualsImportService = new IASAPActualsImportServicePO($this->context, $po_numbers, $multilog);
//
//				try
//				{
//					switch ($iASAPActualsImportService->runImport())
//					{
//						case iASAPActualsImportServicePO::RESULT_SUCCESS:
//							$this->flashMessage('iASAP actuals import was successful.', 'success');
//							break;
//						case iASAPActualsImportServicePO::RESULT_WARNING:
//							$this->flashMessage('iASAP actuals import was successful with warnings.', 'warning');
//							break;
//						case iASAPActualsImportServicePO::RESULT_ERROR:
//							$this->flashMessage('iASAP actuals import failed!', 'error');
//							break;
//						case iASAPActualsImportServicePO::RESULT_NO_DATA:
//							$this->flashMessage('For PO number: "' . $taskPONumber->PONumber . '" was not found any actuals!', 'warning');
//							break;
//						default: break;
//					}
//				}
//				catch (IASAPActualsImportPOServiceException $e)
//				{
//					$import_result = FALSE;
//					$import_ex_error = $e->getMessage();
//					$multilog->log('iASAP actuals import failed: ' . $e->getMessage(), ILogger::ERROR);
//					Debugger::log('iASAP actuals import failed: ' . $e->getMessage(), Debugger::ERROR);
//					$this->flashMessage('Actuals imported unsuccessfully.', 'error');
//				}
//				$filelog->close();
//			}
//		}
		
		
		// update remark
		$task->remark = $values['remark'];
		if($this->context->taskRepository->update($task))
			$this->flashMessage('Task remark update successfully.', 'success');
		else
			$this->flashMessage('Task remark update unsuccessfully.', 'error');
//		if($poCreated)
//			$this->redirect('TaskPlan:labor', $values['task_id'], $values['year']);
        $this->redirect('this');
    }
	
    /**
     * Creates component CurrentResourcesForm for updating workers assigned hours to the taskplan.
     * 
     * @param string $name Name of the form.
     * @return AppForm Instance of the form.
     */
	public function createComponentCurrentResourcesForm($name)
	{
        $form = new AppForm($this, $name);
		$taskID = $this->getHttpRequest()->getQuery('id');
		$task = $this->context->taskRepository->find($taskID);
		$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($taskID, $this->year);
        if ($task_plan === NULL)
        {
            $task_plan = new TaskYearPlan($this->year);
            $task_plan->TaskID = $taskID;
        }
		$taskPlanner = new TaskResourceYearPlanner($this->context, $task_plan);
        $assigned_workers = $taskPlanner->getAssignedWorkerEACArray();
        foreach ($assigned_workers as $eid => $workerAssignment)
        {
            foreach ($workerAssignment as $monthNumber => $eacHours)
            {
				$form->addText($eid . "_" . $monthNumber, NULL, 1)
						->setDefaultValue($eacHours)
						->addRule(AppForm::RANGE, 'EAC hours must be number from 0 to 9999.', array(0, 9999));
				$person = $this->context->personRepository->find($eid);
				$assignmentControl = new AssignmentControl($person, $task);
				if(!$assignmentControl->assignable)
					$form[$eid . "_" . $monthNumber]->setAttribute('readonly', 'readonly');
            }
        }
		
        $form->addSubmit('save', 'Save Task Assignments')
				->onClick[] = callback($this, 'saveTaskAssignmentsClicked');
		$form->addSubmit('delete', 'Delete Task Assignments')
				->setValidationScope(FALSE)
				->setAttribute('onclick', '
				return confirm("Do you really want to delete all task assignment?");
				')->onClick[] = callback($this, 'deleteTaskAssignmentsClicked');
		$form->addSubmit('discard', 'Discard changes')
			->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'discardClicked');
        
        return $form;
	}
	
    /**
     * Current Resources Form Submitted - Save.
     * @param SubmitButton $button 
     */
    public function saveTaskAssignmentsClicked(SubmitButton $button)
    {
        $values = $button->form->getValues();
		if($button->form->isValid())
		{
			$first_updateable_month = FiscalCalendar::currentFiscalMonth() + 1;
			$taskID = $this->getHttpRequest()->getQuery('id');
			$persons = array();
			// process values
			foreach($values as $key => $value)
			{
				$tempArray = explode('_', $key);
				$persons[$tempArray[0]][$tempArray[1]] = $value;

			}
			$success = false;
			foreach($persons as $eid => $value)
			{
				$person_task = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($taskID, $this->year, $eid);
				// update person assignments
				foreach($person_task as $month)
				{
					if ($month->Month >= $first_updateable_month && $this->year == Date('Y'))
						$month->setHours($value[$month->Month]);
					elseif($this->year > Date('Y'))
						$month->setHours($value[$month->Month]);
				}
				// save person assignments
				foreach($person_task as $month)
				{
					if ($month->Month >= $first_updateable_month && $this->year == Date('Y'))
					{
						if ($this->context->personTaskAssignmentRepository->save($month))
						{
							$success = true;
						}
						else
						{
							$success = false;
						}
						if(!$success)
							break 2;
					}
					elseif($this->year > Date('Y'))
					{
						if ($this->context->personTaskAssignmentRepository->save($month))
						{
							$success = true;
						}
						else
						{
							$success = false;
						}
						if(!$success)
							break 2;
					}
				}
			}

			if($success)
			{
				$this->flashMessage("Task plan updated.", 'success');
			}
			else
			{
				$this->flashMessage("Task plan updating failed.", 'error');
			}
		}
		else
        {  
            $this->flashMessage("Invalid form values.");
        }
        $this->redirect('TaskPlan:labor', $taskID, $this->year);
    }
	
    /**
     * Current Resources Form Submitted - Delete.
     * @param SubmitButton $button 
     */
    public function deleteTaskAssignmentsClicked(SubmitButton $button)
    {
		$taskID = $this->getHttpRequest()->getQuery('id');
		$year = $this->getHttpRequest()->getQuery('year');
		if($this->context->personTaskAssignmentRepository->deleteByTaskAndYearAssignment($taskID, $year))
		{
			$this->flashMessage("Task assignment was deleted.", 'success');
		}
		else
		{
			$this->flashMessage("Task assignment deleting failed.", 'error');
		}
        $this->redirect('TaskPlan:labor', $taskID, $year);
    }
    
    /**
     * Creates component NavigationForm for bottom navigation buttons 'Discard Changes' and 'Go Back'.
     * 
     * @param string $name Name of the form.
     * @return \AppForm Instance of the form.
     */
    public function createComponentNavigationForm($name)
    {
        $form = new AppForm($this, $name);
        
		$form->addHidden('task_id');
        
        $form->addSubmit('discard', 'Discard changes')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'discardClicked');
        
        return $form;
    }

    /**
     * DiscardClicked method used to discard changes when bottom navigation 'Discard Changes' button pressed.
     */
    public function discardClicked()
    {
        $this->flashMessage('Changes discarded.', 'info');
        $this->redirect('this');
    }
	
	/**
     * Creates component costAssignmentsForm.
	 * Cost Actuals added to the form.
     * 
     * @param string $name Name of the form.
     * @return \AppForm Instance of the form.
     */
	public function createComponentCostAssignmentsForm($name)
	{
		$form = new AppForm($this, $name);
        for($i = 1; $i <= 12; $i++)
        {
            $form->addText($i, '')
				->addCondition(Form::FILLED, TRUE)
				->addRule(Form::INTEGER, 'EAC must be integer.');
            $form[$i]->setAttribute('size', '7');
			$form->addText('pmb' . $i, '')
				->addCondition(Form::FILLED, TRUE)
				->addRule(Form::INTEGER, 'SAP must be integer.');
            $form['pmb' . $i]->setAttribute('size', '7');
			$form->addText('actuals' . $i, '')
				->addCondition(Form::FILLED, TRUE)
				->addRule(Form::INTEGER, 'Actuals must be integer.');
            $form['actuals' . $i]->setAttribute('size', '7');
        }
		$form->addHidden('taskID');
		$form->addHidden('year');
		
		$form->addSubmit('discard', 'Discard changes')
			->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'discardClicked');
		
		if (!$this->testUserRoles('PMO'))
            $form['discard']->setDisabled(TRUE);
				
        $form->addSubmit('save', 'Save Values');
		if (!$this->testUserRoles('PMO'))
            $form['save']->setDisabled(TRUE);
        $form->onSuccess[] = callback($this, 'costAssignmentsFormSubmitted');
		
        return $form;
	}
	
    /**
     * Cost assignments form submitted.
     * 
     * @param AppForm $form 
     */
    public function costAssignmentsFormSubmitted(AppForm $form)
	{
		if($form->isValid())
		{
			$values = $form->getValues();
			
            $current_year = intval(date('Y'));
            $year = intval($values->year);
			
            // Past
            if ($current_year > $year)
                $first_updatable_month = 13;
            // Present
            elseif ($current_year == $year)
                $first_updatable_month = FiscalCalendar::currentFiscalMonth() + 1;
            // Future
            else
                $first_updatable_month = 1;
                
			$task = $this->context->taskRepository->find($values['taskID']);
			$project = $this->context->projectRepository->find($task->ProjectID);
			$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($values['taskID'], $values['year']);
			$assignmentsArray = $this->context->assignmentsCostRepository->findTaskYearAssignments($values['taskID'], $values['year']);
			$success = true;
			//assignment
			for($i = 1; $i <= 12; $i++)
			{
				if($i >= $first_updatable_month)
				{
					$assignmentsCost = new AssignmentsCost();
					if(isset($assignmentsArray[$i-1]))
					{
						$assignmentsCost->ID = $assignmentsArray[$i-1]->ID;
					}
					$assignmentsCost->taskID = $values['taskID'];
					$assignmentsCost->year = $values['year'];
					$assignmentsCost->month = $i;
					$assignmentsCost->cost = $values[$i];
					if(!$this->context->assignmentsCostRepository->save($assignmentsCost))
					{
						$success = false; 
						break;
					}
				}
			}
			// Plan
			for($i = 1; $i <= 12; $i++)
			{
				$budget = (is_numeric($values['pmb' . $i])) ? $values['pmb' . $i] : 0;
				$task_plan->setMonthPlan($i, new MonthPlan(0, $budget));
			}
			if($task->Fake == Task::TASK_TYPE_PLACEHOLDER)
			{
				if(!$this->context->taskPlanRepository->update($task_plan))
					$success = false;
			}
			//Deletes old actuals
			if(($task->Fake == Task::TASK_TYPE_PLACEHOLDER) && $task->ActualsImportDataSource == TASK::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL)
			{
				try
				{
					$this->context->actualsCostRepository->deleteDataFromTaskYear($task->ID, $year, ActualsCost::SOURCE_MANUAL);
				}
				catch (Exception $e)
				{
					Debugger::log("Deleting Actuals costs for year = " . $year . " and task " . $task->SAPID . " failed: " . $e->getMessage(), Debugger::ERROR);
					$this->flashMessage('Costs actuals wasn\'t deleted before inserting.', 'error');
					$this->redirect('this');
				}
				// insert new Actuals
				for($i = 1; $i <= 12; $i++)
				{
					if(is_numeric($values['actuals' . $i]) && $values['actuals' . $i] <> 0)
					{
						$actualsCost = new ActualsCost();
						$actualsCost->Project_sap_id = $project->SAPID;
						$actualsCost->Task_id = $task->ID;
						$actualsCost->Activity_name = $task->Name;
						$actualsCost->Creation_date = Date('Y-m-d');
						$actualsCost->Posting_date = Date('Y-m-d');
						$actualsCost->Year = $year;
						$actualsCost->Fiscal_month = ($i - 1);
						$actualsCost->Description = '';
						$actualsCost->Category = '';
						$actualsCost->Quantity = 0;
						$actualsCost->Value_usd = (is_numeric($values['actuals' . $i])) ? $values['actuals' . $i] : 0; // HPT use only USD values //if it changes, here is need to be done rate recalculation
						$actualsCost->Value_czk = 0;
						$actualsCost->Value = 0;
						$actualsCost->Currency = 'USD';
						$actualsCost->Source = ActualsCost::SOURCE_MANUAL;
						if($task->Fake == Task::TASK_TYPE_PLACEHOLDER)
						{
							if (!$this->context->actualsCostRepository->save($actualsCost))
								$success = false;
						}
					}
				}
			}
			if($success)
				$this->flashMessage('Costs values updated.', 'success');
			else
				$this->flashMessage('Costs values updated unsuccessfully.', 'error');
			$this->redirect('this');
		}
		else
		{
			$this->flashMessage('Invalid form values', 'error');
			$this->redirect('this');
		}
	}
	
	/**
     * Creates component assignAssignWorkerForm. Form assign non-assigned worker who already charged.
     * 
     * @param string $name Name of the form.
     * @return \AppForm Instance of the form.
     */
	public function createComponentAssignWorkerForm($name)
	{
        $form = new AppForm($this, $name);
		$taskID = $this->getHttpRequest()->getQuery('id');
		$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($taskID, $this->year);
        if ($task_plan === NULL)
        {
            $task_plan = new TaskYearPlan($this->year);
            $task_plan->TaskID = $taskID;
        }
		$taskPlanner = new TaskResourceYearPlanner($this->context, $task_plan);
		$notAssignedWorkersEAC = $taskPlanner->getUnassignedWorkersEACArray();
		foreach($notAssignedWorkersEAC as $eid => $monthArray)
		{
			$worker = $this->context->personRepository->find($eid);
			if($worker)
			{
				$form->addSubmit('assign_' . $eid, 'Assign')
					->onClick[] = callback($this, 'assignWorkerClicked');
			}
		}
		
        return $form;
	}
	
    /**
     * AssignWorkerForm Submitted - Assign.
     * @param SubmitButton $button 
     */
    public function assignWorkerClicked(SubmitButton $button)
    {
		$assignment = new PersonTaskMonthAssignment();
		$assignment->TaskID = $this->getHttpRequest()->getQuery('id');
		$assignment->Hours = 0;
		$assignment->Month = 1;
		$assignment->Year = $this->getHttpRequest()->getQuery('year');
		$assignment->PersonID = substr($button->htmlName, 7);
		$task = $this->context->taskRepository->find($assignment->TaskID);
		if(is_null($task))
		{
			$this->flashMessage("Task with ID " . $assignment->TaskID . " was not found. ", 'error');
			$this->redirect('this');
		}
		$person = $this->context->personRepository->find($assignment->PersonID);
		if(is_null($person))
		{
			$this->flashMessage("Person with EID " . $assignment->PersonID . " was not found. ", 'error');
			$this->redirect('this');
		}
		$assignmentControl = new AssignmentControl($person, $task);
		if(!$assignmentControl->assignable)
		{
			$this->flashMessage("Person with EID " . $assignment->PersonID . " was not assigned. " . $assignmentControl->Message, 'error');
			$this->redirect('this', $assignment->TaskID, $assignment->Year);
		}
		if($this->context->personTaskAssignmentRepository->save($assignment))
		{
			//Licence control mechanism
			$project = $this->context->projectRepository->findProjectByTaskId($assignment->TaskID);
			if(is_null($this->context->personECLicenceRepository->find($assignment->PersonID, $project->ECLicenceID)))// check if person has already other assignment under this licence. If other assignment doesnt exist, add licence to person.
			{
				$personLicence = new PersonECLicence();
				$personLicence->EID = $assignment->PersonID;
				$personLicence->ECLicenceID = $project->ECLicenceID;
				$this->context->personECLicenceRepository->save($personLicence);
			}
			$this->flashMessage("Person with EID " . $assignment->PersonID . " was assigned.", 'success');
		}
		else
			$this->flashMessage("Person with EID " . $assignment->PersonID . " was not assigned.", 'error');
		$this->redirect('this');
    }
	
	/**
     * Creates component assignNewWorkerForm. Form assign new worker
     * 
     * @param string $name Name of the form.
     * @return \AppForm Instance of the form.
     */
	public function createComponentAssignNewWorkerForm($name)
	{
        $form = new AppForm($this, $name);

		$form->addSelect('assignNewWorker', 'Select new resource: ')
				->setRequired('Please select new resource!')
				->setAttribute('style', 'width: 300px;');
		
		// filter select box
        $orgUnits = array();
        $sbes = $this->context->organizationUnitRepository->findAllOrgUnitByType('SBE');
        foreach ($sbes as $sbe)
            if ($sbe->Short != 'Guest')
                $orgUnits['SBE / Tier 3'][$sbe->ID] = $sbe->Name;
        $departments = $this->context->organizationUnitRepository->findAllOrgUnitByType('DEP');
        foreach ($departments as $dep)
            $orgUnits['Department / Tier 4'][$dep->ID] = $dep->Name;
        $clusters = $this->context->organizationUnitRepository->findAllOrgUnitByType('CLUS');
        foreach ($clusters as $clus)
            $orgUnits['Cluster / Tier 5'][$clus->ID] = $clus->Name;
        $techGroups = $this->context->organizationUnitRepository->findAllOrgUnitByType('TG');
        foreach ($techGroups as $techGroup)
                $orgUnits['Technical Group'][$techGroup->ID] = $techGroup->Name;
        $form->addSelect('OrgUnit', 'Filter workers by organization unit: ', $orgUnits)
				->setPrompt('*')
				->setAttribute('style', 'width: 400px;');
		
        return $form;
	}
	
	/**
	 * Signal filters assignable workers by given Org unit ID.
	 * 
	 * @param type $q
	 * @param type $id
	 * @param type $year
	 * @param type $orgUnitID
	 */
    public function handleAssignNewWorkerByOrgUnit($q, $id, $year, $orgUnitID)
    {
		$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($id, $year);
        if ($task_plan === NULL)
        {
            $task_plan = new TaskYearPlan($year);
            $task_plan->TaskID = $id;
        }
		$taskPlanner = new TaskResourceYearPlanner($this->context, $task_plan);
        //selected org unit
		$orgUnit = $this->context->organizationUnitRepository->find($orgUnitID);
		if(is_null($orgUnit))
		{
			$orgUnitPath = '0';
		}
		else
		{
			$orgUnitPath = $orgUnit->Path;
		}
			
		// Gets person by name, surname, eid
		$items = $persons = $assinableWorkers = $resultArray = array();
		$text = trim($q);
		if ($text !== '')
		{
			$filter = new PersonFilter();
			$filter->setFirstname($text)->setSurname($text)->setID($text);
			$persons = $this->context->personRepository->getPersonsByOrFilter($filter);
		}
		else
		{
			$persons = $this->context->personRepository->findAll();
		}
		// Gets assignabel workers
		$assinableWorkers = $taskPlanner->getAssignableWorkers();
		
		//intersects arrays
		foreach($persons as $eid => $person)
		{
			if(isset($assinableWorkers[$eid]))
				$resultArray[$eid] = $person;
		}
		
		
		foreach($resultArray as $worker)
		{
			$workerOrgUnit = $this->context->organizationUnitRepository->find($worker->OrganizationUnitID);
			if($workerOrgUnit->Short != 'Guest') // We dont want assign guests - hardcoded
			{
				$person_item = array();
				if (strpos($workerOrgUnit->path, $orgUnitPath) === 0) // process only if worker belongs under selected org unit.
				{
					$person_item = array('id' => strtoupper(trim($worker->ID)), 'text' => $worker->getReversedFullNameWithEID(), 'disabled' => false);
					$items[] = $person_item;
				}
			}
		}
		$this->payload->items = $items;
		
		$this->terminate();
	}
	
    /**
     * Creates component TaskLockForm.
     * 
     * @param string $name Name of the form.
     * @return \AppForm Instance of the form.
     */
    public function createComponentTaskLockForm($name)
    {
        $form = new AppForm($this, $name);
        
        $form->addHidden('task_id');
		
		$form->addRadioList('AssignmentLock', 'Task lock', array(1 => 'Locked', 0 => 'Unlocked'))
				->setRequired('Please select Assignment Lock!');
        
        $form->addSubmit('save', 'Save');
        
        $form->onSuccess[] = callback($this, 'taskLockFormSubmitted');
        
        return $form;
    }
    
    /**
     * Component TaskLockForm submit method.
     * Locks/Unlocks Task for assignment.
     * 
     * @param AppForm $form TaskLockForm.
     */
    public function taskLockFormSubmitted(AppForm $form)
    {
		if($this->isAllowed(self::RESOURCE_TASK_ASSIGNMENT_LOCK, self::ACTION_EDIT))
		{
			if($form->isValid())
			{
				$values = $form->getValues();
				$task = $this->context->taskRepository->find($values['task_id']);
				if($task)
				{
					$task->Locked = $values['AssignmentLock'];
					if($this->context->taskRepository->update($task))
						$this->flashMessage("Task lock has been updated.", 'success');
					else
						$this->flashMessage("Task lock has not been updated.", 'error');
				}
				else
				{
					$this->flashMessage("Task has not been found.", 'error');
				}
			}
			else
			{
				$this->flashMessage('Invalid form values', 'error');
			}
		}
		else
		{
			$this->flashMessage("You are not authorized lock/unlock task. PMO role required.", 'error');
		}
		$this->redirect('this');
    }
	
    /**
     * Creates component Task assignment trasfer form.
     * 
     * @param string $name Name of the form.
     * @return \AppForm Instance of the form.
     */
    public function createComponentTaskAssignmentTransferForm($name)
    {
        $form = new AppForm($this, $name);
        
//		$form->addCheckbox('DeletePreviousAssignment', HPTForm::createLabel('Delete task previous assignment', 'If checked, tool deletes all previous assignement on targeted task.'));
		$form->addSelect('AssignmentTransfer', 'Transfer assignment to task')
				->setAttribute('style', 'width: 400px;')
				->setRequired('Please select Transfer assignment to Task!');
        
        return $form;
    }
	
	/**
	 * Signal filters tasks for assignment transfer.
	 * 
	 * @param type $q
	 * @param type $year
	 */
	public function handleTaskAssignmentTransferSelect($q, $year)
	{
		// Gets tasks with task plan for given year
		$taskItems = $items = array();
		$text = trim($q);
		$taskWithPlanForYear = $this->context->taskRepository->findLabourTaskWithTaskPlanForYear($year);
		foreach ($taskWithPlanForYear as $task)
		{
			if($text !== '')
			{
				if((stripos($task->Name, $text) !== false) || (stripos($task->SAPID, $text)!== false))
				{
					$taskItems = array('id' => $task->ID, 'text' => $task->Name . ' (' . $task->SAPID . ')', 'disabled' => false);
					$items[] = $taskItems;
				}
			}
			else
			{
				$taskItems = array('id' => $task->ID, 'text' => $task->Name . ' (' . $task->SAPID . ')', 'disabled' => false);
				$items[] = $taskItems;	
			}
		}
		
		$this->payload->items = $items;
		
		$this->terminate();
	}
    
	/**
	 * Signal assign person to task for given year.
	 * 
	 * @param int $fromID From TaskID.
	 * @param int $toID To TaskID.
	 */
	public function handleTaskAssignmentTransferAction($fromID, $toID)
	{
		$fromTask = $this->context->taskRepository->find($fromID);
		$toTask = $this->context->taskRepository->find($toID);
		
		if(is_null($fromTask))
		{
			$this->flashMessage("Task with ID " . $fromID . " was not found. ", 'error');
			$this->redirect('this');
		}
		if(is_null($toTask))
		{
			$this->flashMessage("Task with ID " . $toID . " was not found. ", 'error');
			$this->redirect('this');
		}
		$fromProject = $this->context->projectRepository->findProjectByTaskId($fromTask->ID);
		$toProject = $this->context->projectRepository->findProjectByTaskId($toTask->ID);
		if(is_null($fromProject))
		{
			$this->flashMessage("Task " . $fromTask->Name . " project was not found. ", 'error');
			$this->redirect('this');
		}
		if(is_null($toProject))
		{
			$this->flashMessage("Task " . $toTask->Name . " project was not found. ", 'error');
			$this->redirect('this');
		}
		
		// get task plan editing control object for from task
		$fromTaskPlanEditing = new TaskPlanEditing($fromTask, Date('Y'));
		// get task plan editing control object for to task
		$toTaskPlanEditing = new TaskPlanEditing($toTask, Date('Y'));
		
		if(!$fromTaskPlanEditing->editable)
		{
			$this->flashMessage('Origin task do not allow assignment transfer. ' . $fromTaskPlanEditing->message, 'error');
			$this->redirect('this');
		}
		
		if(!$toTaskPlanEditing->editable)
		{
			$this->flashMessage('Targeted task do not allow assignment transfer. ' . $toTaskPlanEditing->message, 'error');
			$this->redirect('this');
		}
		
		// chcek persons
		$atleastOnePersonTransferred = false;
		$first_updateable_month = FiscalCalendar::currentFiscalMonth() + 1;
		$taskYears = $this->context->taskRepository->findTaskYears($fromID);
		foreach($taskYears as $trasferedYear)
		{
			if($trasferedYear >= Date('Y'))
			{
				if(is_null($this->context->taskPlanRepository->findTaskYearPlan($toTask->ID, $trasferedYear)))
				{
					$task_plan = new TaskYearPlan($trasferedYear);
					$task_plan->TaskID = $toTask->ID;
					$this->context->taskPlanRepository->save($task_plan);
				}
				$assigned_workers = $this->context->personTaskAssignmentRepository->findTaskYearAssignments($fromID, $trasferedYear);
				foreach ($assigned_workers as $eid => $value)
				{
					$person = $this->context->personRepository->find($eid);
					if(is_null($person))
					{
						$this->flashMessage("Person with EID " . $eid . " was not found. ", 'error');
						break;
					}
					$assignmentControl = new AssignmentControl($person, $toTask);
					if(!$assignmentControl->assignable)
					{
						$this->flashMessage($person->ReversedFullNameWithEID . " task assignment was not transferred. " . $assignmentControl->Message, 'error');
					}
					else
					{
						// trasfer person task assignment
						$targetAssignment = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($toID, $trasferedYear, $eid);
						$success = false;
						for($i = 1; $i < 13; $i++)
						{
							if($trasferedYear == Date('Y'))
							{
								if($first_updateable_month <= $i)
								{
									if($value[$i]->hours > 0)
									{
										$targetAssignment[$i]->hours += $value[$i]->hours;
										if($this->context->personTaskAssignmentRepository->save($targetAssignment[$i]))
										{
											$success = true;
										}
									}
								}
							}
							else
							{
								if($value[$i]->hours > 0)
								{
									$targetAssignment[$i]->hours += $value[$i]->hours;
									if($this->context->personTaskAssignmentRepository->save($targetAssignment[$i]))
									{
										$success = true;
									}
								}
							}
						}
						if($success)
						{
							$this->flashMessage($person->ReversedFullNameWithEID . ' task assignment for year: ' . $trasferedYear . ' was transferred successfully.', 'success');
							for($i = 1; $i < 13; $i++)
							{
								if($trasferedYear == Date('Y'))
								{
									if($first_updateable_month <= $i)
									{
										$this->context->personTaskAssignmentRepository->deleteByPersonAndYearAndMonth($eid, $fromID, $trasferedYear, $i);
									}
								}
								else
								{
									$this->context->personTaskAssignmentRepository->deleteByPersonAndYearAndMonth($eid, $fromID, $trasferedYear, $i);
								}
							}
							$atleastOnePersonTransferred = true;
						}
					}
				}
			}
		}
		if($atleastOnePersonTransferred)
			$this->redirect('labor', array('id' => $toID, 'year' => Date('Y')));
		else
		{
			$this->flashMessage('There is no assignment to transfer.', 'warning');
			$this->redirect('labor', array('id' => $fromID, 'year' => Date('Y')));
		}
	}
	
	/**
	 * Check if there are any values bound to this task, e.g.
	 * Plan, Actuals or ActualCosts.
	 * 
	 * I was trying to get this working by using only storeRequest() and restoreRequest()
	 * function calls, but it wasn't working half of the times.
	 */
	private function createNewTaskPlanOrRedirect($task, $taskYear)
	{
		$hasAnyValues = FALSE;
		$reporter = new ProjectEACReporter($this->context, $task, $taskYear);
		$currency = "USD";
		$actualMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index + 1;

		$taskPlan = $this->context->taskPlanRepository->findTaskYearPlan($task->ID, $taskYear);
		if (is_null($taskPlan))
		{
			if ($task->isLabourTask())
			{
				// doesn't have a TaskPlan, has at least some actuals?
				$people = $this->context->workRepository->findPeopleWorkingOnTaskInYear($task->ID, $taskYear);
				if (!empty($people))
					$hasAnyValues = TRUE;
			}
			else
			{
				// doesn't have a TaskPlan, has at least some actualsCosts?
				if ($this->context->costCenterRepository->findActualCostsYearSummary($task->ID, $this->year, 'USD')->sum() != 0)
					$hasAnyValues = TRUE;
			}
		}
		else
		{
			$hasAnyValues = TRUE;
		}
		
		if (!$hasAnyValues)
		{
			$this->flashMessage('There are not any values (plan, actuals) related to this task. '
				. 'Therefore it is empty. '
				. 'You have been redirected.', 'warning');
			
			// redirect
			$prevPresenter = $this->getParameter('prevPresenter');
			if (!empty($prevPresenter))
			{
				$params = explode('+', $this->getParameter('prevPresenter'));
				
				if (substr($this->getParameter('prevPresenter'), 0, 7) === 'project')
				{
					$projectID = $params[1];
					$projectYear = $params[2];
					
					$this->redirect('Project:detail', $projectID, $projectYear);
				}
				elseif (substr($this->getParameter('prevPresenter'), 0, 2) === 'wp')
				{
					$workpackageID = $params[1];
					$workpackageYear = $params[2];
					
					$this->redirect('Workpackage:detail', $workpackageID, $workpackageYear);
				}
				elseif (substr($this->getParameter('prevPresenter'), 0, 8) === 'taskPlan')
				{
					$taskPlanID = $params[1];
					$taskPlanYear = $params[2];
					$taskPlanAction = $params[3];
					
					if ($taskPlanAction === 'nonLabor')
					{
						$this->year = $taskPlanYear;
						$this->redirect('TaskPlan:' . $taskPlanAction, $taskPlanID);
					}
					elseif ($taskPlanAction === 'labor')
					{
						$this->redirect('TaskPlan:' . $taskPlanAction, $taskPlanID, $taskPlanYear);
					}
				}
			}
			
			// default fallback
			$projectID = $this->context->projectRepository->findProjectByTaskID($task->ID)->getID();
			$this->redirect('Project:detail', $projectID);
		}
		else
		{
			if (is_null($this->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year)))
			{
				// create a new one
				$task_plan = new TaskYearPlan($this->year);
				$task_plan->TaskID = $task->ID;
				$this->context->taskPlanRepository->save($task_plan);
				return $task_plan;
			}
		}
	}
}
