<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * PursuitPresenter enables managing of pursuits.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class PursuitPresenter extends SecuredPresenter
{
	/** @var session */
	private $session;
	/** @var array Mandatory items of Pursuit Plan */
	private $mandatoryMilestones = array(
		PursuitPlan::MILESTONE_PNP,
		PursuitPlan::MILESTONE_BNB,
		PursuitPlan::MILESTONE_PRICING,
		PursuitPlan::MILESTONE_GREEN_TEAM_REVIEW,
		PursuitPlan::MILESTONE_RED_TEAM_REVIEW,
		PursuitPlan::MILESTONE_PROPOSAL_SUBMISSION
	);
	/** @const Determining of creation of new pursuit. Should not be a number. */
	const NEW_PURSUIT = 'x';
	/** @const If this expires, we assume the session mutex is no longer acquired.
	 *  Value is in seconds. */
	const AJAX_BEACON_TIMEOUT = 3;
	
	/** 
	 * Inicialization
	 */
	public function startup()
    {
		parent::startup();
		$this->session = $this->getSession('Pursuit');
        $this->session->setExpiration(0);
        if (!isset($this->session->form))
            $this->session->form = array();
		// setting only for pages create and edit
		if (in_array($this->getAction(), array('create', 'edit')))
		{
			if (!isset($this->session->redTeam))
				$this->session->redTeam = array();
			if (!isset($this->session->pursuitRisk))
				$this->session->pursuitRisk = array();
			if (!isset($this->session->pursuitAction))
				$this->session->pursuitAction = array();
		}
	}

	/**
	 * Returns only cutout of programs or calls according to selected customer.
	 * @param string $json_values Values of combo boxes encoded by JSON.stringify()
	 * @return array Creates associative array indexed by name of combo boxes in payload. Each index contains related options.
	 */
	public function actionFilterCustomerProgramCall($json_values)
	{
		$values = json_decode($json_values, TRUE);
		
		if ($values['CustomerID'])
		{
			$related_programs = $this->context->customerProgramRepository->findByCustomer($values['CustomerID']);
			$customer_related_calls = $this->context->customerProgramCallRepository->findByCustomer($values['CustomerID']);
		}
		else
		{
			$related_programs = $customer_related_calls = array();
		}
		
		// Programs
		$programs = $programs_list = array();
		foreach ($related_programs as $program)
		{
			$programs[$program->ProgramID] = $program->Program;
			$programs_list[] = $program->ProgramID;
		}
		
		if (($values['name'] == 'ProgramID') || (in_array($values['ProgramID'], $programs_list)))
		{
			$program_related_calls = $this->context->customerProgramCallRepository->findByProgram($values['ProgramID']);
		}
		
		// Calls
		if (isset($program_related_calls))
			$related_calls = $program_related_calls;
		else
			$related_calls = $customer_related_calls;
		
		$calls = array();
		foreach ($related_calls as $call)
			$calls[$call->CallID] = $call->Call;
		
		$this->payload->customerProgramCallFiltered = array(
			'ProgramID' => $programs,
			'CallID' => $calls
		);
		
		$this->terminate();
	}
	
	/**
	 * Returns only cutout of programs or calls according to selected customer.
	 * @param string $json_values Values of combo boxes encoded by JSON.stringify()
	 * @return array Creates associative array indexed by name of combo boxes in payload. Each index contains related options.
	 */
	public function actionFilterCustomerProgramCallFilter($json_values)
	{
		$values = json_decode($json_values, TRUE);
		
		if ($values['Customer'])
		{
			$related_programs = $this->context->customerProgramRepository->findByCustomer($values['Customer']);
			$customer_related_calls = $this->context->customerProgramCallRepository->findByCustomer($values['Customer']);
		}
		else
		{
			$related_programs = $customer_related_calls = array();
		}
		
		// Programs
		$programs = $programs_list = array();
		foreach ($related_programs as $program)
		{
			$programs[$program->ProgramID] = $program->Program;
			$programs_list[] = $program->ProgramID;
		}
		
		if (($values['name'] == 'Program_ID') || (in_array($values['Program_ID'], $programs_list)))
		{
			$program_related_calls = $this->context->customerProgramCallRepository->findByProgram($values['Program_ID']);
		}
		
		// Calls
		if (isset($program_related_calls))
			$related_calls = $program_related_calls;
		else
			$related_calls = $customer_related_calls;
		
		$calls = array();
		foreach ($related_calls as $call)
			$calls[$call->CallID] = $call->Call;
		
		$this->payload->customerProgramCallFiltered = array(
			'Program_ID' => $programs,
			'Call_ID' => $calls
		);
		
		$this->terminate();
	}
	
	/**
	 * Returns closing date of call belonging to customer program.
	 * @param int $call Should be program call ID.
	 * @return
	 */
	public function actionGetCallClosingDate($call)
	{
		$call = $this->context->customerProgramCallRepository->find($call);
		$date = ($call) ? $call->ClosingDate : NULL;		
		$this->payload->callClosingDate = $date;
		$this->terminate();
	}
	
	/**
	 * Is called when user has open an instance of B&P Pursuit - Create/Edit window
	 * and so holds the mutex.
	 * @return
	 */
	public function actionUserRefreshEditingStatus($removeFlag = FALSE)
	{
		if($this->isAjax())
		{
			if($removeFlag)
			{
				$this->session->creatingOrEditingFlag = NULL;
			}
			else
			{
				// updates last timestamp
				$this->session->creatingOrEditingFlag = time();
			}
			$this->terminate();
		}
	}
	
	/**
     * Handle for auto completion of some fields in the form.
     */
    public function handleAutoComplete()
    {
       $this->payload->autoComplete = array();

       $text = trim($_GET['q']);

       if ($text !== '')
       {
           $filter = new PersonFilter();
           $filter->setFirstname($text)->setSurname($text)->setID($text);
           $persons = $this->context->personRepository->getPersonsByOrFilter($filter);
           foreach ($persons as $person) 
           {
               $person_item = array($person->getReversedFullNameWithEID(), strtoupper(trim($person->ID)), $person->getReversedFullName());
               $this->payload->autoComplete[] = $person_item;
           }
       }
       $this->terminate();                
    }
	
	/**
	 * Common settings for all pages
	 */
	public function beforeRender()
	{
		// pursuit statuses
		$this->template->statuses = Pursuit::getStatusesConstantNames();
	}
	
	/**
	 * Shows page with Pursuits overview.
	 */
	public function renderDefault()
	{
		$this->template->form = $this['filterPursuitForm'];
		$filtered_pursuits = array();
		
		if ($this->isAjax()) // is ajax request
		{
			$filter = new PursuitFilter();
			if (isset($this->payload->filter)) // filter is defined
			{
				$entry = $this->payload->filter; // $entry contains content of filterPursuitForm

				$filter->Name = $entry->Name;
				$filter->PursuitManager = $entry->PursuitManager;
				$filter->PursuitPI = $entry->PursuitPI;
				$filter->ProposalSubmissionAfter = $entry->ProposalSubmissionAfter;
				$filter->ProposalSubmissionBefore = $entry->ProposalSubmissionBefore;
				$statuses = array();
				foreach ($entry->Status as $status)
					$statuses[] = $status;
				$filter->Status = $statuses;
				$thresholds = array();
				foreach ($entry->ProbabilityOfWin as $threshold)
					$thresholds[] = $threshold;
				$filter->ProbabilityOfWin = $thresholds;
				$filter->Customer = $entry->Customer;
				$filter->ProgramID = $entry->Program_ID;
				$filter->CallID = $entry->Call_ID;
			}
			
			$this->context->formStateRepository->save('filterPursuitForm', $this->template->form); // save filter state
			$filtered_pursuits = $this->context->pursuitRepository->findByFilter($filter);			
		}
		else
		{
			if ($this->context->formStateRepository->exists('filterPursuitForm')) // exists saved filter
				$this->context->formStateRepository->load('filterPursuitForm', $this->template->form);
		}
		
		$pursuits = array(); // pursuits to be shown
		foreach ($filtered_pursuits as $pursuit)
		{ 
			$item = array();
			$item['PursuitID'] = $pursuit->PursuitID;
			$item['Name'] = $pursuit->Name;
			$item['Status'] = $pursuit->StatusFullName;
			$item['PursuitManager'] = ($pursuit->PursuitManager) ? $this->context->personRepository->find($pursuit->PursuitManager)->ReversedShortName : '-';
			$item['PursuitManager_Full'] = ($pursuit->PursuitManager) ? $this->context->personRepository->find($pursuit->PursuitManager)->ReversedFullNameWithEID : '-';
			$item['PursuitPI'] = ($pursuit->PursuitPI) ? $this->context->personRepository->find($pursuit->PursuitPI)->ReversedShortName : '-';
			$item['PursuitPI_Full'] = ($pursuit->PursuitPI) ? $this->context->personRepository->find($pursuit->PursuitPI)->ReversedFullNameWithEID : '-';
			$item['Customer'] = ($pursuit->CustomerID) ? $this->context->customerRepository->find($pursuit->CustomerID)->Name : '-';
			    $pursuitPlan = $this->context->pursuitPlanRepository->find($pursuit->PursuitID, PursuitPlan::MILESTONE_PROPOSAL_SUBMISSION);
			$item['ProposalSubmissionDate'] = ($pursuitPlan->PlannedDate) ? $pursuitPlan->PlannedDate : '-';
			
			// Find the nearest upcoming milestone. If Proposal Submission Date(last mandatory milestone) is over or no milestone left, the nearest risk/plan is found.
			$nearestDateArray = array();
			$nearestMilestone = $this->context->pursuitPlanRepository->findNearestByPursuitID($pursuit->PursuitID);
			if($nearestMilestone)
			{
				$date = new DateTime($nearestMilestone->PlannedDate);
				$nearestDateArray[$date->format('U')] = $nearestMilestone->Milestone . ' - ' . $nearestMilestone->PlannedDate;
			}
			
			$nearestRisk = $this->context->pursuitRiskRepository->findNearestByPursuitID($pursuit->PursuitID);
			if($nearestRisk)
			{
				$date = new DateTime($nearestRisk->Deadline);
				$nearestDateArray[$date->format('U')] = 'Risk - ' . $nearestRisk->Deadline;
			}
			
			$nearestAction = $this->context->pursuitActionRepository->findNearestByPursuitID($pursuit->PursuitID);
			if($nearestAction)
			{
				$date = new DateTime($nearestAction->Deadline);
				$nearestDateArray[$date->format('U')] = 'Action - ' . $nearestAction->Deadline;
			}
			if($nearestDateArray)
			{
				ksort($nearestDateArray);
				reset($nearestDateArray);
				$key = key($nearestDateArray);
				$item['NearestMilestone'] = $nearestDateArray[$key];
			}
			else
				$item['NearestMilestone'] = 'None';
			// find if there is missed milestone = milestone planned date which is over and no actual date exist
			// or
			// find if there is missed action/risk = action/risk deadline date which is over and status is not completed
			$item['MissedDates'] = '-';
			$missedDates = array();
			if($this->context->pursuitPlanRepository->existsMissedMilestone($pursuit->PursuitID) 
					|| $this->context->pursuitRiskRepository->existsMissedRisk($pursuit->PursuitID)
					|| $this->context->pursuitActionRepository->existsMissedAction($pursuit->PursuitID))
			{
				$missedPlans = $this->context->pursuitPlanRepository->findMissedByPursuitID($pursuit->PursuitID);
				$missedRisks = $this->context->pursuitRiskRepository->findMissedByPursuitID($pursuit->PursuitID);
				$missedActions = $this->context->pursuitActionRepository->findMissedByPursuitID($pursuit->PursuitID);
				foreach ($missedPlans as $value)
				{
					$missedDates[] = $value->Milestone . ' - ' . $value->PlannedDate;
				}
				foreach ($missedRisks as $value)
				{
					$missedDates[] = 'Risk - ' . $value->Deadline;
				}
				foreach ($missedActions as $value)
				{
					$missedDates[] = 'Action - ' . $value->Deadline;
				}
				$item['MissedDates'] = implode(', ', $missedDates);
				$item['missedMilestone'] = true;
			}
			else
				$item['missedMilestone'] = false;
			
			$item['ProbabilityOfWin'] = $pursuit->ProbabilityOfWinPercentage;
			//Calculate total budget for each pursuit
			$activities = $this->context->itemActivityRepository->findAllActivitiesByPursuitID($pursuit->PursuitID);
			$totalBudget = 0;
			$totalBudgetFromCurrentYear = 0;
			foreach($activities as $activity)
			{
				$activityBudgets = $this->context->itemActivityYearlyBudgetRepository->findAllByID($activity->ID);
				foreach($activityBudgets as $budget)
				{
					$totalBudget += $budget->LaborCosts + $budget->SubcontractingCosts + $budget->TravelCosts + $budget->MaterialCosts;
					if($budget->Year >= Date('Y'))
					{
						$totalBudgetFromCurrentYear += $budget->LaborCosts + $budget->SubcontractingCosts + $budget->TravelCosts + $budget->MaterialCosts;
					}
				}
			}
			$item['TotalBudgetFromCurrentYear'] = $totalBudgetFromCurrentYear;
			$item['TotalBudget'] = $totalBudget;
			$pursuits[] = $item;
		}
		$this->template->pursuits = $pursuits;
		
		if (!$this->hasPermission()) // only some users may create pursuit
		{
			$this->template->form['CreateNew']->setDisabled(TRUE);
		}
	}
	
	/**
	 * Shows page with form for creation of new Pursuit.
	 */
	public function renderCreate()
	{
		if (!$this->hasPermission()) // only some users may create pursuit
		{
			$this->flashMessage('You don\'t have permission to create new Pursuit.', 'warning');
			$this->redirect('Pursuit:default');
		}
		
		$this->creatingOrEditingFlagRedirect();
		
		// get form
		$form = $this['pursuitForm'];
		
		if (count($this->session->form))
		{
			$form->setDefaults($this->session->form);
			$this->session->form = array();
		}
		
		if (!$form['PursuitID']->getValue())
		{
			$this->session->redTeam = array();
			$this->session->pursuitRisk = array();
			$this->session->pursuitAction = array();
			// just in case of creation value is filled by some string because we need to determine moment when to clear session
			$form['PursuitID']->setValue(self::NEW_PURSUIT);
		}
		
		foreach ($this->session->redTeam as $key => $eid)
		{
			$form['redTeam']['EID' . $key]->setDefaultValue($eid);
			    $member = $this->context->personRepository->find($eid);
			$form['redTeam']['EID' . $key . '_info']->setDefaultValue(($member) ? $member->ReversedFullName : NULL);
		}
		
		foreach ($this->session->pursuitRisk as $key => $risk)
		{
			$form['pursuitRisk']['Risk' . $key]->setDefaultValue($risk['Risk']);
			    $owner = $this->context->personRepository->find($risk['Owner']);
			$form['pursuitRisk']['Owner' . $key]->setDefaultValue(($owner) ? $owner->ID : NULL);
			$form['pursuitRisk']['Owner' . $key . '_info']->setDefaultValue(($owner) ? $owner->ReversedFullName : NULL);
			$form['pursuitRisk']['Deadline' . $key]->setDefaultValue($risk['Deadline']);
			$form['pursuitRisk']['Status' . $key]->setDefaultValue($risk['Status']);
			$form['pursuitRisk']['Remarks' . $key]->setDefaultValue($risk['Remarks']);
		}
		
		foreach ($this->session->pursuitAction as $key => $action)
		{
			$form['pursuitAction']['Action' . $key]->setDefaultValue($action['Action']);
			    $owner = $this->context->personRepository->find($action['Owner']);
			$form['pursuitAction']['Owner' . $key]->setDefaultValue(($owner) ? $owner->ID : NULL);
			$form['pursuitAction']['Owner' . $key . '_info']->setDefaultValue(($owner) ? $owner->ReversedFullName : NULL);
			$form['pursuitAction']['Deadline' . $key]->setDefaultValue($action['Deadline']);
			$form['pursuitAction']['Status' . $key]->setDefaultValue($action['Status']);
			$form['pursuitAction']['Remarks' . $key]->setDefaultValue($action['Remarks']);
		}
		
		// set dependencies on customer triplet
		$this->setCustomerDependencies($form);
		
		$this->template->webalizedMilestones = PursuitPlan::getWebalizedMilestones();
		$this->template->mandatoryMilestones = $this->mandatoryMilestones;
		$this->template->redTeam = $this->session->redTeam;
		$this->template->pursuitRisk = $this->session->pursuitRisk;
		$this->template->pursuitAction = $this->session->pursuitAction;
	}
	
	/**
	 * Shows page with form for editation of Pursuit.
	 * @param int $id Should be Pursuit ID.
	 */
	public function renderEdit($id)
	{
		$pursuit = $this->context->pursuitRepository->find($id);
		if ($pursuit === NULL)
		{
			$this->flashMessage('Invalid Pursuit ID. You have been redirected.', 'warning');
            $this->redirect('Pursuit:default');
		}
		
		$this->creatingOrEditingFlagRedirect();
		
		// get forms
		$pursuitForm = $this['pursuitForm'];
		$closePursuitForm = $this['closePursuitForm'];
		
		// set close pursuit form default values
		$closePursuitDefaults = array();
		$closePursuitDefaults['PursuitID'] = $pursuit->PursuitID;
		$closePursuitForm->setDefaults($closePursuitDefaults);
		
		if (count($this->session->form))
		{
			$pursuitForm->setDefaults($this->session->form);
			$this->session->form = array();
		}		
		
		// if id does not exist set default values
		if (!$pursuitForm['PursuitID']->getValue())
		{
			// set general pursuit form default values
			$pursuitDefaults = array();
			$pursuitDefaults['PursuitID'] = $pursuit->PursuitID;
			$pursuitDefaults['CustomerID'] = $pursuit->CustomerID;
			$pursuitDefaults['ProgramID'] = $pursuit->ProgramID;
			$pursuitDefaults['CallID'] = $pursuit->CallID;
			$pursuitDefaults['CostShare'] = $pursuit->CostSharePercentage;
			$pursuitDefaults['Name'] = $pursuit->Name;
			$pursuitDefaults['Status'] = $pursuit->Status;
			$pursuitDefaults['SFDCNumber'] = $pursuit->SFDCNumber;
			$pursuitDefaults['PursuitManager'] = $pursuit->PursuitManager;
			$pursuitDefaults['PursuitManager_info'] = ($pursuit->PursuitManager) ? $this->context->personRepository->find($pursuit->PursuitManager)->ReversedFullName : NULL;
			$pursuitDefaults['PursuitPI'] = $pursuit->PursuitPI;
			$pursuitDefaults['PursuitPI_info'] = ($pursuit->PursuitPI) ? $this->context->personRepository->find($pursuit->PursuitPI)->ReversedFullName : NULL;
			$pursuitDefaults['Remarks'] = $pursuit->Remarks;
			$pursuitDefaults['CreatedDate'] = $pursuit->CreatedDate;
			$pursuitDefaults['LastUpdatedDate'] = $pursuit->LastUpdatedDate;
			$pursuitDefaults['ProbabilityOfWin'] = $pursuit->ProbabilityOfWinPercentage;
			$pursuitDefaults['DocumentStorage'] = $pursuit->DocumentStorage;

			// Pursuit Plan
			$milestones = PursuitPlan::getWebalizedMilestones();
			$all_pursuitPlans = $this->context->pursuitPlanRepository->findByPursuitID($pursuit->PursuitID);
			foreach ($all_pursuitPlans as $pursuitPlan)
			{
				$pursuitDefaults['pursuitPlan'][$milestones[$pursuitPlan->Milestone] . 'Planned'] = $pursuitPlan->PlannedDate;
				$pursuitDefaults['pursuitPlan'][$milestones[$pursuitPlan->Milestone] . 'Actual'] = $pursuitPlan->ActualDate;
				$pursuitDefaults['pursuitPlan'][$milestones[$pursuitPlan->Milestone] . 'Link'] = $pursuitPlan->Link;
				$pursuitDefaults['pursuitPlan'][$milestones[$pursuitPlan->Milestone] . 'BnPPlan'] = $pursuitPlan->BnPPlan;
				$pursuitDefaults['pursuitPlan'][$milestones[$pursuitPlan->Milestone] . 'BnPActuals'] = $pursuitPlan->BnPActuals;
				$pursuitDefaults['pursuitPlan'][$milestones[$pursuitPlan->Milestone] . 'Remarks'] = $pursuitPlan->Remarks;
			}
			
			// RED Team
			$all_redTeam = $this->context->redTeamRepository->findByPursuitID($pursuit->PursuitID);
			$this->session->redTeam = array();
			foreach ($all_redTeam as $redTeam)
			{
				$this->session->redTeam[] = $redTeam->ID;
			}
			
			// Pursuit Risk
			$all_pursuitRisk = $this->context->pursuitRiskRepository->findByPursuitID($pursuit->PursuitID);
			$arrPursuitRisk = array();
			foreach ($all_pursuitRisk as $pursuitRisk)
			{
				$risk = array();
				$risk['RiskID'] = $pursuitRisk->ID;
				$risk['Risk'] = $pursuitRisk->Risk;
				$risk['Owner'] = $pursuitRisk->Owner;
				$risk['Deadline'] = $pursuitRisk->Deadline;
				$risk['Status'] = $pursuitRisk->Status;
				$risk['Remarks'] = $pursuitRisk->Remarks;
				$arrPursuitRisk[] = $risk;
			}
			$this->session->pursuitRisk = $arrPursuitRisk;
			
			// Pursuit Action
			$all_pursuitAction = $this->context->pursuitActionRepository->findByPursuitID($pursuit->PursuitID);
			$arrPursuitAction = array();
			foreach ($all_pursuitAction as $pursuitAction)
			{
				$action = array();
				$action['ActionID'] = $pursuitAction->ID;
				$action['Action'] = $pursuitAction->Action;
				$action['Owner'] = $pursuitAction->Owner;
				$action['Deadline'] = $pursuitAction->Deadline;
				$action['Status'] = $pursuitAction->Status;
				$action['Remarks'] = $pursuitAction->Remarks;
				$arrPursuitAction[] = $action;
			}
			$this->session->pursuitAction = $arrPursuitAction;
			
			// important to get form fields according to loaded data 
			$this->session->form = $pursuitDefaults;
			$this->redirect('this');			
		}
		
		foreach ($this->session->redTeam as $key => $eid)
		{
			$pursuitForm['redTeam']['EID' . $key]->setDefaultValue($eid);
			    $member = $this->context->personRepository->find($eid);
			$pursuitForm['redTeam']['EID' . $key . '_info']->setDefaultValue(($member) ? $member->ReversedFullName : NULL);
		}
		
		foreach ($this->session->pursuitRisk as $key => $risk)
		{
			if (isset($risk['RiskID']))
			    $pursuitForm['pursuitRisk']['RiskID' . $key]->setDefaultValue($risk['RiskID']);
			$pursuitForm['pursuitRisk']['Risk' . $key]->setDefaultValue($risk['Risk']);
			    $owner = $this->context->personRepository->find($risk['Owner']);
			$pursuitForm['pursuitRisk']['Owner' . $key]->setDefaultValue(($owner) ? $owner->ID : NULL);
			$pursuitForm['pursuitRisk']['Owner' . $key . '_info']->setDefaultValue(($owner) ? $owner->ReversedFullName : NULL);
			$pursuitForm['pursuitRisk']['Deadline' . $key]->setDefaultValue($risk['Deadline']);
			$pursuitForm['pursuitRisk']['Status' . $key]->setDefaultValue($risk['Status']);
			$pursuitForm['pursuitRisk']['Remarks' . $key]->setDefaultValue($risk['Remarks']);
		}
		
		foreach ($this->session->pursuitAction as $key => $action)
		{
			if (isset($action['ActionID']))
			    $pursuitForm['pursuitAction']['ActionID' . $key]->setDefaultValue($action['ActionID']);
			$pursuitForm['pursuitAction']['Action' . $key]->setDefaultValue($action['Action']);
			$owner = $this->context->personRepository->find($action['Owner']);
			$pursuitForm['pursuitAction']['Owner' . $key]->setDefaultValue(($owner) ? $owner->ID : NULL);
			$pursuitForm['pursuitAction']['Owner' . $key . '_info']->setDefaultValue(($owner) ? $owner->ReversedFullName : NULL);
			$pursuitForm['pursuitAction']['Deadline' . $key]->setDefaultValue($action['Deadline']);
			$pursuitForm['pursuitAction']['Status' . $key]->setDefaultValue($action['Status']);
			$pursuitForm['pursuitAction']['Remarks' . $key]->setDefaultValue($action['Remarks']);
		}
		
		// set call closing date if exists
		$call = $this->context->customerProgramCallRepository->find($pursuit->CallID);
		$date = ($call) ? $call->ClosingDate : NULL;
		if ($date)
			$this->template->callClosingDate = $date;
		
		// if status == Committed, then lock probability of win
		if ($pursuitForm['Status']->getValue() == Pursuit::PURSUIT_STATUS_COMMITTED)
		{
			$pursuitForm['ProbabilityOfWin']
			        ->setAttribute('readonly', 'readonly');
		}
		
		// only some users may update pursuit
		if (!$this->hasPermission())
		{
			$pursuitForm['Save']->setDisabled(TRUE);
			$pursuitForm['Close']->setDisabled(TRUE);
			$closePursuitForm['Close']->setDisabled(TRUE);
			$this->template->isReadonly = TRUE;
		}
		
		// set dependencies on customer triplet
		$this->setCustomerDependencies($pursuitForm);
		
		$this->template->webalizedMilestones = PursuitPlan::getWebalizedMilestones();
		$this->template->mandatoryMilestones = $this->mandatoryMilestones;
		$this->template->redTeam = $this->session->redTeam;
		$this->template->pursuitRisk = $this->session->pursuitRisk;
		$this->template->pursuitAction = $this->session->pursuitAction;
		    $editor = $this->context->personRepository->find($pursuit->UpdatedBy);
		$this->template->updatedBy = ($editor) ? $editor->ReversedFullNameWithEID : NULL;
		$this->template->date = $pursuit->LastUpdatedDate;
		
		// load all strap item activities connected to current pursuit
		$this->template->itemActivities = $this->context->itemActivityRepository->findAllActivitiesByPursuitID($pursuit->PursuitID);
		
		// pursuit can not be updated anymore
		if ($pursuit->Status == Pursuit::PURSUIT_STATUS_CLOSED)
		{
			// pursuit closed with result OnHold means pursuit is frozen only
			if ($pursuit->ClosureResult == Pursuit::PURSUIT_CLOSURE_RESULT_ON_HOLD)
			{
				$pursuitForm['Status']->setPrompt('Closed with result: ' . $pursuit->ClosureResultFullName);
				return; // pursuit can not be locked by following lines
			}
			
			$pursuitForm['Save']->setDisabled(TRUE);
			$pursuitForm['Close']->setDisabled(TRUE);
			$closePursuitForm['Close']->setDisabled(TRUE);
			$this->template->isClosed = TRUE;
			$this->template->closureResult = $pursuit->ClosureResultFullName;
		}
	}
	
	/**
	 * Creates Pursuit filter form.
	 * @param string $name Name of component.
	 * @return \AppForm
	 */
	public function createComponentFilterPursuitForm($name)
	{
		$form = new AppForm($this, $name);
		$form->getElementPrototype()->class('ajax');
		$dateFormat = 'YYYY-MM-DD';
		$dateFormatRegexp = '[0-9]{4}-[0-1][0-9]-[0-3][0-9]';
		
		
		$form->addText('Name', 'Name', 30, 200);
		
		$statuses = array('' => '*');
		foreach (Pursuit::getStatusesFullNames() as $status => $name)
			$statuses[$status] = $name;
		$form->addMultiSelect('Status', 'Status', $statuses, 6)
				->setDefaultValue('')
				->setAttribute('style', 'width: 170px;');
		
		$form->addText('PursuitManager', 'Pursuit Manager', 7)
				->setAttribute('class', 'autocomplete');
			   
        $form->addText('PursuitManager_info', NULL, 20)
                ->setAttribute('readonly', 'readonly');
		
		$form->addText('PursuitPI', 'Pursuit Investigator', 7)
				->setAttribute('class', 'autocomplete');
			   
        $form->addText('PursuitPI_info', NULL, 20)
                ->setAttribute('readonly', 'readonly');
		
		$form->addText('ProposalSubmissionAfter', 'Proposal Submission After')
				->addCondition(Form::FILLED)
				->addRule(Form::PATTERN, 'Proposal Submission After must be in valid format ' . $dateFormat . '.', $dateFormatRegexp);
				$form['ProposalSubmissionAfter']
				        ->setAttribute('class', 'datepicker');
		
		$form->addText('ProposalSubmissionBefore', 'Proposal Submission Before')
				->addCondition(Form::FILLED)
				->addRule(Form::PATTERN, 'Proposal Submission Before must be in valid format ' . $dateFormat . '.', $dateFormatRegexp);
		        $form['ProposalSubmissionBefore']
				        ->setAttribute('class', 'datepicker');
		
		$thresholds = array('' => '*');
		foreach (Pursuit::getProbabilityOfWinThresholdsFullNames() as $threshold => $name)
			$thresholds[$threshold] = $name;
		$form->addMultiSelect('ProbabilityOfWin', 'Probability of Win (%)', $thresholds, 4)
				->setDefaultValue('')
				->setAttribute('style', 'width: 170px;');
		
		/** 
		 * Program_ID and Call_ID are dynamically related with Customer.
		 * Each customer has some related programs and each program has some related calls.
		 * Here are fetched all of programs as well as all calls, but displayed items are
		 * filtered in render method according to current selection.
		 * It is simpler to obtain values from form if there are fetched all items.
		 */
        $all_customers = $this->context->customerRepository->findAll();
        $customers = array('' => '*');
        foreach ($all_customers as $customer)
            $customers[$customer->ID] = $customer->Name;
        $form->addSelect('Customer', 'Customer', $customers);
		
		$all_customerPrograms = $this->context->customerProgramRepository->findAll();
		$customerPrograms = array('' => '*');
		foreach ($all_customerPrograms as $program)
			$customerPrograms[$program->ProgramID] = $program->Program;
        $form->addSelect('Program_ID', 'Program', $customerPrograms)
				->setAttribute('style', 'width: 201px;');
		
		$all_customerProgramCalls = $this->context->customerProgramCallRepository->findAll();
		$customerProgramCalls = array('' => '*');
		foreach ($all_customerProgramCalls as $call)
			$customerProgramCalls[$call->CallID] = $call->Call;
        $form->addSelect('Call_ID', 'Call', $customerProgramCalls)
				->setAttribute('style', 'width: 201px;');
				
		$form->addSubmit('Search', 'Search')
				->onClick[] = array($this, 'searchFilterPursuitFormSubmitted');
		
		$form->addSubmit('Reset', 'Reset Filter')
				->onClick[] = array($this, 'resetFilterPursuitFormSubmitted');
		
		$form->addSubmit('CreateNew', 'Create New')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'createNewFilterPursuitFormSubmitted');
		
		$form->onSuccess[] = array($this, 'filterPursuitFormSubmitted');
				
		return $form;
	}
	
	/**
	 * Processing of pursuit filter form submitted by ajax.
	 * @param AppForm $form
	 */
	public function filterPursuitFormSubmitted(AppForm $form)
	{		
		$this->payload->filter = $form->getValues();
		$this->invalidateControl('pursuits');
	}
	
	/**
	 * Processing of pursuit filter form submitted by search button.
	 * @param SubmitButton $button
	 */
	public function searchFilterPursuitFormSubmitted(SubmitButton $button)
	{		
		$this->payload->filter = $button->form->getValues();
		$this->invalidateControl('pursuits');
	}
	
	/**
	 * Processing of pursuit filter form submitted by reset button.
	 * @param SubmitButton $button
	 */
	public function resetFilterPursuitFormSubmitted(SubmitButton $button)
	{
		$this->context->formStateRepository->drop('filterPursuitForm');
		$this->redirect('this');
	}
	
	/**
	 * Processing of pursuit filter form submitted by create new button.
	 * @param SubmitButton $button
	 */
	public function createNewFilterPursuitFormSubmitted(SubmitButton $button)
	{
		$this->redirect('Pursuit:create');
	}
	
	/**
	 * Creates general Pursuit form.
	 * @param string $name Name of component.
	 * @return \AppForm
	 */
	public function createComponentPursuitForm($name)
	{
		$form = new AppForm($this, $name);
		$dateFormat = 'YYYY-MM-DD';
		$dateFormatRegexp = '[0-9]{4}-[0-1][0-9]-[0-3][0-9]';
		
		
		$form->addHidden('PursuitID');
		
		$form->addText('Name', 'Name', 30, 200)
				->addRule(Form::FILLED, 'Name must be filled.');
		
		$statuses = Pursuit::getStatusesFullNames();
		unset($statuses[Pursuit::PURSUIT_STATUS_CLOSED]); // there is only one way to close pursuit => by close form with mandatory reason of closure
		$form->addSelect('Status', 'Status', $statuses)
				->setPrompt('')
				->addRule(Form::FILLED, 'Status must be filled.')
				->setAttribute('style', 'width: 200px;');
		
		$form->addTextArea('Remarks', 'Remarks', 30, 5);
		
		$form->addText('PursuitManager', 'Pursuit Manager', 7)
				->addRule(Form::FILLED, 'Pursuit Manager must be filled.')
				->setAttribute('class', 'autocomplete');
			   
        $form->addText('PursuitManager_info', NULL, 20)
                ->setAttribute('readonly', 'readonly');
		
		$form->addText('PursuitPI', 'Pursuit Investigator', 7)
				->addRule(Form::FILLED, 'Pursuit Investigator must be filled.')
				->setAttribute('class', 'autocomplete');
			   
        $form->addText('PursuitPI_info', NULL, 20)
                ->setAttribute('readonly', 'readonly');
		
		$form->addText('SFDCNumber', 'SFDC Number', NULL, 10);
		
		/** 
		 * ProgramID and CallID are dynamically related with CustomerID.
		 * Each customer has some related programs and each program has some related calls.
		 * Here are fetched all of programs as well as all calls, but displayed items are
		 * filtered in render method according to current selection.
		 * It is simpler to obtain values from form if there are fetched all items.
		 */
        $all_customers = $this->context->customerRepository->findAll();
        $customers = array();
        foreach ($all_customers as $customer)
            $customers[$customer->ID] = $customer->Name;
        $form->addSelect('CustomerID', 'Customer', $customers)
				->setPrompt('')
				->addRule(Form::FILLED, 'Customer must be filled.');
		
		$all_customerPrograms = $this->context->customerProgramRepository->findAll();
		$customerPrograms = array();
		foreach ($all_customerPrograms as $program)
			$customerPrograms[$program->ProgramID] = $program->Program;
        $form->addSelect('ProgramID', 'Program', $customerPrograms)
				->setPrompt('')
				->setAttribute('style', 'width: 200px;');
		
		$all_customerProgramCalls = $this->context->customerProgramCallRepository->findAll();
		$customerProgramCalls = array();
		foreach ($all_customerProgramCalls as $call)
			$customerProgramCalls[$call->CallID] = $call->Call;
        $form->addSelect('CallID', 'Call', $customerProgramCalls)
				->setPrompt('')
				->setAttribute('style', 'width: 200px;');
		
		$form->addText('CostShare', 'Cost Share (%)', 5, 3)
				->addCondition(Form::FILLED)
				->addRule(Form::PATTERN, 'Cost Share must be in range <0;100>.', '[0-9]{1,2}|100');
		
		$form->addText('ProbabilityOfWin', 'Probability of Win (%)', 5, 3)
				->addCondition(Form::FILLED)
				->addRule(Form::PATTERN, 'Probability of Win must be in range <0;100>.', '[0-9]{1,2}|100');
		
		$form->addText('DocumentStorage', 'Document Storage', NULL, 200)
				->setAttribute('class', 'dynamicLink');
		
		$form->addText('CreatedDate', 'Created Date')
				->setAttribute('readonly', 'readonly');
		
		$form->addText('LastUpdatedDate', 'Last Updated')->setAttribute('readonly', 'readonly');
		
		// Pursuit Plan
		$form_pursuitPlan = $form->addContainer('pursuitPlan');
		foreach (PursuitPlan::getWebalizedMilestones() as $key => $milestone)
		{
			$form_pursuitPlan->addText($milestone . 'Planned', NULL, 12)
				->addCondition(Form::FILLED)
				->addRule(Form::PATTERN, $key . ' Planned Date must be in valid format ' . $dateFormat . '.', $dateFormatRegexp);
				$form_pursuitPlan[$milestone . 'Planned']
					->setAttribute('class', 'datepicker');

			// those milestones are mandatory
			if (in_array($key, $this->mandatoryMilestones))
			{
				$form_pursuitPlan[$milestone . 'Planned']
						->addRule(Form::FILLED, $key . ' Planned Date must be filled.');
			}
			
			$form_pursuitPlan->addText($milestone . 'Actual', NULL, 12)
				->addCondition(Form::FILLED)
				->addRule(Form::PATTERN, $key . ' Actual Date must be in valid format ' . $dateFormat . '.', $dateFormatRegexp);
				$form_pursuitPlan[$milestone . 'Actual']
					->setAttribute('class', 'datepicker');
			
			$form_pursuitPlan->addText($milestone . 'Link', NULL, 20, 200)
					->setAttribute('class', 'dynamicLink');
			
			$form_pursuitPlan->addText($milestone . 'BnPPlan', NULL, 12);
			
			$form_pursuitPlan->addText($milestone . 'BnPActuals', NULL, 12);
			
			$form_pursuitPlan->addTextArea($milestone . 'Remarks', NULL, 30, 3);
		}
		
		
		// RED Team
		$form_redTeam = $form->addContainer('redTeam');
		$redTeamKeys = array_keys($this->session->redTeam);
	    foreach ($redTeamKeys as $key)
		{
			$form_redTeam->addText('EID' . $key, NULL, 7)
					->addRule(Form::FILLED, 'Member of RED Team must be filled.')
					->setAttribute('class', 'autocomplete');
			   
			$form_redTeam->addText('EID' . $key . '_info', NULL, 20)
					->setAttribute('readonly', 'readonly');
			
			$form_redTeam->addButton('Remove' . $key, 'Remove')
					->setAttribute('onclick', 'removeRedTeam(' . $key . ')');
		}
		$form_redTeam->addText('EID', NULL, 7)
				->setAttribute('class', 'autocomplete');
			   
		$form_redTeam->addText('EID_info', NULL, 20)
				->setAttribute('readonly', 'readonly');
		
		$form_redTeam->addSubmit('AddRedTeam', 'Add')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'addRedTeamPursuitFormSubmitted');
		
		$form_redTeam->addHidden('MemberToRemove');
		
		$form_redTeam->addSubmit('RemoveRedTeam', 'Remove')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'removeRedTeamPursuitFormSubmitted');
		        $form_redTeam['RemoveRedTeam']
		                ->setAttribute('style', 'display: none;');
		
		
		// Pursuit Risk
		$form_pursuitRisk = $form->addContainer('pursuitRisk');
		$pursuitRiskKeys = array_keys($this->session->pursuitRisk);
		foreach ($pursuitRiskKeys as $key)
		{
			$form_pursuitRisk->addHidden('RiskID' . $key);
					
			$form_pursuitRisk->addTextArea('Risk' . $key, NULL, 30, 3)
					->addRule(Form::FILLED, 'Risk must be filled.');
			
			$form_pursuitRisk->addText('Owner' . $key, NULL, 7)
					->addRule(Form::FILLED, 'Owner of Pursuit Risk must be filled.')
					->setAttribute('class', 'autocomplete');
			   
			$form_pursuitRisk->addText('Owner' . $key . '_info', NULL, 20)
					->setAttribute('readonly', 'readonly');
			
			$form_pursuitRisk->addText('Deadline' . $key, NULL, 12)
					->addCondition(Form::FILLED)
					->addRule(Form::PATTERN, 'Risk Deadline Date must be in valid format ' . $dateFormat . '.', $dateFormatRegexp);
					$form_pursuitRisk['Deadline' . $key]
						->setAttribute('class', 'datepicker');
					
		    $form_pursuitRisk->addSelect('Status' . $key)
				->setItems(PursuitRisk::getStatuses())
				->setPrompt('- Choose -');
			
			$form_pursuitRisk->addTextArea('Remarks' . $key, NULL, 30, 3);
			
			$form_pursuitRisk->addButton('Remove' . $key, 'Remove')
					->setAttribute('onclick', 'removePursuitRisk(' . $key . ')');
		}
		$form_pursuitRisk->addTextArea('Risk', NULL, 30, 3);
		
		$form_pursuitRisk->addText('Owner', NULL, 7)
				->setAttribute('class', 'autocomplete');
			   
		$form_pursuitRisk->addText('Owner_info', NULL, 20)
				->setAttribute('readonly', 'readonly');
		
		$form_pursuitRisk->addText('Deadline', NULL, 12)
					->setAttribute('class', 'datepicker');
		
		$form_pursuitRisk->addSelect('Status')
				->setItems(PursuitRisk::getStatuses())
				->setPrompt('- Choose -');
		
		$form_pursuitRisk->addTextArea('Remarks', NULL, 30, 3);
		
		$form_pursuitRisk->addSubmit('AddPursuitRisk', 'Add')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'addPursuitRiskPursuitFormSubmitted');
		
		$form_pursuitRisk->addHidden('PursuitRiskToRemove');
		
		$form_pursuitRisk->addSubmit('RemovePursuitRisk', 'Remove')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'removePursuitRiskPursuitFormSubmitted');
		        $form_pursuitRisk['RemovePursuitRisk']
		                ->setAttribute('style', 'display: none;');
		
				
		// Pursuit Action
		$form_pursuitAction = $form->addContainer('pursuitAction');
		$pursuitActionKeys = array_keys($this->session->pursuitAction);
		foreach ($pursuitActionKeys as $key)
		{
			$form_pursuitAction->addHidden('ActionID' . $key);
					
			$form_pursuitAction->addTextArea('Action' . $key, NULL, 30, 3)
					->addRule(Form::FILLED, 'Action must be filled.');
			
			$form_pursuitAction->addText('Owner' . $key, NULL, 7)
					->addRule(Form::FILLED, 'Owner of Pursuit Action must be filled.')
					->setAttribute('class', 'autocomplete');
			   
			$form_pursuitAction->addText('Owner' . $key . '_info', NULL, 20)
					->setAttribute('readonly', 'readonly');
			
			$form_pursuitAction->addText('Deadline' . $key, NULL, 12)
					->addCondition(Form::FILLED)
					->addRule(Form::PATTERN, 'Action Deadline Date must be in valid format ' . $dateFormat . '.', $dateFormatRegexp);
					$form_pursuitAction['Deadline' . $key]
						->setAttribute('class', 'datepicker');
					
		    $form_pursuitAction->addSelect('Status' . $key)
					->setItems(PursuitAction::getStatuses())
					->setPrompt('- Choose -');
			
			$form_pursuitAction->addTextArea('Remarks' . $key, NULL, 30, 3);
			
			$form_pursuitAction->addButton('Remove' . $key, 'Remove')
					->setAttribute('onclick', 'removePursuitAction(' . $key . ')');
		}
		$form_pursuitAction->addTextArea('Action', NULL, 30, 3);
		
		$form_pursuitAction->addText('Owner', NULL, 7)
				->setAttribute('class', 'autocomplete');

		$form_pursuitAction->addText('Owner_info', NULL, 20)
				->setAttribute('readonly', 'readonly');
		
		$form_pursuitAction->addText('Deadline', NULL, 12)
					->setAttribute('class', 'datepicker');
		
		$form_pursuitAction->addSelect('Status')
				->setItems(PursuitAction::getStatuses())
				->setPrompt('- Choose -');
		
		$form_pursuitAction->addTextArea('Remarks', NULL, 30, 3);
		
		$form_pursuitAction->addSubmit('AddPursuitAction', 'Add')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'addPursuitActionPursuitFormSubmitted');
		
		$form_pursuitAction->addHidden('PursuitActionToRemove');
		
		$form_pursuitAction->addSubmit('RemovePursuitAction', 'Remove')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'removePursuitActionPursuitFormSubmitted');
		        $form_pursuitAction['RemovePursuitAction']
		                ->setAttribute('style', 'display: none;');
		
		
		$form->addSubmit('Save', 'Save')
				->onClick[] = array($this, 'savePursuitFormSubmitted');
		
		$form->addSubmit('Back', 'Back to pursuits')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'backPursuitFormSubmitted');
		
		$form->addButton('Close', 'Close') // only shows overlay window with another part of the form.
				->setAttribute('onclick', '$.fancybox({content: $("#close").html()});');
		
		return $form;
	}

	/**
	 * Redirects user if he tries to do more things (create B&P Pursuit, edit existing)
	 * simultaneously. That way those tabs won't interfere and break session information,
	 * for instance Pursuit Risk. Better way of doing this would be using HTML5's sessionStorage via
	 * Javascript to store all data in a form on client's side.
	 * @return
	 */
	private function creatingOrEditingFlagRedirect()
	{
		// limit STRAP B&P - Create/Edit instance to only one
		if(!empty($this->session->creatingOrEditingFlag) && (time() - $this->session->creatingOrEditingFlag) < self::AJAX_BEACON_TIMEOUT)
		{
			$this->flashMessage('You can create or edit only one B&P Pursuit at a time. We apologize for any inconvenience caused.', 'warning');
			$this->redirect('Pursuit:default');
		}
	}

	/**
	 * Processing of pursuit form submitted by addRedTeam button.
	 * @param SubmitButton $button
	 */
	public function addRedTeamPursuitFormSubmitted(SubmitButton $button)
	{
		if (!$this->hasPermission())
		{
			$this->flashMessage('You don\'t have permission to modify Pursuits.', 'warning');
			$this->redirect('Pursuit:default');
		}
		
		$entry = $button->form->getValues();
		
		if ($entry->redTeam->EID)
		{			
			$exists = FALSE;
			foreach ($this->session->redTeam as $eid)
			{
				if ($eid == $entry->redTeam->EID)
				{
					$exists = TRUE;
					break;
				}
			}
			
			if (!$exists)
			{
				$keys = array_keys($this->session->redTeam);
				$lastIndex = end($keys);
				$newIndex = ($lastIndex !== FALSE) ? $lastIndex + 1 : 0;
				$this->session->redTeam[$newIndex] = $entry->redTeam->EID;
			}
		}
		
		unset($entry->redTeam->EID);
		unset($entry->redTeam->EID_info);
		$this->session->form = $entry;
		$this->redirect('this#redTeam');
	}
	
	/**
	 * Processing of pursuit form submitted by removeRedTeam button.
	 * @param SubmitButton $button
	 */
	public function removeRedTeamPursuitFormSubmitted(SubmitButton $button)
	{
		if (!$this->hasPermission())
		{
			$this->flashMessage('You don\'t have permission to modify Pursuits.', 'warning');
			$this->redirect('Pursuit:default');
		}
		
		$entry = $button->form->getValues();
		
		if ($entry->redTeam->MemberToRemove != '')
		{
			unset($this->session->redTeam[$entry->redTeam->MemberToRemove]);
		}
		
		unset($entry->redTeam->MemberToRemove);
		$this->session->form = $entry;
		$this->redirect('this#redTeam');
	}
	
	/**
	 * Processing of pursuit form submitted by addPursuitRisk button.
	 * @param SubmitButton $button
	 */
	public function addPursuitRiskPursuitFormSubmitted(SubmitButton $button)
	{
		if (!$this->hasPermission())
		{
			$this->flashMessage('You don\'t have permission to modify Pursuits.', 'warning');
			$this->redirect('Pursuit:default');
		}
		
		$entry = $button->form->getValues();
		
		if ($entry->pursuitRisk->Risk || $entry->pursuitRisk->Deadline || $entry->pursuitRisk->Status || $entry->pursuitRisk->Remarks)
		{
			$keys = array_keys($this->session->pursuitRisk);
			$lastIndex = end($keys);
			$newIndex = ($lastIndex !== FALSE) ? $lastIndex + 1 : 0;
			$this->session->pursuitRisk[$newIndex]['Risk'] = $entry->pursuitRisk->Risk;
			    $owner = $this->context->personRepository->find($entry->pursuitRisk->Owner);
			$this->session->pursuitRisk[$newIndex]['Owner'] = ($owner) ? $owner->ID : NULL;
			$this->session->pursuitRisk[$newIndex]['Deadline'] = $entry->pursuitRisk->Deadline ? $entry->pursuitRisk->Deadline : NULL;
			$this->session->pursuitRisk[$newIndex]['Status'] = $entry->pursuitRisk->Status ? $entry->pursuitRisk->Status : NULL;
			$this->session->pursuitRisk[$newIndex]['Remarks'] = $entry->pursuitRisk->Remarks ? $entry->pursuitRisk->Remarks : NULL;
		}
		
		unset($entry->pursuitRisk->Risk);
		unset($entry->pursuitRisk->Owner);
		unset($entry->pursuitRisk->Owner_info);
		unset($entry->pursuitRisk->Deadline);
		unset($entry->pursuitRisk->Status);
		unset($entry->pursuitRisk->Remarks);
		$this->session->form = $entry;
		$this->redirect('this#pursuitRisk');
	}
	
	/**
	 * Processing of pursuit form submitted by removePursuitRisk button.
	 * @param SubmitButton $button
	 */
	public function removePursuitRiskPursuitFormSubmitted(SubmitButton $button)
	{
		if (!$this->hasPermission())
		{
			$this->flashMessage('You don\'t have permission to modify Pursuits.', 'warning');
			$this->redirect('Pursuit:default');
		}
		
		$entry = $button->form->getValues();
		
		if ($entry->pursuitRisk->PursuitRiskToRemove != '')
		{
			unset($this->session->pursuitRisk[$entry->pursuitRisk->PursuitRiskToRemove]);
		}
		
		unset($entry->pursuitRisk->PursuitRiskToRemove);
		$this->session->form = $entry;
		$this->redirect('this#pursuitRisk');
	}
	
	/**
	 * Processing of pursuit form submitted by addPursuitAction button.
	 * @param SubmitButton $button
	 */
	public function addPursuitActionPursuitFormSubmitted(SubmitButton $button)
	{
		if (!$this->hasPermission())
		{
			$this->flashMessage('You don\'t have permission to modify Pursuits.', 'warning');
			$this->redirect('Pursuit:default');
		}
		
		$entry = $button->form->getValues();
		
		if ($entry->pursuitAction->Action || $entry->pursuitAction->Deadline || $entry->pursuitAction->Status || $entry->pursuitAction->Remarks)
		{
			$keys = array_keys($this->session->pursuitAction);
			$lastIndex = end($keys);
			$newIndex = ($lastIndex !== FALSE) ? $lastIndex + 1 : 0;
			$this->session->pursuitAction[$newIndex]['Action'] = $entry->pursuitAction->Action;
			    $owner = $this->context->personRepository->find($entry->pursuitAction->Owner);
			$this->session->pursuitAction[$newIndex]['Owner'] = ($owner) ? $owner->ID : NULL;
			$this->session->pursuitAction[$newIndex]['Deadline'] = $entry->pursuitAction->Deadline ? $entry->pursuitAction->Deadline : NULL;
			$this->session->pursuitAction[$newIndex]['Status'] = $entry->pursuitAction->Status ? $entry->pursuitAction->Status : NULL;
			$this->session->pursuitAction[$newIndex]['Remarks'] = $entry->pursuitAction->Remarks ? $entry->pursuitAction->Remarks : NULL;
		}
		
		unset($entry->pursuitAction->Action);
		unset($entry->pursuitAction->Owner);
		unset($entry->pursuitAction->Owner_info);
		unset($entry->pursuitAction->Deadline);
		unset($entry->pursuitAction->Status);
		unset($entry->pursuitAction->Remarks);
		$this->session->form = $entry;
		$this->redirect('this#pursuitAction');
	}
	
	/**
	 * Processing of pursuit form submitted by removePursuitAction button.
	 * @param SubmitButton $button
	 */
	public function removePursuitActionPursuitFormSubmitted(SubmitButton $button)
	{
		if (!$this->hasPermission())
		{
			$this->flashMessage('You don\'t have permission to modify Pursuits.', 'warning');
			$this->redirect('Pursuit:default');
		}
		
		$entry = $button->form->getValues();
		
		if ($entry->pursuitAction->PursuitActionToRemove != '')
		{
			unset($this->session->pursuitAction[$entry->pursuitAction->PursuitActionToRemove]);
		}
		
		unset($entry->pursuitAction->PursuitActionToRemove);
		$this->session->form = $entry;
		$this->redirect('this#pursuitAction');
	}
	
	/**
	 * Processing of pursuit form submitted by save button.
	 * @param SubmitButton $button
	 */
	public function savePursuitFormSubmitted(SubmitButton $button)
	{
		if (!$this->hasPermission())
		{
			$this->flashMessage('You don\'t have permission to modify Pursuits.', 'warning');
			$this->redirect('Pursuit:default');
		}
		
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues();
			
			// creation of new pursuit
			// just in case of creation value is filled by some string because we need to determine moment when to clear session
			if ($entry->PursuitID == self::NEW_PURSUIT)
			{
				$entry->PursuitID = NULL;
			}
			
            // check pursuit state if exists
			$pursuit = $this->context->pursuitRepository->find($entry->PursuitID);
			if ($pursuit)
			{   // pursuit closed with result OnHold means pursuit is frozen only
				if (($pursuit->Status == Pursuit::PURSUIT_STATUS_CLOSED) && ($pursuit->ClosureResult != Pursuit::PURSUIT_CLOSURE_RESULT_ON_HOLD))
				{
					$this->flashMessage('Pursuit can not be modified, because it has been closed.', 'warning');
			        $this->redirect('Pursuit:default');
				}
				unset($pursuit);
			}
			
			$pursuit = new Pursuit();
			if ($entry->PursuitID)
			    $pursuit->PursuitID = $entry->PursuitID;
			$pursuit->CustomerID = $entry->CustomerID;
			$pursuit->ProgramID = ($entry->ProgramID) ? $entry->ProgramID : NULL;
			$pursuit->CallID = ($entry->CallID) ? $entry->CallID : NULL;
			$pursuit->CostShare = $entry->CostShare ? ($entry->CostShare / 100) : NULL;
			$pursuit->Name = $entry->Name;
			$pursuit->Status = $entry->Status;
			$pursuit->SFDCNumber = $entry->SFDCNumber;
			$pursuit->PursuitManager = $entry->PursuitManager;
			$pursuit->PursuitPI = $entry->PursuitPI;
			$pursuit->Remarks = $entry->Remarks;
			$pursuit->DocumentStorage = ($entry->DocumentStorage) ? $entry->DocumentStorage : NULL;
			$pursuit->LastUpdatedDate = date('Y-m-d');
			$pursuit->UpdatedBy = $this->getUser()->getId();
			if ($pursuit->Status != Pursuit::PURSUIT_STATUS_COMMITTED) // if pursuit is not committed, use user estimate
				$pursuit->ProbabilityOfWin = $entry->ProbabilityOfWin ? ($entry->ProbabilityOfWin / 100) : NULL;
			else // committed pursuit has always full probability of win
				$pursuit->ProbabilityOfWin = 1;
			
			$result = FALSE; // result of storing of pursuit object
			
			if ($entry->PursuitID) // existing ID means update
			{
				$result = $this->context->pursuitRepository->update($pursuit);
			}
			else
			{
				$pursuit->CreatedDate = date('Y-m-d');
				$result = $this->context->pursuitRepository->save($pursuit);
			}
			
			// Pursuit Plan
			foreach (PursuitPlan::getWebalizedMilestones() as $key => $milestone)
		    {
				$pursuitPlan = new PursuitPlan();
				$pursuitPlan->PursuitID = $pursuit->PursuitID;
				$pursuitPlan->Milestone = $key;
				$pursuitPlan->PlannedDate = $entry['pursuitPlan']->{$milestone . 'Planned'} ? $entry['pursuitPlan']->{$milestone . 'Planned'} : NULL;
				$pursuitPlan->ActualDate = $entry['pursuitPlan']->{$milestone . 'Actual'} ? $entry['pursuitPlan']->{$milestone . 'Actual'} : NULL;
				$pursuitPlan->Link = $entry['pursuitPlan']->{$milestone . 'Link'} ? $entry['pursuitPlan']->{$milestone . 'Link'} : NULL;
				$pursuitPlan->BnPPlan = $entry['pursuitPlan']->{$milestone . 'BnPPlan'} ? $entry['pursuitPlan']->{$milestone . 'BnPPlan'} : NULL;
				$pursuitPlan->BnPActuals = $entry['pursuitPlan']->{$milestone . 'BnPActuals'} ? $entry['pursuitPlan']->{$milestone . 'BnPActuals'} : NULL;
				$pursuitPlan->Remarks = $entry['pursuitPlan']->{$milestone . 'Remarks'} ? $entry['pursuitPlan']->{$milestone . 'Remarks'} : NULL;
				
				if ($entry->PursuitID)
					$result = $this->context->pursuitPlanRepository->update($pursuitPlan);
				else
					$result = $this->context->pursuitPlanRepository->save($pursuitPlan);
				
				unset($pursuitPlan);
			}
			
			// RED Team
			$redTeamKeys = array_keys($this->session->redTeam);
			if ($entry->PursuitID)
			{
                $all_redTeam = $this->context->redTeamRepository->findByPursuitID($pursuit->PursuitID);
				
				$oldRedTeam = array();
				foreach ($all_redTeam as $redTeam)
				    $oldRedTeam[] = $redTeam->ID;
				
				$actualRedTeam = array();
				foreach ($redTeamKeys as $key)
					$actualRedTeam[$key] = $entry['redTeam']->{'EID' . $key};
					
			    $membersToRemove = array_diff($oldRedTeam, $actualRedTeam);
				foreach ($membersToRemove as $eid)
					$result = $this->context->redTeamRepository->delete($eid, $pursuit->PursuitID);
				
				foreach ($actualRedTeam as $key => $eid)
				{
					$redTeam = new RedTeam();
					$redTeam->ID = $entry['redTeam']->{'EID' . $key};
					$redTeam->PursuitID = $pursuit->PursuitID;
					
					if (in_array($eid, $oldRedTeam))
						$result = $this->context->redTeamRepository->update($redTeam);
					else
						$result = $this->context->redTeamRepository->save($redTeam);
				}
			}
			else
			{
				foreach ($redTeamKeys as $key)
				{
					$redTeam = new RedTeam();
					$redTeam->ID = $entry['redTeam']->{'EID' . $key};
					$redTeam->PursuitID = $pursuit->PursuitID;

					$result = $this->context->redTeamRepository->save($redTeam);
				}
			}
			
			// Pursuit Risk
			$pursuitRiskKeys = array_keys($this->session->pursuitRisk);
			if ($entry->PursuitID)
			{
                $all_pursuitRisk = $this->context->pursuitRiskRepository->findByPursuitID($pursuit->PursuitID);
				
				$oldPursuitRisk = array();
				foreach ($all_pursuitRisk as $pursuitRisk)
				    $oldPursuitRisk[] = $pursuitRisk->ID;
				
				$actualPursuitRisk = array();
				foreach ($pursuitRiskKeys as $key)
					$actualPursuitRisk[$key] = $entry['pursuitRisk']->{'RiskID' . $key};
					
			    $risksToRemove = array_diff($oldPursuitRisk, $actualPursuitRisk);
				foreach ($risksToRemove as $id)
					$result = $this->context->pursuitRiskRepository->delete($id);
				
				foreach ($actualPursuitRisk as $key => $id)
				{
					$pursuitRisk = new PursuitRisk();
					if ($id)
					    $pursuitRisk->ID = $id;
					$pursuitRisk->PursuitID = $pursuit->PursuitID;
					$pursuitRisk->Risk = $entry['pursuitRisk']->{'Risk' . $key};
					$pursuitRisk->Owner = $entry['pursuitRisk']->{'Owner' . $key};
					$pursuitRisk->Deadline = ($entry['pursuitRisk']->{'Deadline' . $key}) ? $entry['pursuitRisk']->{'Deadline' . $key} : NULL;
					$pursuitRisk->Status = ($entry['pursuitRisk']->{'Status' . $key}) ? $entry['pursuitRisk']->{'Status' . $key} : NULL;
					$pursuitRisk->Remarks = ($entry['pursuitRisk']->{'Remarks' . $key}) ? $entry['pursuitRisk']->{'Remarks' . $key} : NULL;

					if ($id)
						$result = $this->context->pursuitRiskRepository->update($pursuitRisk);
					else
						$result = $this->context->pursuitRiskRepository->save($pursuitRisk);
				}
			}
			else
			{
				foreach ($pursuitRiskKeys as $key)
				{
					$pursuitRisk = new PursuitRisk();
					$pursuitRisk->PursuitID = $pursuit->PursuitID;
					$pursuitRisk->Risk = $entry['pursuitRisk']->{'Risk' . $key};
					$pursuitRisk->Owner = $entry['pursuitRisk']->{'Owner' . $key};
					$pursuitRisk->Deadline = ($entry['pursuitRisk']->{'Deadline' . $key}) ? $entry['pursuitRisk']->{'Deadline' . $key} : NULL;
					$pursuitRisk->Status = ($entry['pursuitRisk']->{'Status' . $key}) ? $entry['pursuitRisk']->{'Status' . $key} : NULL;
					$pursuitRisk->Remarks = ($entry['pursuitRisk']->{'Remarks' . $key}) ? $entry['pursuitRisk']->{'Remarks' . $key} : NULL;

					$result = $this->context->pursuitRiskRepository->save($pursuitRisk);
				}
			}
			
			// Pursuit Action
			$pursuitActionKeys = array_keys($this->session->pursuitAction);
			if ($entry->PursuitID)
			{
                $all_pursuitAction = $this->context->pursuitActionRepository->findByPursuitID($pursuit->PursuitID);
				
				$oldPursuitAction = array();
				foreach ($all_pursuitAction as $pursuitAction)
				    $oldPursuitAction[] = $pursuitAction->ID;
				
				$actualPursuitAction = array();
				foreach ($pursuitActionKeys as $key)
					$actualPursuitAction[$key] = $entry['pursuitAction']->{'ActionID' . $key};
					
			    $actionsToRemove = array_diff($oldPursuitAction, $actualPursuitAction);
				foreach ($actionsToRemove as $id)
					$result = $this->context->pursuitActionRepository->delete($id);
				
				foreach ($actualPursuitAction as $key => $id)
				{
					$pursuitAction = new PursuitAction();
					if ($id)
					    $pursuitAction->ID = $id;
					$pursuitAction->PursuitID = $pursuit->PursuitID;
					$pursuitAction->Action = $entry['pursuitAction']->{'Action' . $key};
					$pursuitAction->Owner = $entry['pursuitAction']->{'Owner' . $key};
					$pursuitAction->Deadline = ($entry['pursuitAction']->{'Deadline' . $key}) ? $entry['pursuitAction']->{'Deadline' . $key} : NULL;
					$pursuitAction->Status = ($entry['pursuitAction']->{'Status' . $key}) ? $entry['pursuitAction']->{'Status' . $key} : NULL;
					$pursuitAction->Remarks = ($entry['pursuitAction']->{'Remarks' . $key}) ? $entry['pursuitAction']->{'Remarks' . $key} : NULL;

					if ($id)
						$result = $this->context->pursuitActionRepository->update($pursuitAction);
					else
						$result = $this->context->pursuitActionRepository->save($pursuitAction);
				}
			}
			else
			{
				foreach ($pursuitActionKeys as $key)
				{
					$pursuitAction = new PursuitAction();
					$pursuitAction->PursuitID = $pursuit->PursuitID;
					$pursuitAction->Action = $entry['pursuitAction']->{'Action' . $key};
					$pursuitAction->Owner = $entry['pursuitAction']->{'Owner' . $key};
					$pursuitAction->Deadline = ($entry['pursuitAction']->{'Deadline' . $key}) ? $entry['pursuitAction']->{'Deadline' . $key} : NULL;
					$pursuitAction->Status = ($entry['pursuitAction']->{'Status' . $key}) ? $entry['pursuitAction']->{'Status' . $key} : NULL;
					$pursuitAction->Remarks = ($entry['pursuitAction']->{'Remarks' . $key}) ? $entry['pursuitAction']->{'Remarks' . $key} : NULL;

					$result = $this->context->pursuitActionRepository->save($pursuitAction);
				}
			}
			
			if ($result) // check result of storing of pursuit object
				$this->flashMessage('Pursuit "' . $pursuit->Name . '" saved successfully.', 'success');
			else
			{
				$this->flashMessage('Pursuit "' . $pursuit->Name . '" can not be saved.', 'error');
				return;
			}
			
			if ($entry->PursuitID) // is editation
			    $this->redirect('this');
			else
				$this->redirect('Pursuit:edit', array('id' => $pursuit->PursuitID));
		}
		
		$this->flashMessage("Error occured during submitting the form. You has been redirected.", 'error');
        $this->redirect('Pursuit:default');
	}
	
	/**
	 * Processing of pursuit form submitted by back button.
	 * @param SubmitButton $button
	 */
	public function backPursuitFormSubmitted(SubmitButton $button)
	{
		$this->redirect('Pursuit:default');
	}	
	
	/**
	 * Creates Pursuit close form.
	 * @param string $name Name of component.
	 * @return \AppForm
	 */
	public function createComponentClosePursuitForm($name)
	{
		$form = new AppForm($this, $name);
		
		
		$form->addHidden('PursuitID');
		
		$form->addRadioList('ClosureResult', NULL, Pursuit::getClosureResultsFullNames())
				->setDefaultValue(Pursuit::PURSUIT_CLOSURE_RESULT_ON_HOLD);
		
		$form->addButton('Cancel', 'Cancel')
		        ->setAttribute('onclick', '$.fancybox.close()');
		
		$form->addSubmit('Close', 'Close')
				->onClick[] = array($this, 'closeClosePursuitFormSubmitted');
		
		return $form;
	}
	
	/**
	 * Processing of pursuit close form submitted by close button.
	 * @param SubmitButton $button
	 */
	public function closeClosePursuitFormSubmitted(SubmitButton $button)
	{
		if (!$this->hasPermission())
		{
			$this->flashMessage('You don\'t have permission to modify Pursuits.', 'warning');
			$this->redirect('Pursuit:default');
		}
		
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues();
			
			// check pursuit state if exists
			$pursuit = $this->context->pursuitRepository->find($entry->PursuitID);
			if ($pursuit)
			{
				if ($pursuit->Status == Pursuit::PURSUIT_STATUS_CLOSED)
				{
					$this->flashMessage('Pursuit can not be modified, because it has been closed.', 'warning');
			        $this->redirect('Pursuit:default');
				}
				unset($pursuit);
			}
			
			$pursuit = $this->context->pursuitRepository->find($entry->PursuitID);
			if ($pursuit === NULL)
			{
				$this->flashMessage('Invalid Pursuit ID. You have been redirected.', 'warning');
				$this->redirect('Pursuit:default');
			}
			
			$pursuit->Status = Pursuit::PURSUIT_STATUS_CLOSED;
			$pursuit->ClosureResult = $entry->ClosureResult;
			$pursuit->LastUpdatedDate = date('Y-m-d');
			$pursuit->UpdatedBy = $this->getUser()->getId();
			if ($this->context->pursuitRepository->update($pursuit))
			{
				$this->flashMessage('Pursuit "' . $pursuit->Name . '" closed successfully.', 'success');
				$this->redirect('this');
			}
			else
			{
				$this->flashMessage('Pursuit "' . $pursuit->Name . '" can not be closed.', 'success');
				return;
			}
		}
		$this->flashMessage("Error occured during submitting the form. You has been redirected.", 'error');
        $this->redirect('Pursuit:default');
	}
	
	/**
	 * Filters and sets programs as well as calls according to selected customer.
	 * @param AppForm $form Form with fields dynamically related with field Customer.
	 */
	protected function setCustomerDependencies($form)
	{
		// capture selected customer and related program. To be able to filter list of programs and list of calls
		$customerID = $form['CustomerID']->getValue();
		$programID = $form['ProgramID']->getValue();
		
		// filter related programs according to selected customer
		$programs = $this->context->customerProgramRepository->findByCustomer($customerID);
		$customerPrograms = array('' => '');
		foreach ($programs as $program)
			$customerPrograms[$program->ProgramID] = $program->Program;
		$form['ProgramID']->setItems($customerPrograms);
		
		// filter related calls according to selected program
		$calls = $this->context->customerProgramCallRepository->findByProgram($programID);
		$customerProgramCalls = array('' => '');
		foreach ($calls as $call)
			$customerProgramCalls[$call->CallID] = $call->Call;
		$form['CallID']->setItems($customerProgramCalls);
	}
	
	/**
	 * Has current user sufficient rights to work with pursuits. 
	 * @return boolean TRUE in case user has rights. FALSE otherwise.
	 */
	protected function hasPermission()
	{
		// all users have permissions now

//		if (!$this->getUser()->isInRole('STRAP-DL'))
//			return FALSE;
		
		return TRUE;
	}
}
