<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * StrapSettingsPresenter presents forms for setting upp
 * the strap application.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class StrapSettingsPresenter extends SecuredPresenter
{
	/* CTQ template value setup */
	const CTQ_TEMPLATE_VALUE = 'ctqTemplateValue';
    
	/**
	 * Renders Empty output. Used only for redirecting when user does
     * not have enough rights.
	 */
	public function renderDefault()
	{
        
    }
    
	/**
	 * Renders ctq template form.
	 */
	public function renderCtq()
	{
		$updateCTQTemplate = $this['updateCTQForm'];
		$defaulValue = array();
		$defaulValue['ctq'] = $this->context->globalSetting->get(self::CTQ_TEMPLATE_VALUE);
		$updateCTQTemplate->setDefaults($defaulValue);
		$this->template->updateCTQForm = $updateCTQTemplate;
    }
	
	/**
     * Creates form for ctq template update.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentUpdateCTQForm($name)
    {
        $form = new AppForm($this, $name);
		$form->addTextArea('ctq', 'CTQ:', 55, 5)
				->addRule(Form::FILLED, 'CTQ template value has to be filled.');
		$submit_button = $form->addSubmit('update', 'Update');
		if (!$this->testUserRoles('STRAP-ADMIN'))
			$submit_button->setDisabled(true);
		$form->onSuccess[] = array($this, 'updateCTQFormSubmitted');
    }
	
    /**
     * Component createComponentUpdateCTQForm submit change ctq template.
     * 
     * @param AppForm $form 
     */
    public function updateCTQFormSubmitted(AppForm $form) 
	{
        $entry = $form->getValues();
		//Only STRAP-ADMIN user can update file
		if ($this->testUserRoles('STRAP-ADMIN'))
		{
            if ($form->isValid())
            {
				if($this->context->globalSetting->set(self::CTQ_TEMPLATE_VALUE, $entry['ctq'], false, 'string') == 0)
				{
					$this->flashMessage('CTQ template value updated succesfully.', 'success');
				}
				else
				{
					$this->flashMessage('CTQ template value update failed.', 'error');
				}
				$this->redirect('ctq');
            }
            else
            {
                $this->flashMessage("Invalid form values", 'error');
				$this->redirect('default');
            }
		}
		else
		{
			$this->flashMessage("You are not authorized to update ctq template. STRAP-ADMIN role required!", 'error');
			$this->redirect('default');
		}
	}
}