<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Organization unit selector
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class OrganizationUnitSelectorContainer extends BaseContainer
{
	/** @var int Width of container fields */
	private $width;
	/** @var boolean Cut short each field in hierarchy */
	private $showHierarchy;

	/**
	 * @param int $width=300 Width of container fields. Min value = 60
	 * @param boolean $showHierarchy=FALSE Cut short each field in hierarchy
	 */
	public function __construct($width = 300, $showHierarchy = FALSE)
    {
		parent::__construct();
		
		$this->width = ($width >= 60) ? $width : 60;
		$this->showHierarchy = $showHierarchy;
	}


	/**
	 * Handles change of value in organization unit selector
	 * @param int $id Should be organization unit id
	 * @return array
	 *     $array(
	 *         [SBE|Department|Cluster|TechnicalGroup] => array(
	 *             id => value,     // selected value
	 *             values => array( // list of organization units
	 *                 id => unit name
	 *             )
	 *         )
	 *     )
	 */
	public function ajaxHandle($id)
	{
	    $unit_branch = $this->context->organizationUnitRepository->findAllAncestor($id);
		
		$sbe_id = ($unit_branch[OrganizationUnit::ORG_UNIT_TYPE_SBE]) ? $unit_branch[OrganizationUnit::ORG_UNIT_TYPE_SBE]->ID : NULL;
		if ($sbe_id)
		{
		    $related_dep = $this->getAllChildrenByType($sbe_id, OrganizationUnit::ORG_UNIT_TYPE_DEP);
			$related_clus = $this->getAllChildrenByType($sbe_id, OrganizationUnit::ORG_UNIT_TYPE_CLUS);
			$related_tg = $this->getAllChildrenByType($sbe_id, OrganizationUnit::ORG_UNIT_TYPE_TG);
		}
		else
		{
			$related_dep = $this->getAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_DEP);
			$related_clus = $this->getAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_CLUS);
			$related_tg = $this->getAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_TG);
		}
		
		$dep_id = ($unit_branch[OrganizationUnit::ORG_UNIT_TYPE_DEP]) ? $unit_branch[OrganizationUnit::ORG_UNIT_TYPE_DEP]->ID : NULL;
		if ($dep_id)
		{
		    $related_clus = $this->getAllChildrenByType($dep_id, OrganizationUnit::ORG_UNIT_TYPE_CLUS);
		    $related_tg = $this->getAllChildrenByType($dep_id, OrganizationUnit::ORG_UNIT_TYPE_TG);
		}
		
		$clus_id = ($unit_branch[OrganizationUnit::ORG_UNIT_TYPE_CLUS]) ? $unit_branch[OrganizationUnit::ORG_UNIT_TYPE_CLUS]->ID : NULL;
		if ($clus_id)
		    $related_tg = $this->getAllChildrenByType($clus_id, OrganizationUnit::ORG_UNIT_TYPE_TG);
		
		$tg_id = ($unit_branch[OrganizationUnit::ORG_UNIT_TYPE_TG]) ? $unit_branch[OrganizationUnit::ORG_UNIT_TYPE_TG]->ID : NULL;


		$technical_groups = array();
		foreach ($related_tg as $tg)
			$technical_groups[$tg->ID] = $this->getOrgUnitSelectName($tg);
		
		$clusters = array();
		foreach ($related_clus as $clus)
			$clusters[$clus->ID] = $this->getOrgUnitSelectName($clus);

		$departments = array();
		foreach ($related_dep as $dep)
			$departments[$dep->ID] = $this->getOrgUnitSelectName($dep);

		return	array(
			'SBE' => array(
				'id' => $sbe_id,
				'values' => array()
			),
			'Department' => array(
				'id' => $dep_id,
				'values' => $departments
			),
			'Cluster' => array(
				'id' => $clus_id,
				'values' => $clusters
			),
			'TechnicalGroup' => array(
				'id' => $tg_id,
				'values' => $technical_groups
			)
		);
	}
	
	/**
	 * Adds container content to form
	 */
	protected function configure()
	{
		// Here place link to ajax handle method
		$this->addHidden('ajaxHandle');
		
		// Get data to SBE field
		$all_sbe = $this->getAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE);
		$sbe_list = array();
		foreach ($all_sbe as $sbe)
				$sbe_list[$sbe->ID] = $this->getOrgUnitSelectName($sbe);

		$this->addSelect('SBE', 'SBE / Tier3', $sbe_list)
				->setPrompt('')
			    ->setAttribute('style', 'width: ' . $this->width . 'px;');
		
		// Get data to Department field
		$all_dep = $this->getAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_DEP);
		$dep_list = array();
		foreach ($all_dep as $dep)
			$dep_list[$dep->ID] = $this->getOrgUnitSelectName($dep);

		$this->addSelect('Department', 'Department  / Tier4', $dep_list)
				->setPrompt('');
		
		// Get data to Cluster field
		$all_clus = $this->getAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_CLUS);
		$clus_list = array();
		foreach ($all_clus as $clus)
			$clus_list[$clus->ID] = $this->getOrgUnitSelectName($clus);
		
		$this->addSelect('Cluster', 'Cluster  / Tier5', $clus_list)
				->setPrompt('');
		
		// Get data to TechnicalGroup field
		$all_tg = $this->getAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_TG);
		$tg_list = array();
		foreach ($all_tg as $tg)
			$tg_list[$tg->ID] = $this->getOrgUnitSelectName($tg);
		
		$this->addSelect('TechnicalGroup', 'Technical Group', $tg_list)
				->setPrompt('');
		
		if ($this->showHierarchy)
		{
			$this['Department']->setAttribute('style', 'width: ' . ($this->width - 10) . 'px;');
			$this['Cluster']->setAttribute('style', 'width: ' . ($this->width - 20) . 'px;');
			$this['TechnicalGroup']->setAttribute('style', 'width: ' . ($this->width - 30) . 'px;');
		}
		else
		{
			$this['Department']->setAttribute('style', 'width: ' . $this->width . 'px;');
			$this['Cluster']->setAttribute('style', 'width: ' . $this->width . 'px;');
			$this['TechnicalGroup']->setAttribute('style', 'width: ' . $this->width . 'px;');
		}
	}
	
	/**
	 * Function gets name of organization unit for select form input.
	 * 
	 * @param OrganizationUnit $orgUnit
	 * @return string Name what will be shown in select.
	 */
	protected function getOrgUnitSelectName(OrganizationUnit $orgUnit)
	{
		return $orgUnit->Name;
	}
	
	/**
	 * Gets all org unit by type.
	 * 
	 * @param string $type Type of organization unit in range of OrganizationUnit::ORG_UNIT_TYPE_*.
	 * @return array Array of OrganizationUnit where Organization unit ID is key.
	 */
	protected function getAllOrgUnitByType($type)
	{
		return $this->context->organizationUnitRepository->findAllOrgUnitByType($type);
	}
	
	/**
	 * Gets all children by type
	 * 
	 * @param int $orgUnitID organization unit ID.
	 * @param string $type Type of organization unit in range of OrganizationUnit::ORG_UNIT_TYPE_*.
	 * @return	array of OrganizationUnit where id is key or empty array.
	 */
	protected function getAllChildrenByType($orgUnitID, $type)
	{
		return $this->context->organizationUnitRepository->findAllChildrenByType($orgUnitID, $type);
	}
}
