<?php
/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009 - 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package forms
 */

/**
 * Extends FormContainer class of method for SelectBox2 and MultiSelectBox2
 * 
 * @author Jiri Pirochta <vojtech.obrusnik@honeywell.com>
 */
class SelectBox2 extends Object
{
	/**
	 * Adds select box control that allows single item selection by select2 lib.
	 * 
	 * @param  FormContainer  nette form container
	 * @param  string         control name
	 * @param  string         label
	 * @param  array          options from which to choose
	 * @param  int            number of rows that should be visible
	 * @param  array          select2 config params
	 * @return SelectBox
	 */
	public static function addSelect2(FormContainer $form, $name, $label, array $items = NULL, $size = NULL, array $select2Config = NULL)
	{
		$select2 = $form->addSelect($name, $label, $items, $size)
		->setAttribute('class', 'select2-single');
		if(!empty($select2Config))
		{
			foreach($select2Config as $configName => $configValue)
			{
				$select2->setAttribute('data-' . $configName, $configValue);
			}
		}
		return $select2;
	}
	
	/**
	 * Adds select box control that allows multiple item selection by select2 lib.
	 * 
	 * @param  FormContainer  nette form container
	 * @param  string         control name
	 * @param  string         label
	 * @param  array          options from which to choose
	 * @param  int            number of rows that should be visible
	 * @param  array          select2 config params
	 * @return MultiSelectBox
	 */
	public static function addMultiSelect2(FormContainer $form, $name, $label, array $items = NULL, $size = NULL, array $select2Config = null)
	{
		$select2 = $form->addMultiSelect($name, $label, $items, $size)
		->setAttribute('class', 'select2-multi');
		if(!empty($select2Config))
		{
			foreach($select2Config as $configName => $configValue)
			{
				$select2->setAttribute('data-' . $configName, $configValue);
			}
		}
		return $select2;
	}
}
