<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package grids
 */

/**
 * Grid for displaying logged messages.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */

class LogDataGrid extends Grid
{
	protected $logs;

	public function __construct($logs)
	{
		parent::__construct();
		$this->logs = $logs;
	}

	/** function instead of PHP 5.3 closure */
	public function configureColumnTypeCellRenderer($row)
	{
		switch ($row['Type'])
		{
			case ILogger::DEBUG:
				return 'background-color: #ccc;';
			case ILogger::INFO:
				return '' /*'background-color: #fff;'*/;
			case ILogger::NOTICE:
				return 'background-color: #bbff99;';
			case ILogger::WARNING:
				return 'background-color: #FFF9D7;';
			case ILogger::ERROR:
				return 'background-color: #ff9999;';
			case ILogger::FATAL:
				return 'background-color: #ff0000; color: #fff; font-weight: bold;;';
		}
		return '';
	}
	
	/** function instead of PHP 5.3 closure */
	public function configureRendererCreatedDate($row)
	{
		return date('j.n.Y', strtotime($row['Date']));
	}
	
	/** function instead of PHP 5.3 closure */
	public function configureFormRendererCreatedDate($row)
	{
		return date('Y-m-d', strtotime($row['Date']));
	}
	
	/** function instead of PHP 5.3 closure */
	public function configureRendererCreatedTime($row)
	{
		return date('H:i:s', strtotime($row['Time']));
	}
	
	protected function configure($presenter)
	{
		$source = new NDataSource($this->logs->select('LogID, LogID AS id, Type, Message, Responsible, Batch, Group, CAST(Created AS DATE) AS Date, CAST(Created AS TIME) AS Time'));

		$this->setDataSource($source);

		//$this->setWidth("1000px");
		$this->setDefaultOrder("LogID DESC");
		$this->setPerPageValues(array(50, 100, 200));

		$this->addColumn('LogID', 'ID', '40px')
			->setTextEditable()
			->setNumericFilter();

		$log_message_types = array(ILogger::DEBUG => "Debug", ILogger::INFO => "Information",
					ILogger::NOTICE => "Notice", ILogger::WARNING => "Warning", 
					ILogger::ERROR => "Error", ILogger::FATAL => "Fatal Error");
		$this->addColumn('Type', 'Type', '75px')
			->setSelectFilter($log_message_types)
			->setSelectEditable($log_message_types)
			->setCellRenderer(array($this, 'configureColumnTypeCellRenderer'));

		$this->addColumn('Message', 'Message', '510px', 300)
			->setTextEditable()
			->setTextFilter()
			->setSortable(false)
			->setAutocomplete(5);

		$this->addColumn('Responsible', 'Responsible', '65px')
			->setSortable(false)
			->setTextFilter();

		$this->addColumn('Batch', 'Batch', '110px')
			->setSortable(false)
			->setTextFilter();

		$this->addColumn('Group', 'Group', '110px', 30)
			->setTextEditable()
			->setTextFilter();
		
		$this->addColumn('Date', 'Date', '90px')
			->markAsAlias()
			->setRenderer(array($this, 'configureRendererCreatedDate'))
			->setDateFilter()
			->setDateEditable()
			->setFormRenderer(array($this, 'configureFormRendererCreatedDate'));
		
		$this->addColumn('Time', 'Time', '50px')
			->markAsAlias()
			->setRenderer(array($this, 'configureRendererCreatedTime'))
			->setSortable(false);

	}

}
