<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009 - 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class with Annual Operating Plan (AOP).
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class AnnualOperatingPlan extends Object
{
    /** @var array AOP array with operating hours for months in year. */
	private $AOP;
	
    /** @var int Year of the plan */
    private $year;

    public function  __construct($year = 0) 
    {
		$this->setYear($year);
		$this->AOP = array();
		for ($i = 1; $i <= 12; $i++)
			$this->AOP[$i] = 0;
    }

    /**
     * Sets year of AOP.
     * 
     * @param int $year Year number
     */
    public function setYear($year)
    {
		if (NumericUtils::isPositiveInteger($year))
			$this->year = 0 + $year;
		else
			throw new InvalidArgumentException("Year must be positive integer. Value: " . $year);
    }
    
    /**
     * Gets year of workalbe hours.
     * 
     * @return int
     */
	public function getYear()
	{
		return $this->year;
	}

    /**
     * Sets hours for month in AOP.
     * 
     * @param int $hours Hours for the month in AOP.
     * @param int $month Month number <1, 12>
     */
    public function setMonthAOPHours($hours, $month)
    {
		if (!NumericUtils::isNonNegativeInteger($hours))
			throw new InvalidArgumentException("Hours must be non-negative integer. Value: " . $hours);
		if (NumericUtils::isPositiveInteger($month)
			&& ($month >= 1) & ($month <= 12))
			$this->AOP[0 + $month] = $hours;
		else
			throw new InvalidArgumentException("Month must be integer in range <1, 12> inclusively. Value: " . $month);
    }	

	/**
     * Returns hours for month in AOP.
     * 
     * @return int
     */
    public function getMonthAOPHours($month)
    {
		if (NumericUtils::isPositiveInteger($month)
			&& ($month >= 1) & ($month <= 12))
			return $this->AOP[0 + $month];
		else
			throw new InvalidArgumentException("Month must be integer in range <1, 12> inclusively. Value: " . $month);
    }	
	
	/**
	 * Function returns array with all hours.
	 * 
	 * @return array $AOP
	 */
	public function getAOP()
	{
		return $this->AOP;
	}

	
}
