<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Represents cost plan assigned to a task for 
 * one month in a year.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class AssignmentsCost extends Object
{
	/** @var int Unique assigment ID. */
	private $id;
	/** @var int ID of the task */
	private $taskID;
	/** @var int */
	private $year;
	/** @var int Must be number from 1 to 12 */
	private $month;
	/** @var int Must be non-negative integer */
	private $cost = 0;

	public function __construct()
	{
	}
	
   /**
     * Set ID of assignment.
     * 
     * @param int $id Must be positive integer.
     */
	public function setID($id)
	{
		if ($id > 0)
			$this->id = $id;
		else
			throw new InvalidArgumentException('ID must be integer greater than 0. Wrong param: '.$id);
	}
	
   /**
     * Gets ID of assignment.
     * 
     * @return int
     */
	public function getID()
	{
		return $this->id;
	}
	
   /**
     * Set ID of task to which this plan belogs.
     * 
     * @param int $id Must be positive integer.
     */
	public function setTaskID($id)
	{
		if ($id > 0)
			$this->taskID = $id;
		else
			throw new InvalidArgumentException('Task ID must be integer greater than 0. Wrong param: '.$id);
	}
	
    /**
     * Gets ID of task to which this plan belogs.
     * 
     * @return int
     */
	public function getTaskID()
	{
		return $this->taskID;
	}

   /**
     * Sets number of cost assigned to a task.
     * 
     * @param int $cost Must be positive integer.
     */
	public function setCost($cost)
	{
		$this->cost = $cost;
	}
	
    /**
     * Gets number of cost assigned to a task.
     * 
     * @return int
     */
	public function getCost()
	{
		return $this->cost;
	}

   /**
     * Sets month.
     * Must be number in range <1,12>.
     * 
     * @param int $month 
     */
	public function setMonth($month)
	{
		if (($month >= 1) && ($month <= 12))
			$this->month = $month;
		else
			throw new InvalidArgumentException('Month must be integer in range <1,12>. Wrong param: '.$month);
	}
	
    /**
     * Gets month.
     * 
     * @return int
     */
	public function getMonth()
	{
		return $this->month;
	}

   /**
     * Sets year.
     * 
     * @param int $year Must be positive integer.
     */
	public function setYear($year)
	{
		if ($year > 0)
			$this->year = $year;
		else
			throw new InvalidArgumentException('Year must be integer greater than 0. Wrong param: '.$year);
	}
	
    /**
     * Gets year.
     * 
     * @return int
     */
	public function getYear()
	{
		return $this->year;
	}
}
