<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and licence information, please view
 * the file licence.txt that was distributed with this source code.
 * @package models
 */

/**
 * ECLicenceName repository access class.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ECLicenceNameRepository
{
	/** @var IECLicenceNameMapper */
 	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;
	
	public function __construct(IECLicenceNameMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ExportControl-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "ExportControl"));
    }
	
	/**
	 * Saves ECLicenceName.
	 * @param ECLicenceName $licence_name
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(ECLicenceName $licence_name)
	{
		if ($this->mapper->save($licence_name))
		{
			$this->multilog->log('ECLicenceName with ID: ' . $licence_name->ECLicenceNameID . ' saved.', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Saves ECLicenceName if not exists.
	 * @param string $name ECLicenceName.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function add($name)
	{
		$old = $this->findByName($name);
		if (!$old)
		{
			$new = new ECLicenceName();
			$new->Name = $name;
            return $this->save($new);
		}
		
		return TRUE;
	}
	
	/**
	 * Updates ECLicenceName.
	 * @param ECLicenceName $licence_name
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(ECLicenceName $licence_name)
	{
		$oldObject = $this->find($licence_name->ECLicenceNameID);
		if ($this->mapper->update($licence_name))
		{
			$this->multilog->log('ECLicenceName with ID: ' . $licence_name->ECLicenceNameID . ' updated. ' . $this->multilog->differences($oldObject, $licence_name), ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Deletes ECLicenceName.
	 * @param int $id ECLicenceName id.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		if ($this->mapper->delete($id))
		{
			$this->multilog->log('ECLicenceName with ID: ' . $id . ' deleted.', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Finds ECLicenceName by id.
	 * @param int $id EC licence id.
	 * @return ECLicenceName or NULL if not found.
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}
	
	/**
	 * Finds all ECLicenceNames.
	 * @return array Array of ECLicenceName or empty array.
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
	
	/**
	 * Finds all ECLicenceName by name.
	 * @param string $name
	 * @return ECLicenceName or NULL if not found.
	 */
	public function findByName($name)
	{
		return $this->mapper->findByName($name);
	}
}