<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing cost centers.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
interface ICostCenterMapper
{
	/**
	 * Saves cost center.
	 * 
     * @param CostCenter $costCenter cost center.
	 */
	public function save(CostCenter $costCenter);
    
	/**
	 * Updates cost center.
	 * 
	 * @param CostCenter $costCenter cost center.
	 */
	public function update(CostCenter $costCenter);
    
    /**
     * Saves cost center labour rates.
     * 
     * @param CostCenterLabourRates $costCenterLR
     * @return boolean Returns true if cost center labour rates is successfully saved, otherwise false.
     */
    public function saveRates(CostCenterLabourRates $costCenterLR);
    
	/**
	 * Updates cost center labour rates.
	 * 
	 * @param CostCenter $costCenter cost center.
	 */
	public function updateRates(CostCenter $costCenter);

	/**
	 * Returns cost center labour rate by cost center id and year.
	 * 
	 * @param string $id Cost center ID.
	 * @param int $year Year of whitch rates should be found.
	 * @return	CostCenterLabourRate or NULL if not found.
	 */
	public function findCostCenterLabourRates($id, $year);
	
	/**
	 * Returns cost center by its id.
	 * 
	 * @param int $id Cost center ID.
	 * @return	CostCenterLabourRate
	 */
	public function findCostCenter($id);

	/**
	 * Returns all cost centers as an array sorted by name.
	 * 
	 * @return	array of CostCenterLabourRate
	 */
	public function findAllCostCenters();

	/**
	 * Returns sum of costs on current task in month in a year.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $month Month index (0-11).
	 * @param int $year Year.
	 * @param string $currency Currency code (only USD or CZK accepted)
	 * @return	float 
	 */
	public function findActualTaskCostsMonthSummary($task_id, $month, $year, $currency);
	
}

