<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing person found.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
interface IPersonFoundMapper
{
	/**
	 * Saves found person
	 * 
	 * @param PersonFound $person
	 * @return boolean
	 */
	public function save(PersonFound $person);
    
	/**
	 * Updates found person
	 * 
	 * @param PersonFound $person
	 * @return boolean
	 */
	public function update(PersonFound $person);

	/**
	 * Finds person stub by EID.
	 * 
	 * @param string $eid EID.
	 * @return	PersonFound or NULL if not found.
	 */
	public function findByEID($eid);
	
	/**
	 * Finds all PersonFound.
	 * 
	 * @return array Array of Persons where EID is key.
	 */
	public function findAll();
	
	/**
	 * Finds all infos for found person stubs.
	 * 
	 * @return array Array of PersonFound where EID is key.
	 */
	public function findAllBlacklisted();
	
	/**
	 * Finds all infos for found person stubs.
	 * 
	 * @return array Array of PersonFound where EID is key.
	 */
	public function findAllNonBlacklisted();
	
	/**
	 * Deletes found person stub by EID.
	 * 
	 * @param string $eid found person EID
	 * @return boolean Return true if delete was successfull, otherwise returns false.
	 */
	public function delete($eid);
}