<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing persons.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
interface IPersonMapper
{
	/**
	 * Saves person.
	 * 
	 * @param Person $person
	 */
	public function save(Person $person);
    
	/**
	 * Updates person.
	 * 
	 * @param Person $person
	 */
	public function update(Person $person);
	
	/**
	 * Finds person by id.
	 * 
	 * @param string $id person ID.
	 * @return	Person or NULL if not found.
	 */
	public function find($id);
	
	/**
	 * Finds all Persons.
	 * 
	 * @return array Array of Persons where EID is key.
	 */
	public function findAll();
	
	/**
	 * Finds all persons included into specific organization unit.
	 * 
	 * @param int $id ID of Organization unit.
	 * @return array Array of Persons where EID is key.
	 */
	public function findByOrganizationUnitID($id);
	
	/**
	 * Deletes person by id.
	 * 
	 * @param string $id person ID.
	 * @return boolean Return true if delete was successfull, otherwite returns false.
	 */
	public function delete($id);
	
    /**
     * Gets Persons by PersonFilter.
     *
     * @param PersonFilter $personFilter
     * @return array of Person Objects
     */
    public function getPersonsByFilter(PersonFilter $personFilter);
	
    /**
     * Gets Persons by PersonFilter. Returns every person that matches
	 * at least one set condition in the filter.
     *
     * @param PersonFilter $personFilter
     * @return array of Person
     */
    public function getPersonsByOrFilter(PersonFilter $personFilter);
	
	
	/**
	 * Gets all people, who have the 'TG' prefix in their EID.
	 * In fact, they are placeholders to shadow Technical Groups.
	 * 
	 * @return array 
	 */
	public function findAllTGs();
}


