<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing task assignment.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
interface IPersonTaskAssignmentMapper
{
	/**
	 * Saves persons task assignment.
	 * 
	 * @param PersonTaskMonthAssignment $assignment
	 * @return	boolean Returns TRUE if success, otherwise false.
	 */
	public function save(PersonTaskMonthAssignment $assignment);

	/**
	 * Deletes all person assignment from task in given year.
	 * 
	 * @param int $person_id Person ID.
	 * @param int $task_id Task ID.
	 * @param int $year Year.
	 */
	public function deleteByPersonAndYear($person_id, $task_id, $year);
	
	/**
	 * Finds assignment by id.
	 * 
	 * @param int $id task assignment ID.
	 * @return	PersonTaskMonthAssignment 
	 */
	public function find($id);

	/**
	 * Returns twelve task assigments for a person for given task ID in given year.
	 * Each for every fiscal month in a year.
	 * Month indexes are 0 (January) - 11 (December). 
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year year.
	 * @param string $person_id Person/Employee ID.
	 * @return	array of 12 PersonTaskMonthAssignment where key is a month index.
	 */
	public function findPersonTaskYearAssignment($task_id, $year, $person_id);

	/**
	 * Returns array of arrays of twelve task assigments 
	 * for every person that is assigned to given task ID in given year.
	 * 
	 * Each assignment (inner array) corresponds to a fiscal month in a year.
	 * Month indexes are 1 (January) - 12 (December). 
	 * If month assignment is missing only empty object is available on index.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year year.
	 * @param string $person_id Person/Employee ID.
	 * @return	array or arrays of 12 PersonTaskMonthAssignment 
	 *					where outer array keys are person IDs
	 *					and inner arrays keys are month indexes.
	 */
	public function findTaskYearAssignments($task_id, $year);

	/**
	 * Returns array of twelve values with total hours assigned every month
	 * to given person.
	 * 
	 * Month indexes are 1 (January) - 12 (December). 
	 * 
	 * @param int $person_id Person ID.
	 * @param int $year year.
	 * @return	array or integers where keys are month indexes.
	 */
    public function findPersonTotalYearAssignments($person_id, $year);

	/**
	 * Returns wheter person is assigned to to any task year.
	 * 
	 * @param int $person_id Person ID.
	 * @return	boolean
	 */
    public function isPersonAssignedToAnyTasks($person_id);
	
}


