<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing project charter approver history.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com> 
 */
interface IProjectCharterApproverHistoryMapper
{
    /**
	 * Saves project charter approver history.
	 * 
	 * @param ProjectCharterApproverHistory $projectCharterApproverHistory
	 * @return boolean
	 */
	public function save(ProjectCharterApproverHistory $projectCharterApproverHistory);

	/**
	 * Updates project charter approver history.
	 * 
	 * @param ProjectCharterApproverHistory $projectCharterApproverHistory
     * @return boolean
	 */
	public function update(ProjectCharterApproverHistory $projectCharterApproverHistory);
	
	/**
	 * Deletes existing project charter approver history.
     * 
	 * @param int $projectCharterID ProjectCharter ID.
	 * @param int $version Version of round of approval.
	 * @return boolean
	 */
	public function delete($projectCharterID, $version);
    
	/**
	 * Finds project charter approver history by project charter id and version.
	 * 
	 * @param int $projectCharterID ProjectCharter ID.
	 * @param int $version Version of round of approval.
	 * @return	ProjectCharterApproverHistory or NULL if not found.
	 */
	public function find($projectCharterID, $version);
        
	/**
	 * Finds latest version of project charter approver history by project charter id.
	 * 
     * @param int $projectCharterID ProjectCharter ID.
	 * @return	ProjectCharterApproverHistory or NULL if not found.
	 */
	public function findLatestVersion($projectCharterID);
        
	/**
	 * Initializes flag of latest version
	 * 
	 * @return boolean
	 */
	public function initLatestVersion($projectCharterID);
}
