<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing tasks.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
interface ITaskMapper
{
	/**
	 * Saves new task.
	 * 
	 * @param Task $task
	 */
	public function save(Task $task);
	
	/**
	 * Updates existing task.
	 * 
	 * @param Task $task
	 */
	public function update(Task $task);
	
	/**
	 * Finds task by id.
	 * 
	 * @param int $id task ID.
	 * @return	Task 
	 */
	public function find($id);

	/**
	 * Returns all task that are part of a project.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findAllProjectTasks($project_id);
	
	/**
	 * Returns all task that are part of a project and are active
	 * in selected year which means task that have some persons
	 * assigned.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findAssignedProjectTasksInYear($project_id, $year);

	/**
	 * Returns all tasks that are part of a project and have
	 * plan for selected year.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findPlannedProjectTasksInYear($project_id, $year);

	/**
	 * Returns all tasks that are active in a project in given year even if they
	 * are not planned for the current year. 
	 * 
	 * Returns only tasks that exist in the system . Deleted are not returned
	 * even if there is some activity (e.g. invoicing or work).
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findActiveProjectTasksInYear($project_id, $year);
	
}


