<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing task plans.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
interface ITaskPlanMapper
{
	/**
	 * Saves task year plan.
	 * 
	 * @param TaskYearPlan $plan
	 * @return boolean
	 */
	public function save(TaskYearPlan $plan);
	
	/**
	 * Updates task year plan.
	 * 
	 * @param TaskYearPlan $plan
	 * @return boolean
	 */
	public function update(TaskYearPlan $plan);
	
	/**
	 * Deletes task year plan by ID.
	 * 
	 * @param int $plan_id
	 * @return boolean
	 */
	public function delete($plan_id);

	/**
	 * Returns task plan by id.
	 * 
	 * @param int $id Task plan ID.
	 * @return	TaskYearPlan or NULL if not found.
	 */
    public function find($id);

	/**
	 * Returns all task year plans by task id.
	 * 
	 * @param int $id Task ID.
	 * @return	array Returns associative array of year plans (key is year), value TaskYearPlan.
	 */
	public function findTaskPlans($id);

	/**
	 * Returns task year plan by task id and year.
	 * 
	 * @param int $id Task ID.
	 * @param int $id year.
	 * @return	TaskYearPlan
	 */
	public function findTaskYearPlan($id, $year);
	
}

