<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class provides basic methods for sending e-mails.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class Mailer extends Object
{
	/** @var DIContainer DI context with application repositories access */
	private $context;
	/** @var boolean Development mode. */
    private $developmentMode;
	/** @var array List of email addresses */
    private $testerList;
	/** @var string Sender email address. */
    protected $commonOptions;
	/** @var SmtpMailer Provider of email services. */
    private $mailer;

	public function __construct($commonOptions)
    {
		$this->context = Environment::getContext();
		
		$this->commonOptions = array();
		$this->commonOptions['from'] = isset($commonOptions['from']) ? $commonOptions['from'] : 'HON Planning Tool <hon.planning.tool@honeywell.com>';
		$this->commonOptions['host'] = isset($commonOptions['host']) ? $commonOptions['host'] : 'smtp.honeywell.com';
		$this->commonOptions['timeout'] = isset($commonOptions['timeout']) ? $commonOptions['timeout'] : 50;
		
		$this->mailer = new SmtpMailer(
			array(
			    'host' => $this->commonOptions['host'],
			    'timeout' => $this->commonOptions['timeout']
		    )
		);
		
		// if development mode is enabled email should be routed to tester list only, NOT APPLICABLE for production server
		if(Environment::getContext()->parameters['serverType'] == 'Production')
			$this->developmentMode = false;
		else
		{
			$this->developmentMode = $this->context->globalSetting->get('mail.developmentMode');
			if ($this->developmentMode)
				$this->testerList = explode(',', $this->context->globalSetting->get('mail.testerList'));
		}
	}
	
	/**
	 * Returns blank template for later use.
	 * @param string $path Path to requested template
	 * @return \Nette\Templating\FileTemplate
	 */
	protected function getTemplate($path)
	{
		$template = new FileTemplate($path);
		$template->registerFilter(new LatteFilter);
		$template->registerHelperLoader('Nette\Templating\Helpers::loader');
		return $template;
	}
	
	/**
	 * Returns new mail object included mailer options
	 * @return \Mail
	 */
	protected function getMail()
	{
		$mail = new Mail;
		$mail->setMailer($this->mailer);
		$mail->setFrom($this->commonOptions['from']);
		return $mail;
	}
	
	/**
	 * Gets context
	 * @return DIContainer
	 */
	public function getContext()
	{
		return $this->context;
	}
	
	/**
	 * Gets development mode
	 * @return boolean
	 */
	public function getDevelopmentMode()
	{
		return $this->developmentMode;
	}
	
	/**
	 * Gets tester list
	 * @return array
	 */
	public function getTesterList()
	{
		return $this->testerList;
	}
}
