<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and licence information, please view
 * the file licence.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing Nationality from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class NationalityDibiMapper extends DibiMapper implements INationalityMapper
{
	/**
	 * Creates Nationality instance from DibiRow.
	 * @param DibiRow $row
	 * @return Nationality
	 */
    private function load(DibiRow $row)
    {
		$nationality = new Nationality();
		$nationality->NationalityID = $row->Nationality_ID;
		$nationality->Name = $row->Name;
		$nationality->Value = $row->Value;
		return $nationality;
    }
	
	/**
	 * Saves Nationality.
	 * @param Nationality $nationality
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(Nationality $nationality)
	{
		try
        {
            $array = array();
            $array['Name'] = $nationality->Name;
            $array['Value'] = $nationality->Value;
            $db_result = $this->conn->insert('Nationality', $array)->execute();
                        $nationality->NationalityID = $this->conn->insertId();
            return TRUE;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving Licence with Name = " . $nationality->Name . "failed. " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Updates Nationality.
	 * @param Nationality $nationality
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(Nationality $nationality)
	{
		try
        {
            $array = array();
            $array['Nationality_ID'] = $nationality->NationalityID;
            $array['Name'] = $nationality->Name;
            $array['Value'] = $nationality->Value;
            $db_result = $this->conn->update('Nationality', $array)->where('[Nationality_ID] = %i', $nationality->NationalityID)->execute();
            return TRUE;
        }
        catch (DibiException $e)
        {
            Debugger::log("Licence update with ID = " . $nationality->NationalityID . " failed. " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Deletes Nationality.
	 * @param int $id Nationality id.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		try
		{
		    $db_result = $this->conn->delete('[Nationality]')->where('[Nationality_ID] = %i', $id)->execute();
			if ($db_result)
				Debugger::log("Licence with ID = " . $id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of Licence with ID = " . $id . " failed.", Debugger::ERROR);
			return ($db_result) ? TRUE : FALSE;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of Licence with ID = " . $id . " failed: " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Finds Nationality by id.
	 * @param int $id Nationality id.
	 * @return Nationality or NULL if not found.
	 */
	public function find($id)
	{
		$db_result = $this->conn->select('*')->from('[Nationality]')->where('[Nationality_ID] = %i', $id)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds all Nationalities.
	 * @return array Array of Nationality or empty array.
	 */
	public function findAll()
	{
		$db_result = $this->conn->select('*')->from('[Nationality]')->fetchAll();     
		$nationality_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $nationality_array[$row->Nationality_ID] = $this->load($row);
        }		
		return $nationality_array;
	}
	
	/**
	 * Finds Nationality by name.
	 * @param string $name
	 * @return Nationality or NULL if not found.
	 */
	public function findByName($name)
	{
		$db_result = $this->conn->select('*')->from('[Nationality]')->where('[Name] = %s', $name)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
}
