<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * POAttributesAndBudget definition.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class POAttributesAndBudget extends Object
{	
	/** @var int ID of record */
	private $poAttributesAndBudgetID;
	/** @var int ID of project */
	private $projectID;
	/** @var int */
	private $year;
	/** @var string */
	private $poValue;
	/** @var string */
	private $poNumber;
	/** @var float */
	private $budget;
	
	/**
     * Sets ID of POAttributesAndBudget.
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
	public function setPOAttributesAndBudgetID($id)
	{
		if ($id > 0)
			$this->poAttributesAndBudgetID = $id;
		else
			throw new InvalidArgumentException('POAttributesAndBudget ID must be integer greater than 0. Wrong param: ' . $id);
	}
	
    /**
     * Gets ID of POAttributesAndBudget.
     * @return int
     */
	public function getPOAttributesAndBudgetID()
	{
		return $this->poAttributesAndBudgetID;
	}
	
	/**
     * Sets ID of project.
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
	public function setProjectID($id)
	{
		if ($id > 0)
			$this->projectID = $id;
		else
			throw new InvalidArgumentException('Project ID must be integer greater than 0. Wrong param: ' . $id);
	}
	
    /**
     * Gets ID of project.
     * @return int
     */
	public function getProjectID()
	{
		return $this->projectID;
	}
	
	/**
     * Sets year.
     * @param int $year
	 * @throws InvalidArgumentException
     */
	public function setYear($year)
	{
	    $this->year = $year;
	}
	
    /**
     * Gets year.
     * @return int
     */
	public function getYear()
	{
		return $this->year;
	}
	
	/**
     * Sets PO value.
     * @param string $value
	 * @throws InvalidArgumentException
     */
	public function setPOValue($value)
	{
		if ($value !== NULL)
	        $this->poValue = 0 + $value;
	}
	
    /**
     * Gets PO value.
     * @return string
     */
	public function getPOValue()
	{
		return $this->poValue;
	}
	
	/**
     * Sets PO number.
     * @param string $number
	 * @throws InvalidArgumentException
     */
	public function setPONumber($number)
	{
	    $this->poNumber = $number;
	}
	
    /**
     * Gets PO number.
     * @return string
     */
	public function getPONumber()
	{
		return $this->poNumber;
	}
	
	/**
     * Sets budget.
     * @param float $amount
	 * @throws InvalidArgumentException
     */
	public function setBudget($amount)
	{
		if ($amount !== NULL)
	        $this->budget = 0 + $amount;
	}
	
    /**
     * Gets budget.
     * @return float
     */
	public function getBudget()
	{
		return $this->budget;
	}
}
