<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing newly found projects stubs in import from iASAP
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PersonFoundDibiMapper extends DibiMapper implements IPersonFoundMapper
{
	/**
	 * Creates PersonFound instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	PersonFound 
	 */
    private function loadPersonFound(DibiRow $row)
    {
		$personFound = new PersonFound();
		$personFound->ID = $row['ID'];
		$personFound->EID = $row['EID'];
		$personFound->Name = $row['Name'];
		$personFound->Surname = $row['Surname'];
		$personFound->WorkCenterID = $row['WorkCenterID'];
		$personFound->SupervisorName = $row['SupervisorName'];
		$personFound->SupervisorEID = $row['SupervisorEID'];
		$personFound->SupervisorSurname = $row['SupervisorSurname'];
		$personFound->OrganizationUnitID = $row['OrganizationUnitID'];
		$personFound->Blacklisted = $row['Blacklisted'];
		return $personFound;
    }	

	/**
	 * Saves found person
	 * 
	 * @param PersonFound $person
	 * @return boolean
	 */
	public function save(PersonFound $person)
	{
        try
        {
            $array = array();
            $array['EID'] = $person->EID;
            $array['Name'] = $person->Name;
			$array['Surname'] = $person->Surname;
			$array['WorkCenterID'] = $person->WorkCenterID;
			$array['SupervisorName'] = $person->SupervisorName;
			$array['SupervisorEID'] = $person->SupervisorEID;
			$array['SupervisorSurname'] = $person->SupervisorSurname;
			$array['OrganizationUnitID'] = $person->OrganizationUnitID;
			$array['Blacklisted'] = $person->Blacklisted;
		    $update_db_result = $this->conn->insert('PersonFound', $array)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving found person with EID id = ".$person->EID." failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
    
	/**
	 * Updates found person
	 * 
	 * @param PersonFound $person
	 * @return boolean
	 */
	public function update(PersonFound $person)
	{
        try
        {
            $array = array();
            $array['EID'] = $person->EID;
            $array['Name'] = $person->Name;
			$array['Surname'] = $person->Surname;
			$array['WorkCenterID'] = $person->WorkCenterID;
			$array['SupervisorName'] = $person->SupervisorName;
			$array['SupervisorEID'] = $person->SupervisorEID;
			$array['SupervisorSurname'] = $person->SupervisorSurname;
			$array['OrganizationUnitID'] = $person->OrganizationUnitID;
			$array['Blacklisted'] = $person->Blacklisted;
            $person_found_db_result = $this->conn->update('PersonFound', $array)->where('[ID] = %i', $person->ID)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of found person with EID id = ".$person->EID." failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Finds person stub by EID.
	 * 
	 * @param string $eid EID.
	 * @return	PersonFound or NULL if not found.
	 */
	public function findByEID($eid)
	{
		$person_db_result = $this->conn->query('SELECT * FROM [PersonFound] WHERE [EID] = %s', $eid)->fetch();
		if ($person_db_result !== FALSE)
			return $this->loadPersonFound($person_db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds all PersonFound.
	 * 
	 * @return array Array of Persons where EID is key.
	 */
	public function findAll()
	{
        $persons_array = array();
		$persons_db_result = $this->conn->query('SELECT * FROM [PersonFound]')->fetchAll();
		foreach ($persons_db_result as $person)
        {
			$persons_array[$person->EID] = $this->loadPersonFound($person);
        }
        return $persons_array;
	}
	
	/**
	 * Finds all infos for found person stubs.
	 * 
	 * @return array Array of PersonFound where EID is key.
	 */
	public function findAllBlacklisted()
	{
        $person_array = array();
		$person_db_result = $this->conn->query('SELECT * FROM [PersonFound] WHERE [Blacklisted]=1')->fetchAll();
		foreach ($person_db_result as $person)
        {
			$person_array[$person->EID] = $this->loadPersonFound($person);
        }
        return $person_array;
	}
	
	/**
	 * Finds all infos for found person stubs.
	 * 
	 * @return array Array of PersonFound where EID is key.
	 */
	public function findAllNonBlacklisted()
	{
        $person_array = array();
		$person_db_result = $this->conn->query('SELECT * FROM [PersonFound] WHERE [Blacklisted]=0')->fetchAll();
		foreach ($person_db_result as $person)
        {
			$person_array[$person->EID] = $this->loadPersonFound($person);
        }
        return $person_array;
	}
	
	/**
	 * Deletes found person stub by EID.
	 * 
	 * @param string $eid found person EID
	 * @return boolean Return true if delete was successfull, otherwise returns false.
	 */
	public function delete($eid)
	{
        try
        {
            $person_db_result = $this->conn->delete('PersonFound')->where('[EID] = %s', $eid)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Found person stub delete with EID = $eid failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
}