<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Represents hours plan assigned to a person for 
 * one month in a year he/she can spend on a task.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class PersonTaskMonthAssignment extends Object
{
	/** @var int Unique assigment ID. */
	private $id = 0;
	/** @var int ID of the task */
	private $taskID = 0;
	/** @var int Must be non-negative integer */
	private $hours = 0;
	/** @var int Must be number from 1 to 12 */
	private $month = 0;
	/** @var int */
	private $year = 0;
	/** @var string */
	private $personID = '';
	/** @var string Date of assignment modification */
	private $modified = '';
	/** @var int */
	private $version = 0;
	
	public function __construct()
	{
	}

   /**
     * Set ID of assignment.
     * 
     * @param int $id Must be positive integer.
     */
	public function setID($id)
	{
		if ($id > 0)
			$this->id = $id;
		else
			throw new InvalidArgumentException('ID must be integer greater than 0. Wrong param: '.$id);
	}
	
    /**
     * Gets ID of assignment.
     * 
     * @return int
     */
	public function getID()
	{
		return $this->id;
	}
	
   /**
     * Set ID of task to which this plan belogs.
     * 
     * @param int $id Must be positive integer.
     */
	public function setTaskID($id)
	{
		if ($id > 0)
			$this->taskID = $id;
		else
			throw new InvalidArgumentException('Task ID must be integer greater than 0. Wrong param: '.$id);
	}
	
    /**
     * Gets ID of task to which this plan belogs.
     * 
     * @return int
     */
	public function getTaskID()
	{
		return $this->taskID;
	}

   /**
     * Sets number of hours assigned to person for a task.
     * 
     * @param int $hours Must be positive integer.
     */
	public function setHours($hours)
	{
		if ($hours >= 0)
			$this->hours = $hours;
		else
			throw new InvalidArgumentException('Hours must be integer greater o equal to 0. Wrong param: '.$hours);
	}
	
    /**
     * Gets number of hours assigned to person for a task.
     * 
     * @return int
     */
	public function getHours()
	{
		return $this->hours;
	}

   /**
     * Sets month.
     * Must be number in range <1,12>.
     * 
     * @param int $month 
     */
	public function setMonth($month)
	{
		if (($month >= 1) && ($month <= 12))
			$this->month = $month;
		else
			throw new InvalidArgumentException('Month must be integer in range <1,12>. Wrong param: '.$month);
	}
	
    /**
     * Gets month.
     * 
     * @return int
     */
	public function getMonth()
	{
		return $this->month;
	}

   /**
     * Sets year.
     * 
     * @param int $year Must be positive integer.
     */
	public function setYear($year)
	{
		if ($year > 0)
			$this->year = $year;
		else
			throw new InvalidArgumentException('Year must be integer greater than 0. Wrong param: '.$year);
	}
	
    /**
     * Gets year.
     * 
     * @return int
     */
	public function getYear()
	{
		return $this->year;
	}
	
   /**
     * Sets ID of a person this assignment belongs to.
     * 
     * @param string $id
     */
	public function setPersonID($id)
	{
		$this->personID = $id;
	}
	
    /**
     * Gets ID of a person this assignment belongs to.
     * 
     * @return string
     */
	public function getPersonID()
	{
		return $this->personID;
	}		


    /**
     * Set date of last modification.
     * 
     * @param string $datetime Date time in YYYY-MM-DD hh:mm:ss format.
	 * @todo Check format of $datetime.
     */
	public function setModified($datetime)
	{
		$this->modified = $datetime;
	}
	
    /**
     * Gets date of last modification.
     * 
     * @return string
     */
	public function getModified()
	{
		return $this->modified;
	}

   /**
     * Sets version.
     * Must be non-negative integer.
     * 
     * @param int $version 
     */
	public function setVersion($version)
	{
		if (is_numeric($version) && ($version >= 0))
			$this->version = 0 + $version;
		else
			throw new InvalidArgumentException('Version must be non-negative integer. Invalid value: ' . $version);
	}
	
    /**
     * Gets version.
     * 
     * @return int
     */
	public function getVersion()
	{
		return $this->version;
	}
	
}


