<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing project charter approver history from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterApproverHistoryDibiMapper extends DibiMapper implements IProjectCharterApproverHistoryMapper
{
    /**
     * Creates ProjectCharterApproverHistory instance from DibiRow. 
     * @param DibiRow $row
     * @return	ProjectCharterApproverHistory
     */
    private function load(DibiRow $row)
    {
        $projectCharterApproverHistory = new ProjectCharterApproverHistory();
        $projectCharterApproverHistory->ProjectCharterID = $row->ProjectCharter_ID;
        $projectCharterApproverHistory->Version = $row->Version;
        $projectCharterApproverHistory->CreationDateTime = $row->CreationDateTime;
        $projectCharterApproverHistory->Responsible = $row->Responsible;
        $projectCharterApproverHistory->LatestVersion = $row->LatestVersion;
        $projectCharterApproverHistory->BudgetVersion = $row->BudgetVersion;
        $projectCharterApproverHistory->FileVersion = $row->FileVersion;
        $projectCharterApproverHistory->EmailContent = $row->EmailContent;
        $projectCharterApproverHistory->FinalApprovalDate = $row->FinalApprovalDate;
        return $projectCharterApproverHistory;
    }
    
        /**
	 * Saves project charter approver history.
	 * 
	 * @param ProjectCharterApproverHistory $projectCharterApproverHistory
	 * @return boolean
	 */
	public function save(ProjectCharterApproverHistory $projectCharterApproverHistory)
	{
                try
                {
                    $array = array();
                    $array['ProjectCharter_ID'] = $projectCharterApproverHistory->getProjectCharterID();
                    $array['Version'] = $projectCharterApproverHistory->getVersion();
                    $array['CreationDateTime'] = $projectCharterApproverHistory->getCreationDateTime();
                    $array['Responsible'] = $projectCharterApproverHistory->getResponsible();
                    $array['LatestVersion'] = $projectCharterApproverHistory->getLatestVersion();
                    $array['BudgetVersion'] = $projectCharterApproverHistory->getBudgetVersion();
                    $array['FileVersion'] = $projectCharterApproverHistory->getFileVersion();
                    $array['EmailContent'] = $projectCharterApproverHistory->getEmailContent();
                    $array['FinalApprovalDate'] = $projectCharterApproverHistory->getFinalApprovalDate();
                    $approver_history_db_result = $this->conn->insert('ProjectCharterApproverHistory', $array)->execute();
                    return true;
                }
                catch (DibiException $e)
                {
                    Debugger::log("Saving project charter approver history with ProjectCharter_ID = " . $projectCharterApproverHistory->getProjectCharterID() . " and version = " . $projectCharterApproverHistory->getVersion() . " failed. " . $e->getMessage(), Debugger::ERROR);
                    return false;
                }
	}
        
        /**
	 * Updates project charter approver history.
	 * 
	 * @param ProjectCharterApproverHistory $projectCharterApproverHistory
         * @return boolean
	 */
	public function update(ProjectCharterApproverHistory $projectCharterApproverHistory)
	{
                try
                {
                    $array = array();
                    $array['ProjectCharter_ID'] = $projectCharterApproverHistory->getProjectCharterID();
                    $array['Version'] = $projectCharterApproverHistory->getVersion();
                    $array['CreationDateTime'] = $projectCharterApproverHistory->getCreationDateTime();
                    $array['Responsible'] = $projectCharterApproverHistory->getResponsible();
                    $array['LatestVersion'] = $projectCharterApproverHistory->getLatestVersion();
                    $array['BudgetVersion'] = $projectCharterApproverHistory->getBudgetVersion();
                    $array['FileVersion'] = $projectCharterApproverHistory->getFileVersion();
                    $array['EmailContent'] = $projectCharterApproverHistory->getEmailContent();
                    $array['FinalApprovalDate'] = $projectCharterApproverHistory->getFinalApprovalDate();
                    $approver_history_db_result = $this->conn->update('ProjectCharterApproverHistory', $array)->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterApproverHistory->getProjectCharterID(), $projectCharterApproverHistory->getVersion())->execute();
                    return true;
                }
                catch (DibiException $e)
                {
                    Debugger::log("Project charter approver history update with ProjectCharter_ID = " . $projectCharterApproverHistory->getProjectCharterID() . " and version = " . $projectCharterApproverHistory->getVersion() . " failed. ".$e->getMessage());
                    return false;
                }
	}
        
        /**
	 * Deletes existing project charter approver history.
         * 
	 * @param int $projectCharterID ProjectCharter ID.
	 * @param int $version Version of round of approval.
	 * @return boolean
	 */
	public function delete($projectCharterID, $version)
	{
		try
		{
		    $result = $this->conn->delete('ProjectCharterApproverHistory')->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterID, $version)->execute();
			if ($result)
				Debugger::log("Project charter approver history with ProjectCharter_ID = " . $projectCharterID . " and version = " . $version . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of project charter approver history with ProjectCharter_ID = " . $projectCharterID . " and version = " . $version . " failed.", Debugger::ERROR);
			return $result;
		}
                catch (DibiException $e)
                {
                    Debugger::log("Deleting of project charter approver history with ProjectCharter_ID = " . $projectCharterID . " and version = " . $version . " failed: ".$e->getMessage(), Debugger::ERROR);
                    return false;
                }
	}
        
        /**
	 * Finds project charter approver history by project charter id and version.
	 * 
	 * @param int $projectCharterID ProjectCharter ID.
	 * @param int $version Version of round of approval.
	 * @return	ProjectCharterApproverHistory or NULL if not found.
	 */
	public function find($projectCharterID, $version)
	{
		$projectCharterApproverHistory_db_result = $this->conn->select('*')->from('ProjectCharterApproverHistory')->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterID, $version)->fetch();
		if ($projectCharterApproverHistory_db_result !== FALSE)
			return $this->load($projectCharterApproverHistory_db_result);
		else
			return NULL;
	}
        
        /**
	 * Finds latest version of project charter approver history by project charter id.
	 * 
         * @param int $projectCharterID ProjectCharter ID.
	 * @return	ProjectCharterApproverHistory or NULL if not found.
	 */
	public function findLatestVersion($projectCharterID)
	{
                $latestVersion = 1; // Flag of latest version is set
		$projectCharterApproverHistory_db_result = $this->conn->select('*')->from('ProjectCharterApproverHistory')->where('[ProjectCharter_ID] = %i AND [LatestVersion] = %i', $projectCharterID, $latestVersion)->fetch();
		if ($projectCharterApproverHistory_db_result !== FALSE)
			return $this->load($projectCharterApproverHistory_db_result);
		else
			return NULL;
	}
        
        /**
         * Initializes flag of latest version
         * 
         * @return boolean
         */
        public function initLatestVersion($projectCharterID)
        {
            try {
                $array = array();
                $array['LatestVersion'] = 0;
                $this->conn->update('ProjectCharterApproverHistory', $array)->where('[ProjectCharter_ID] = %i AND [Version] IS NOT NULL', $projectCharterID)->execute();
                    return true;
            }
            catch (DibiException $e)
            {
                Debugger::log("Initializing of LatestVersion in project charter approve history failed: ".$e->getMessage(), Debugger::ERROR);
                return false;
            }
        }
}
