<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project charter approver history repository access class.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterApproverHistoryRepository
{
    /** @var IProjectCharterApproverHistoryMapper */
    private $mapper;
	
	/** @var MultiLogger */
	private $multilog;
    
    public function __construct(IProjectCharterApproverHistoryMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ProjectCharterApproverHistory-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Project Charter Approver History"));
    }
    
    /**
	 * Saves project charter approver history.
	 * 
	 * @param ProjectCharterApproverHistory $projectCharterApproverHistory
	 * @return boolean
	 */
	public function save(ProjectCharterApproverHistory $projectCharterApproverHistory)
	{
		if($this->mapper->save($projectCharterApproverHistory))
		{
			$this->multilog->log('Project Charter Approver History for charter with ID: ' . $projectCharterApproverHistory->ProjectCharterID . ' and version: ' . $projectCharterApproverHistory->Version . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Updates project charter approver history.
	 * 
	 * @param ProjectCharterApproverHistory $projectCharterApproverHistory
     * @return boolean
	 */
	public function update(ProjectCharterApproverHistory $projectCharterApproverHistory)
	{
		$oldObject = $this->find($projectCharterApproverHistory->ProjectCharterID, $projectCharterApproverHistory->Version);
		if($this->mapper->update($projectCharterApproverHistory))
		{
			$this->multilog->log('Project Charter Approver History for charter with ID: ' . $projectCharterApproverHistory->ProjectCharterID . ' and version: ' . $projectCharterApproverHistory->Version . ' updated. ' . $this->multilog->differences($oldObject, $projectCharterApproverHistory), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing project charter approver history.
     * 
	 * @param int $projectCharterID ProjectCharter ID.
	 * @param int $version Version of round of approval.
	 * @return boolean
	 */
	public function delete($projectCharterID, $version)
	{
		if($this->mapper->delete($projectCharterID, $version))
		{
			$this->multilog->log('Project Charter Approver History for charter with ID: ' . $projectCharterID . ' and version: ' . $version . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Finds project charter approver history by project charter id and version.
	 * 
	 * @param int $projectCharterID ProjectCharter ID.
	 * @param int $version Version of round of approval.
	 * @return	ProjectCharterApproverHistory or NULL if not found.
	 */
	public function find($projectCharterID, $version)
	{
		return $this->mapper->find($projectCharterID, $version);
	}
        
	/**
	 * Finds latest version of project charter approver history by project charter id.
	 * 
     * @param int $projectCharterID ProjectCharter ID.
	 * @return	ProjectCharterApproverHistory or NULL if not found.
	 */
	public function findLatestVersion($projectCharterID)
	{
		return $this->mapper->findLatestVersion($projectCharterID);
	}
        
	/**
	 * Initializes flag of latest version
	 * 
	 * @return boolean
	 */
	public function initLatestVersion($projectCharterID)
	{
		return $this->mapper->initLatestVersion($projectCharterID);
	}
}
