<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing project charter budget from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterBudgetDibiMapper extends DibiMapper implements IProjectCharterBudgetMapper
{
	/**
	 * Creates ProjectCharterBudget instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	ProjectCharterBudget
	 */
	private function load(DibiRow $row)
	{
		$projectCharterBudget = new ProjectCharterBudget();
		$projectCharterBudget->ID = $row->ProjectCharterBudget_ID;
		$projectCharterBudget->ProjectCharterID = $row->ProjectCharter_ID;
		$projectCharterBudget->Version = $row->Version;
		$projectCharterBudget->ResourceType = $row->ResourceType;
		$projectCharterBudget->TechnicalGroup = $row->TechnicalGroup;
		$projectCharterBudget->Subcontractor = $row->Subcontractor;
		$projectCharterBudget->Rate = $row->Rate;
		$projectCharterBudget->BudgetQ1 = $row->BudgetQ1;
		$projectCharterBudget->BudgetQ2 = $row->BudgetQ2;
		$projectCharterBudget->BudgetQ3 = $row->BudgetQ3;
		$projectCharterBudget->BudgetQ4 = $row->BudgetQ4;
		return $projectCharterBudget;
	}

	/**
	 * Saves project charter budget.
	 * 
	 * @param ProjectCharterBudget $projectCharterBudget
	 */
	public function save(ProjectCharterBudget $projectCharterBudget)
	{
		try
		{
			$array = array();
			$array['ProjectCharter_ID'] = $projectCharterBudget->getProjectCharterID();
			$array['Version'] = $projectCharterBudget->getVersion();
			$array['ResourceType'] = $projectCharterBudget->getResourceType();
			$array['TechnicalGroup'] = $projectCharterBudget->getTechnicalGroup();
			$array['Subcontractor'] = $projectCharterBudget->getSubcontractor();
			$array['Rate'] = $projectCharterBudget->getRate();
			$array['BudgetQ1'] = $projectCharterBudget->getBudgetQ1();
			$array['BudgetQ2'] = $projectCharterBudget->getBudgetQ2();
			$array['BudgetQ3'] = $projectCharterBudget->getBudgetQ3();
			$array['BudgetQ4'] = $projectCharterBudget->getBudgetQ4();
			$budget_db_result = $this->conn->insert('ProjectCharterBudget', $array)->execute();
			$projectCharterBudget->ID = $this->conn->insertId();
			return true;
		}
		catch (DibiException $e)
		{
			Debugger::log("Saving ProjectCharterBudget failed. " . $e->getMessage(), Debugger::ERROR);
			return false;
		}
	}

	/**
	 * Updates project charter budget.
	 * 
	 * @param ProjectCharterBudget $projectCharterBudget
	 */
	public function update(ProjectCharterBudget $projectCharterBudget)
	{
		try
		{
			$array = array();
			$array['ProjectCharter_ID'] = $projectCharterBudget->getProjectCharterID();
			$array['Version'] = $projectCharterBudget->getVersion();
			$array['ResourceType'] = $projectCharterBudget->getResourceType();
			$array['TechnicalGroup'] = $projectCharterBudget->getTechnicalGroup();
			$array['Subcontractor'] = $projectCharterBudget->getSubcontractor();
			$array['Rate'] = $projectCharterBudget->getRate();
			$array['BudgetQ1'] = $projectCharterBudget->getBudgetQ1();
			$array['BudgetQ2'] = $projectCharterBudget->getBudgetQ2();
			$array['BudgetQ3'] = $projectCharterBudget->getBudgetQ3();
			$array['BudgetQ4'] = $projectCharterBudget->getBudgetQ4();
			$budget_db_result = $this->conn->update('ProjectCharterBudget', $array)->where('[ProjectCharterBudget_ID] = %i', $projectCharterBudget->getID())->execute();
			return true;
		}
		catch (DibiException $e)
		{
			Debugger::log("ProjectCharterBudget update with ID = " . $projectCharterBudget->getID() . " failed. ".$e->getMessage());
			return false;
		}
	}
	
	/**
	 * Deletes existing project charter budget.
	 * 
	 * @param int $projectCharterBudget_id
	 * @return boolean
	 */
	public function delete($projectCharterBudget_id)
	{
		try
		{
			$result = $this->conn->delete('ProjectCharterBudget')->where('[ProjectCharterBudget_ID] = %i', $projectCharterBudget_id)->execute();
			if ($result)
				Debugger::log("Project charter budget with ID = " . $projectCharterBudget_id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of project charter budget with ID = " . $projectCharterBudget_id . " failed.", Debugger::ERROR);
			return $result;
		}
		catch (DibiException $e)
		{
			Debugger::log("Deleting of project charter budget with ID = " . $projectCharterBudget_id . " failed: ".$e->getMessage(), Debugger::ERROR);
			return false;
		}
	}
	
	/**
	 * Deletes all Project Charter Budgets with specified Project Charter ID and Version.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return boolean
	 */
	public function deleteByProjectCharterIDAndVersion($projectCharterID, $version)
	{
		try
		{
			$result = $this->conn->delete('ProjectCharterBudget')->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterID, $version)->execute();
			if ($result)
				Debugger::log("Project charter budgets with Charter ID = " . $projectCharterID . " and Version = " . $version . " were deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of project charter budgets with Charter ID = " . $projectCharterID . " and Version = " . $version . " failed.", Debugger::ERROR);
			return $result;
		}
		catch (DibiException $e)
		{
			Debugger::log("Deleting of project charter budgets with Charter ID = " . $projectCharterID . " and Version = " . $version . " failed." . $e->getMessage(), Debugger::ERROR);
			return false;
		}
	}

	/**
	 * Finds project charter buget by id.
	 * 
	 * @param int $id project charter budget ID.
	 * @return	ProjectCharterBudget or NULL if not found.
	 */
	public function find($id)
	{
		$projectCharterBudget_db_result = $this->conn->select('*')->from('ProjectCharterBudget')->where('[ProjectCharterBudget_ID] = %i', $id)->fetch();
		if ($projectCharterBudget_db_result !== FALSE)
			return $this->load($projectCharterBudget_db_result);
		else
			return NULL;
	}

	/**
	 * Finds all project charter budgets.
	 * 
	 * @param int $projectCharterID Should be ID of existing project charter.
	 * @param int $version Version of ProjectCharterBudgetHistory.
	 * @return array Array of ProjectCharterBudget.
	 */
	public function findByProjecCharterIDAndVersion($projectCharterID, $version)
	{
			$projectCharterBudget_array = array();
			$projecCharterBudget_db_result = $this->conn->select('*')->from('ProjectCharterBudget')->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterID, $version)->fetchAll();
			$i = 0;
			foreach ($projecCharterBudget_db_result as $charterBudget)
			{
				$projectCharterBudget_array[$i] = $this->load($charterBudget);
				$i++;
			}
			return $projectCharterBudget_array;
	}
	
	/**
	 * Finds latest veriosn of project charter budget by project charter id.
	 * 
	 * @param int $projectCharterID Project charter ID.
	 * @return	array Array of ProjectCharterBudget where ProjectCharterBudget_Id is key or NULL if not found.
	 */
	public function findProjectCharterBudgetByProjectCharterID($projectCharterID)
	{
		$projectCharterBudget_db_result = $this->conn->query('SELECT * FROM [ProjectCharterBudget] WHERE [ProjectCharter_ID] = %i AND 
																[Version] = (SELECT [Version] FROM [ProjectCharterBudgetHistory] WHERE [ProjectCharter_ID] = [ProjectCharterBudget].[ProjectCharter_ID] AND [LatestVersion] = %i)', $projectCharterID, 1)->fetchAll();
		$projectCharterBudget_array = array();
		if ($projectCharterBudget_db_result !== FALSE)
		{
			foreach ($projectCharterBudget_db_result as $projectCharterBudget)
				$projectCharterBudget_array[$projectCharterBudget->ProjectCharterBudget_ID] = $this->load($projectCharterBudget);
			return $projectCharterBudget_array;
		}
		else
			return null;
	}

	/**
	 * Finds project charter budget by Organization unit id.
	 * 
	 * @param int $OrgUnit_ID Organizaiton unit ID.
	 * @return	array Array of ProjectCharterBudget where ProjectCharterBudget_Id is key or NULL if not found.
	 */
	public function findProjectCharterBudgetByOrgUnitId($OrgUnit_ID)
	{
		$projectCharterBudget_db_result = $this->conn->select('*')->from('ProjectCharterBudget')->where('[TechnicalGroup] = %i', $OrgUnit_ID)->fetchAll();
		$projectCharterBudget_array = array();
		if ($projectCharterBudget_db_result !== FALSE)
		{
			foreach ($projectCharterBudget_db_result as $projectCharterBudget)
				$projectCharterBudget_array[$projectCharterBudget->ProjectCharterBudget_ID] = $this->load($projectCharterBudget);
			return $projectCharterBudget_array;
		}
		else
			return null;
	}

	/**
	 * Finds latest veriosn of project charter budget where is resource type HRS by Organization unit id.
	 * 
	 * @param int $OrgUnit_ID Organizaiton unit ID.
	 * @return	array Array of ProjectCharterBudget where ProjectCharterBudget_Id is key or NULL if not found.
	 */
	public function findProjectCharterBudgetHRSByOrgUnitId($OrgUnit_ID)
	{
		$projectCharterBudget_db_result = $this->conn->query('SELECT * FROM [ProjectCharterBudget] WHERE [TechnicalGroup] = %i AND [ResourceType] = %s AND 
																[Version] = (SELECT [Version] FROM [ProjectCharterBudgetHistory] WHERE [ProjectCharter_ID] = [ProjectCharterBudget].[ProjectCharter_ID] AND [LatestVersion] = %i)', $OrgUnit_ID, 'HRS', 1)->fetchAll();
		$projectCharterBudget_array = array();
		if ($projectCharterBudget_db_result !== FALSE)
		{
			foreach ($projectCharterBudget_db_result as $projectCharterBudget)
				$projectCharterBudget_array[$projectCharterBudget->ProjectCharterBudget_ID] = $this->load($projectCharterBudget);
			return $projectCharterBudget_array;
		}
		else
			return null;
	}

	/**
	 * Finds all budget items which was created in latest version of given responsible person
	 *
	 * @param int $charterId
	 * @param string $eid
	 * @return array array(int[project charter budget id] => ProjecCharterBudget)
	 */
	public function findLatestByCharterIdAndResponsible($charterId, $eid)
	{
		$dbResult = $this->conn->query('
			SELECT [ProjectCharterBudget].*
			FROM [ProjectCharterBudget]
			INNER JOIN [ProjectCharterBudgetHistory] ON [ProjectCharterBudgetHistory.ProjectCharter_ID] = [ProjectCharterBudget.ProjectCharter_ID]
														AND [ProjectCharterBudgetHistory.Version] = [ProjectCharterBudget.Version]
			WHERE [ProjectCharterBudgetHistory.ProjectCharter_ID] = %i
				  AND [ProjectCharterBudgetHistory.Responsible] = %s
				  AND [ProjectCharterBudgetHistory.Version] = (SELECT MAX([Version])
															   FROM [ProjectCharterBudgetHistory]
															   WHERE [ProjectCharter_ID] = %i
																	 AND [Responsible] = %s)
		', $charterId, $eid, $charterId, $eid)
			->fetchAll();

		$budgetItems = array();
		foreach ($dbResult as $row) {
			$budgetItems[$row->ProjectCharterBudget_ID] = $this->load($row);
		}

		return $budgetItems;
	}
}
