<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing project charter from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterDibiMapper extends DibiMapper implements IProjectCharterMapper
{
	/**
	 * Creates ProjectCharter instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	ProjectCharter
	 */
	private function load(DibiRow $row)
	{
		$projectCharter = new ProjectCharter();
		$projectCharter->ID = $row->ProjectCharter_ID;
		$projectCharter->SAP_ID = $row->SAP_ID;
		$projectCharter->Name = $row->Name;
		$projectCharter->Owner = $row->Owner;
		$projectCharter->LineManager = $row->LineManager;
		$projectCharter->Status = $row->Status;
		$projectCharter->CreationDateTime = $row->CreationDateTime;
		$projectCharter->CostShare = $row->CostShare;
		$projectCharter->CustomerID = $row->Customer_ID;
		$projectCharter->ProgramID = $row->Program_ID;
		$projectCharter->CallID = $row->Call_ID;
		$projectCharter->ResponsibilityID = $row->Responsibility_ID;
		$projectCharter->Year = $row->Year;
		$projectCharter->DaysForFinalApproval = $row->DaysForFinalApproval;
		$projectCharter->ECLicenceID = $row->ECLicence_ID;
		$projectCharter->OrganizationUnitID = $row->OrganizationUnit_ID;
		$projectCharter->StrapItemID = $row->StrapItem_ID;
		return $projectCharter;
	}

	/**
	 * Saves project charter.
	 * 
	 * @param ProjectCharter $projectCharter
	 */
	public function save(ProjectCharter $projectCharter)
	{
		try
		{
			$array = array();
			$array['SAP_ID'] = $projectCharter->getSAP_ID();
			$array['Name'] = $projectCharter->getName();
			$array['Owner'] = $projectCharter->getOwner();
			$array['LineManager'] = $projectCharter->getLineManager();
			$array['Status'] = $projectCharter->getStatus();
			$array['CreationDateTime'] = $projectCharter->getCreationDateTime();
			$array['CostShare'] = $projectCharter->getCostShare();
			$array['Customer_ID'] = $projectCharter->getCustomerID();
			$array['Program_ID'] = $projectCharter->getProgramID();
			$array['Call_ID'] = $projectCharter->getCallID();
			$array['Responsibility_ID'] = $projectCharter->getResponsibilityID();
			$array['Year'] = $projectCharter->getYear();
			$array['DaysForFinalApproval'] = $projectCharter->getDaysForFinalApproval();
			$array['ECLicence_ID'] = $projectCharter->getECLicenceID();
			$array['StrapItem_ID'] = $projectCharter->getStrapItemID();
			$orgUnit = $this->conn->query('SELECT [OrganizationUnit_ID] FROM [OrganizationUnit] WHERE [Leader] = %s', $projectCharter->getLineManager())->fetch();
			if($orgUnit !== FALSE)
				$array['OrganizationUnit_ID'] = $orgUnit['OrganizationUnit_ID'];
			else
			{
				$sbesConfig = (array)Environment::getConfig('charter_sbes');
				$array['OrganizationUnit_ID'] = current($sbesConfig);
			}
			$charter_db_result = $this->conn->insert('ProjectCharter', $array)->execute();
						$projectCharter->setID($this->conn->insertId());
			return true;
		}
		catch (DibiException $e)
		{
			// TODO is SAP_ID filled every time?
			Debugger::log("Saving project charter with Name = " . $projectCharter->getName() . " failed. " . $e->getMessage(), Debugger::ERROR);
			return false;
		}
	}

	/**
	 * Updates project charter.
	 * 
	 * @param ProjectCharter $projectCharter
	 */
	public function update(ProjectCharter $projectCharter)
	{
		try
		{
			$array = array();
			$array['SAP_ID'] = $projectCharter->getSAP_ID();
			$array['Name'] = $projectCharter->getName();
			$array['Owner'] = $projectCharter->getOwner();
			$array['LineManager'] = $projectCharter->getLineManager();
			$array['Status'] = $projectCharter->getStatus();
			$array['CostShare'] = $projectCharter->getCostShare();
			$array['Customer_ID'] = $projectCharter->getCustomerID();
			$array['Program_ID'] = $projectCharter->getProgramID();
			$array['Call_ID'] = $projectCharter->getCallID();
			$array['Responsibility_ID'] = $projectCharter->getResponsibilityID();
			$array['Year'] = $projectCharter->getYear();
			$array['DaysForFinalApproval'] = $projectCharter->getDaysForFinalApproval();
			$array['ECLicence_ID'] = $projectCharter->getECLicenceID();
			$array['StrapItem_ID'] = $projectCharter->getStrapItemID();
				
			$orgUnit = $this->conn->query('SELECT [OrganizationUnit_ID] FROM [OrganizationUnit] WHERE [Leader] = %s', $projectCharter->getLineManager())->fetch();
			if($orgUnit !== FALSE)
				$array['OrganizationUnit_ID'] = $orgUnit['OrganizationUnit_ID'];
			else
			{
				$sbesConfig = (array)Environment::getConfig('charter_sbes');
				$array['OrganizationUnit_ID'] = current($sbesConfig);
			}
			$charter_db_result = $this->conn->update('ProjectCharter', $array)->where('[ProjectCharter_ID] = %i', $projectCharter->ID)->execute();
			return true;
		}
		catch (DibiException $e)
		{
			Debugger::log("Project charter update with id = " . $projectCharter->ID . " failed. ".$e->getMessage());
			return false;
		}
	}
	
	/**
	 * Deletes existing project charter.
	 * 
	 * @param int $projectCharter_id
	 * @return boolean
	 */
	public function delete($projectCharter_id)
	{
		try
		{
			$result = $this->conn->delete('ProjectCharter')->where('[ProjectCharter_ID] = %i', $projectCharter_id)->execute();
			if ($result)
				Debugger::log("Project charter with ID = " . $projectCharter_id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of project charter with ID = " . $projectCharter_id . " failed.", Debugger::ERROR);
			return $result;
		}
		catch (DibiException $e)
		{
			Debugger::log("Deleting of project charter with ID = " . $projectCharter_id . " failed: ".$e->getMessage(), Debugger::ERROR);
			return false;
		}
	}

	/**
	 * Finds project charter by id.
	 * 
	 * @param int $id project charter ID.
	 * @return	ProjectCharter or NULL if not found.
	 */
	public function find($id)
	{
		$projectCharter_db_result = $this->conn->select('*')->from('ProjectCharter')->where('[ProjectCharter_ID] = %i', $id)->fetch();
		if ($projectCharter_db_result !== FALSE)
			return $this->load($projectCharter_db_result);
		else
			return NULL;
	}

	/**
	 * Gets ProjectCharter by ProjectCharterFilter. Returns every project charters that matches all
	 * set conditions.
	 *
	 * @param ProjectCharterFilter $charterFilter
	 * @return array of ProjectCharter
	*/
	public function getChartersByFilter(ProjectCharterFilter $charterFilter)
	{
		$db_result = $this->conn->select('*')->from('[ProjectCharter]');

		if ($charterFilter->getName())
			$db_result->where("[Name] LIKE %~like~", $charterFilter->getName());
		if ($charterFilter->getOwner())
			$db_result->where("[Owner] LIKE %~like~", $charterFilter->getOwner());
		if ($charterFilter->getSapID())
			$db_result->where("[SAP_ID] LIKE %~like~", $charterFilter->getSapID());
		if ($charterFilter->getLineManager())
			$db_result->where("[LineManager] LIKE %~like~", $charterFilter->getLineManager());
		if ($charterFilter->getStatus())
		{
			$statuses = $charterFilter->getStatus();
			if (!in_array('', $statuses)) // star selected == all statuses, so do not filter according to status
			{
				$arrStatusCondition = array();
				foreach ($statuses as $status)
					array_push($arrStatusCondition, array("[Status] = %s", $status));
				// add whole condition for statuses
				$db_result->where('(%or)', $arrStatusCondition);
			}
		}
		if ($charterFilter->getYear())
		{
			if(is_array($charterFilter->getYear()))
			{
				$year = $charterFilter->getYear();
				$db_result->where("[Year] IN %in", $year[0]);
			}
			else
				$db_result->where("[Year] = %i", $charterFilter->getYear());
		}
		if ($charterFilter->getCustomerID())
			$db_result->where("[Customer_ID] = %i", $charterFilter->getCustomerID());
		if ($charterFilter->getProgramID())
			$db_result->where("[Program_ID] = %i", $charterFilter->getProgramID());
		if ($charterFilter->getCallID())
			$db_result->where("[Call_ID] = %i", $charterFilter->getCallID());
		if ($charterFilter->getResponsibilityID())
			$db_result->where("[Responsibility_ID] = %i", $charterFilter->getResponsibilityID());

		$db_result->orderBy('[Name], [Status]');

		$db_result->fetchAll();

		$arrayOfCharters = array();
		foreach($db_result as $row)
			$arrayOfCharters[] = $this->load($row);

		return $arrayOfCharters;
	}

	/**
	 * Finds project charters by Organization Unit id.
	 * 
	 * @param int $id Organization Unit ID.
	 * @param int $year Year.
	 * @return	array Array of ProjectCharter where key is ProjectCharter_ID or NULL if not found.
	 */
	public function findProjectCharterByOrgUnitID($id, $year)
	{
		$projectCharter_db_result = $this->conn->query('SELECT [ProjectCharter].* FROM [ProjectCharter]
														LEFT JOIN [ProjectCharterBudget] ON [ProjectCharter].[ProjectCharter_ID] = [ProjectCharterBudget].[ProjectCharter_ID]
														LEFT JOIN [OrganizationUnit] ON [OrganizationUnit].[OrganizationUnit_ID] = [ProjectCharterBudget].[TechnicalGroup]
														WHERE [OrganizationUnit].[OrganizationUnit_ID] = %i AND [ProjectCharter].[Year] = %i', $id, $year)->fetchAll();
		$projectCharter_array = array();
		if ($projectCharter_db_result !== FALSE)
		{
			foreach ($projectCharter_db_result as $projectCharter)
				$projectCharter_array[$projectCharter->ProjectCharter_ID] = $this->load($projectCharter);
			return $projectCharter_array;
		}
		else
			return null;
	}

	/**
	 * Returns all years for that has been planned.
	 * @return array arr[$year] == $year or empty array
	*/
	public function getYears()
	{
		$year_db_result = $this->conn->query('SELECT DISTINCT [Year] FROM [ProjectCharter] ORDER BY [Year]')->fetchAll();

		$years = array();
		if ($year_db_result !== FALSE)
		{
			foreach ($year_db_result as $year)
				$years[strval($year->Year)] = strval($year->Year);
			return $years;
		}
		else
			return array();
	}

	/**
	 * Finds project charters by line manager.
	 * 
	 * @param int $id EID of line manager.
	 * @param int $year Year.
	 * @return	array Array of ProjectCharter where key is ProjectCharter_ID or NULL if not found.
	*/
	public function findProjectCharterByLM($id, $year)
	{
		$projectCharter_db_result = $this->conn->select('*')->from('ProjectCharter')->where('[LineManager] = %s AND [Year] = %i', $id, $year)->fetchAll();
		$projectCharter_array = array();
		if ($projectCharter_db_result !== FALSE)
		{
			foreach ($projectCharter_db_result as $projectCharter)
				$projectCharter_array[$projectCharter->ProjectCharter_ID] = $this->load($projectCharter);
			return $projectCharter_array;
		}
		else
			return null;
	}
	
	/**
	 * Finds planning year of charter specified by its ID.
	 * 
	 * @param int $projectCharterID
	 * @return mixed ProjectCharter or NULL
	 */
	public function findYearByProjectCharterID($projectCharterID)
	{
		$projectPlanningCharter_db_result = $this->conn->select('Year')
				->from('ProjectCharter')->where('[ProjectCharter_ID] = %i', $projectCharterID)->fetch();
		if ($projectPlanningCharter_db_result !== FALSE)
			return $projectPlanningCharter_db_result['Year'];
		else
			return NULL;
	}
	
	/**
	 * Finds all project charters in year.
	 * 
	 * @param int $year Year.
	 * @return	array Array of ProjectCharter where key is ProjectCharter_ID or NULL if not found.
	*/
	public function findAll($year)
	{
		$projectCharter_db_result = $this->conn->select('*')->from('ProjectCharter')->where('[Year] = %i', $year)->fetchAll();
		$projectCharter_array = array();
		if ($projectCharter_db_result !== FALSE)
		{
			foreach ($projectCharter_db_result as $projectCharter)
				$projectCharter_array[$projectCharter->ProjectCharter_ID] = $this->load($projectCharter);
			return $projectCharter_array;
		}
		else
			return null;
	}
}
