<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * ProjectCharterFileHistory.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterFileHistory extends Object
{
    /** @var int ID */
    private $projectCharterID;
    /** @var int version number */
    private $version;
    /** @var string */
    private $creationDateTime;
    /** @var string */
    private $fileName;
    /** @var string EID of responsible person */
    private $responsible;
    /** @var string */
    private $ownerComment;
    /** @var string */
    private $lineManagerComment;
    /** @var int Flag determining latest version of history. */
    private $latestVersion;
	/** @var int Flag determining moment when to create new version of history. */
	private $committed;
    
    /**
     * Set ID of project charter.
     * @param int $id Must be positive integer.
     * @throws InvalidArgumentException if parameter is not integer greater than 0.
     */
        public function setProjectCharterID($id)
        {
            if ($id > 0)
                $this->projectCharterID = $id;
            else
                throw new InvalidArgumentException('Project charter ID must be integer greater than 0. Wrong param: '.$id);
        }
        
    /**
     * Gets ID of project charter.
     * @return int
     */
        public function getProjectCharterID()
        {
            return $this->projectCharterID;
        }
        
    /**
     * Set version of project charter file.
     * @param int $version Must be positive integer.
     * @throws InvalidArgumentException if parameter is not integer greater than 0.
     */
        public function setVersion($version)
        {
            if ($version > 0)
                $this->version = $version;
            else
                throw new InvalidArgumentException('Version of project charter file must be integer greater than 0. Wrong param: '.$version);
        }
        
    /**
     * Gets version of project charter file.
     * @return int
     */
        public function getVersion()
        {
            return $this->version;
        }
        
    /**
     * Set creation date and time of project charter file.
     * @param string $creationDateTime Date in YYYY-MM-DD HH:MM:SS format.
         * @todo Check format of $creationDateTime.
     */
        public function setCreationDateTime($creationDateTime)
        {
            $this->creationDateTime = $creationDateTime;
        }
        
    /**
     * Gets creation date and time of project charter file.
     * @return string
     */
        public function getCreationDateTime()
        {
            return $this->creationDateTime;
        }
        
    /**
     * Set name of project charter file.
     * @param string $fileName
     */
        public function setFileName($fileName)
        {
            $this->fileName = $fileName;
        }
        
    /**
     * Gets name of project charter file.
     * @return string
     */
        public function getFileName()
        {
            return $this->fileName;
        }
        
    /**
     * Set EID of responsible person for project charter file.
     * @param string $eid
     */
        public function setResponsible($eid)
        {
            $this->responsible = Person::processEID($eid);
        }
        
    /**
     * Gets EID of responsible person for project charter file.
     * @return string
     */        
        public function getResponsible()
        {
            return $this->responsible;
        }
        
    /**
     * Set owner's comment to current version.
     * @param string $comment
     */
        public function setOwnerComment($comment)
        {
            $this->ownerComment = $comment;
        }
        
    /**
     * Gets owner's comment to current version.
     * @return string
     */
        public function getOwnerComment()
        {
            return $this->ownerComment;
        }
        
    /**
     * Set manager's comment to current version.
     * @param string $comment
     */
        public function setLineManagerComment($comment)
        {
            $this->lineManagerComment = $comment;
        }
        
    /**
     * Gets manager's comment to current version.
     * @return string
     */
        public function getLineManagerComment()
        {
            return $this->lineManagerComment;
        }
            
    /**
     * Set latest version.
     * @param bool $enabled
     */
        public function setLatestVersion($enabled)
        {
            $this->latestVersion = ($enabled) ? 1 : 0;
        }
        
    /**
     * Gets latest version.
     * @return int
     */
        public function getLatestVersion()
        {
            return $this->latestVersion;
        }
            
    /**
     * Set committed.
     * @param bool $enabled
     */
        public function setCommitted($enabled)
        {
            $this->committed = ($enabled) ? 1 : 0;
        }
        
    /**
     * Gets commited.
     * @return int
     */
        public function getCommitted()
        {
            return $this->committed;
        }
    /**
     * Gets commited.
     * @return bool
     */
        public function getCommittedBool()
        {
            return ($this->committed) ? TRUE : FALSE;
        }     
}
