<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * An interface for accessing planning charter state.
 *
 * @author Leos Navratil <leos.navratil@honeywell.com>
  */
interface IProjectPlanningCharterStateMapper
{
	/**
	 * Saves project planning state.
	 * 
     * @param ProjectPlanningCharterState $projectPlanningCharterState Project planning charter state
	 * @return boolean
	 */
	public function save(ProjectPlanningCharterState $projectPlanningCharterState);

	/**
	 * Updates project planning state.
	 * 
     * @param ProjectPlanningCharterState $projectPlanningCharterState Project planning charter state
	 * @return boolean
	 */
	public function update(ProjectPlanningCharterState $projectPlanningCharterState);
	
	/**
	 * Deletes existing project planning state.
	 * 
	 * @param int $projectID Project's ID
	 * @param int $year Year
	 * @return boolean
	 */
	public function delete($projectID, $year);

	/**
	 * Returns project planning charter state by its Project ID and Year.
	 * 
	 * @param int $projectID Project's ID
	 * @param int $year Year
	 * @return ProjectPlanningCharterState
	 */
	public function find($projectID, $year);
	
	/**
	 * Returns project planning charter state by Project Charter ID and Year.
	 * 
	 * @param int $projectCharterID Project Charter's ID
	 * @param int $year Year
	 * @return ProjectPlanningCharterState
	 */
	public function findByProjectCharterIDAndYear($projectCharterID, $year);
	
	/**
	 * Returns all project planning charter states.
	 * 
	 * @return array of ProjectPlanningCharterState or empty array if not found
	 */
	public function findAll();
}
