<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project planning charter item budget repository access class.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ProjectPlanningCharterItemBudgetRepository extends Object
{
	/** @var IProjectPlanningCharterItemBudgetMapper */
	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

	public function __construct(IProjectPlanningCharterItemBudgetMapper $mapper)
	{
		$this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ProjectPlanningCharterItemBudget-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Project Planning Charter Item Budget"));
	}
	
	/**
	 * Saves project planning charter item budget.
	 * 
     * @param ProjectPlanningCharterItemBudget $projectPlanningCharterItemBudget Project planning charter budget
	 * @return boolean
	 */
	public function save(ProjectPlanningCharterItemBudget $projectPlanningCharterItemBudget)
	{
		if($this->mapper->save($projectPlanningCharterItemBudget))
		{
			$this->multilog->log('Project Planning Charter Item budget with ID: ' . $projectPlanningCharterItemBudget->ID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Updates project planning charter item budget.
	 * 
	 * @param ProjectPlanningCharterItemBudget $projectPlanningCharterItemBudget Project planning charter item budget
	 * @return boolean
	 */
	public function update(ProjectPlanningCharterItemBudget $projectPlanningCharterItemBudget)
	{
		$oldObject = $this->find($projectPlanningCharterItemBudget->ID);
		if($this->mapper->update($projectPlanningCharterItemBudget))
		{
			$this->multilog->log('Project Planning Charter Item Budget with ID: ' . $projectPlanningCharterItemBudget->ID . ' updated. ' . $this->multilog->differences($oldObject, $projectPlanningCharterItemBudget), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing project planning charter item budget.
	 * 
	 * @param int $projectPlanningCharterItemBudget_id
	 * @return boolean
	 */
	public function delete($projectPlanningCharterItemBudget_id)
	{
		if($this->mapper->delete($projectPlanningCharterItemBudget_id))
		{
			$this->multilog->log('Project planning charter item budget with ID: ' . $projectPlanningCharterItemBudget_id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Returns project planning charter item budget by its id.
	 * 
	 * @param int $id ProjectPlanningCharterItemBudget ID.
	 * @return ProjectPlanningCharterItemBudget
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}
	
	/**
	 * Returns project planning charter item budget by project planning charter item id.
	 * The indexes in returned array are month numbers e.g. 1-12.
	 * 
	 * @param int $projectPlanningCharterItemID ProjectPlanningCharterItem ID.
	 * @return array of ProjectPlanningCharterItemBudget or empty array if not found
	 */
	public function findByProjectPlanningCharterItemID($projectPlanningCharterItemID)
	{
		return $this->mapper->findByProjectPlanningCharterItemID($projectPlanningCharterItemID);
	}

	/**
	 * Returns all project planning charter item budgets.
	 * The indexes in returned array are month numbers e.g. 1-12.
	 * 
	 * @return array of ProjectPlanningCharterItemBudget or empty array if not found
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
}
