<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of newly found projects stubs from import from SAP PMB files.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class ProjectFoundRepository extends Object
{
    /** @var IProjectFoundMapper */
 	private $mapper;

    public function __construct(IProjectFoundMapper $mapper)
    {
        return $this->mapper = $mapper;
    }	

	/**
	 * Saves project stub.
	 * 
	 * @param ProjectFound $project
	 * @return boolean
	 */
	public function save(ProjectFound $project)
	{
		return $this->mapper->save($project);
	}
    
	/**
	 * Updates project stub.
	 * 
	 * @param ProjectFound $project
	 * @return boolean
	 */
	public function updateInfo(ProjectFoundInfo $project)
	{
		return $this->mapper->updateInfo($project);
	}
	
	/**
	 * Finds project stub by SAP id including plans if any.
	 * 
	 * @param string $sap_id SAP ID.
	 * @return	ProjectFound or NULL if not found.
	 */
	public function findBySAPID($sap_id)
	{
		return $this->mapper->findBySAPID($sap_id);
	}

			/**
	 * Finds project stub by SAP id.
	 * 
	 * @param string $sap_id SAP ID.
	 * @return	ProjectFound or NULL if not found.
	 */
	public function findInfoBySAPID($sap_id)
	{
		return $this->mapper->findInfoBySAPID($sap_id);
	}
	
	/**
	 * Finds all founds project stubs.
	 * 
	 * @return array Array of Persons where EID is key.
	 */
	public function findAllInfos()
	{
		return $this->mapper->findAllInfos();
	}
	
	/**
	 * Finds all founds blacklisted project stubs.
	 * 
	 * @return array Array of Persons where EID is key.
	 */
	public function findAllBlacklistedInfos()
	{
		return $this->mapper->findAllBlacklistedInfos();
	}
	
	/**
	 * Deletes found project stubs by id.
	 * 
	 * @param string $id founds project stubs SAP ID.
	 * @return boolean Return true if delete was successfull, otherwise returns false.
	 */
	public function delete($sap_id)
	{
		return $this->mapper->delete($sap_id);
	}

	/**
	 * Puts project stub identified by SAP ID on blacklisted.
	 * 
	 * @return	boolean
	 */
	public function putOnBlacklist($sap_id)
	{
		return $this->mapper->putOnBlacklist($sap_id);
	}

	/**
	 * Returns whether project identified by SAP ID is blacklisted.
	 * 
	 * @return	boolean
	 */
	public function isBlacklisted($sap_id)
	{
		return $this->mapper->isBlacklisted($sap_id);
	}
	
	/**
	 * Deletes found project stubs and plans and leaves infos for blacklisted.
	 * 
	 * @return boolean Return true if delete was successfull, otherwise returns false.
	 */
	public function deleteAllButBlacklisted()
	{
		return $this->mapper->deleteAllButBlacklisted();
	}
	
}
