<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for plan vs EAC details export.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PlanEACReportService extends object
{
    /** @var DIContainer DI context with application repositories access */
    private $context;

	/** @var ILogger Logging facility */
    private $logger;
	
	/** @var int Year to generation */
    private $year;
	
	/** @var String Value for freeze user interface during report generation. */
    private $uiFreezeToken;
	
    /** @var string Path to storage report files. */
    private $report_storage_dir;
	
	/**
	 * Creates service for tracking table export.
	 *  
	 * @param $context Application context.
	 * @param ILogger Object for logging the import progress.
	 */
    public function  __construct(DIContainer $context, ILogger $logger, $year, $uiFreezeToken = null)
    {
        $this->context = $context;
		$this->logger = $logger;
		$this->year = $year;
		$this->uiFreezeToken = $uiFreezeToken;
		$this->report_storage_dir = Environment::getConfig('planEACReport_storage_dir');
    }
	
    /**
     * Generates EAC Details as a XLSX (MS-EXCEL file).
	 * File can be downloaded through browser and also file is saved to the data/actual/PlanEACReport folder.
	 * 
     */
    public function runPlanEACReportGeneration()
    {
		$objPHPExcel = new PHPExcel();
		
        $exec_time = 600;
	    if (!ini_set("max_execution_time", $exec_time)) 
		{
			$this->flashMessage('Setup of maximum execution time failed. Contact administrator.', 'error');
			trigger_error('Unable to setup maximum execution time to '.$exec_time.'.', E_USER_ERROR);
		}
		$lock = $this->context->createLock('PlanEAC_Report', 600);
		if (!$lock->lock())
		{
			$this->logger->log('Plan vs EAC generation start failed - active lock detected: ' . date('m-d-Y H:i:s'), ILogger::INFO);
            throw new EeacReportServiceException("Plan vs EAC Report is already running.");
		}
		$this->logger->log('Plan vs EAC report generation start: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		
		$allProjects = $this->context->projectRepository->findAllProjectsInYear($this->year, true);
        $sheet = $objPHPExcel->getActiveSheet();
        $rowIterator = $sheet->getRowIterator();
        $cellIterator = $rowIterator->current()->getCellIterator();
		$currency = 'USD';
		//setting
		$firstFormatedColumn = 'A';
		$firstTaskValueColumn = 'D';
		$lastTaskValueColumn = 'AB';
		$warningColor = 'FFC8C8';
        $borderMediumLeft = array(
        	'borders' => array(
                'left' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderMediumRight = array(
        	'borders' => array(
                'right' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderMediumBottom = array(
        	'borders' => array(
                'bottom' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
		// freeze pane
		$sheet->freezePane($firstTaskValueColumn . '2');
		//collapsing values columns
		for ($column = $firstTaskValueColumn; $column != $lastTaskValueColumn; $column++)
		{
			$sheet->getColumnDimension($column)->setOutlineLevel(1);
			$sheet->getColumnDimension($column)->setVisible(false);
			$sheet->getColumnDimension($column)->setCollapsed(true);
		}
		// set width
		$sheet->getColumnDimension('A')->setWidth(25);
        $sheet->getColumnDimension('B')->setWidth(35);
        // Header
        $cellIterator->current()->setValue("Project name");
        $cellIterator->next();
        $cellIterator->current()->setValue("Task name");
        $cellIterator->next();
		$cellIterator->current()->setValue("Category");
        $cellIterator->next();
        $cellIterator->current()->setValue("Jan plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Feb plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Mar plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Apr plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("May plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Jun plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Jul plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Aug plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Sep plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Oct plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Nov plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Dec plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Jan eac");
        $cellIterator->next();
        $cellIterator->current()->setValue("Feb eac");
        $cellIterator->next();
        $cellIterator->current()->setValue("Mar eac");
        $cellIterator->next();
        $cellIterator->current()->setValue("Apr eac");
        $cellIterator->next();
        $cellIterator->current()->setValue("May eac");
        $cellIterator->next();
        $cellIterator->current()->setValue("Jun eac");
        $cellIterator->next();
        $cellIterator->current()->setValue("Jul eac");
        $cellIterator->next();
        $cellIterator->current()->setValue("Aug eac");
        $cellIterator->next();
        $cellIterator->current()->setValue("Sep eac");
        $cellIterator->next();
        $cellIterator->current()->setValue("Oct eac");
        $cellIterator->next();
        $cellIterator->current()->setValue("Nov eac");
        $cellIterator->next();
		$cellIterator->current()->setValue("Dec eac");
        $cellIterator->next();
		$cellIterator->current()->setValue("Plan total");
        $cellIterator->next();
		$cellIterator->current()->setValue("EAC total");
        $cellIterator->next();
		$cellIterator->current()->setValue("Unit");
        $cellIterator->next();
		$cellIterator->current()->setValue("Placeholder project");
        $cellIterator->next();
		$cellIterator->current()->setValue("Task Type");
        $cellIterator->next();
		$cellIterator->current()->setValue("Generation time");
		$row = $cellIterator->current()->getRow();
		$col = $cellIterator->current()->getColumn();
		// first row formating
		$sheet->getStyle($firstFormatedColumn . $row . ':' . ($col) . $row)->applyFromArray($borderMediumBottom);
		$sheet->getStyle($firstFormatedColumn . $row . ':' . ($col) . $row)->getFont()->setBold(true);
		// generation time
		$sheet->setCellValue($col . ($row + 1), date('m-d-Y H:i:s'));
        $rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
                                   
        // Data generating
        foreach ($allProjects as $project)
        {
            $reporter = new ProjectEACReporter($this->context, $project, $this->year);
            $tasks = $this->context->taskRepository->findPlannedProjectTasksInYear($project->ID, $this->year);               
            foreach ($tasks as $task)
            {
                $eac = array();
				$plan = array();
				$eacTotal = 0;
				$planTotal = 0;
				$row = $cellIterator->current()->getRow();
                if($task->isLabourTask())
                    $eac = $reporter->getTaskLabourEACHoursMonthTotals($task);
                else
                    $eac = $reporter->getTaskNonLabourEACCostsMonthTotals($task, $currency);
                $plan = $this->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year);
				// project and task name
                $cellIterator->current()->setValue($project->Name);
                $cellIterator->next();
                $cellIterator->current()->setValue($task->name);
				$cellIterator->next();
				$cellIterator->current()->setValue($task->getCategory());
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderMediumRight);
                $cellIterator->next();
                // plan       
                for($i=1;$i<=12;$i++)
                {
					if($task->isLabourTask())
					{
						$cellIterator->current()->setValue($plan->getMonthPlan($i)->Hours);
						$planTotal += $plan->getMonthPlan($i)->Hours;
					}
					else
					{
						$cellIterator->current()->setValue($plan->getMonthPlan($i)->Budget);
						$planTotal += $plan->getMonthPlan($i)->Budget;
					}
                    $cellIterator->next();
                }
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderMediumLeft);
				// eac
				for($i=1;$i<=12;$i++)
                {
                    $cellIterator->current()->setValue($eac[$i]);
					$eacTotal += $eac[$i];
                    $cellIterator->next();
                }
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderMediumLeft);
				if($task->isLabourTask())
				{
					$planTotal = round($planTotal);
					$eacTotal = round($eacTotal);
				}
				else
				{
					$planTotal = round($planTotal, -3);
					$eacTotal = round($eacTotal, -3);
				}
				// eac and plan totals
				$cellIterator->current()->setValue($planTotal);
				$cellIterator->next();
				$cellIterator->current()->setValue($eacTotal);
				$cellIterator->next();
					
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderMediumLeft);
                
                if($task->isLabourTask())
                    $cellIterator->current()->setValue("hrs");
                else
                    $cellIterator->current()->setValue($currency);
				
				$cellIterator->next();
				if($project->Fake)
					$cellIterator->current()->setValue('Yes');
				else
					$cellIterator->current()->setValue('No');
				
				$cellIterator->next();
				if($task->Fake == Task::TASK_TYPE_SAP)
					$cellIterator->current()->setValue('SAP');
				elseif($task->Fake == Task::TASK_TYPE_PLACEHOLDER)
					$cellIterator->current()->setValue('Placeholder');
				else
					$cellIterator->current()->setValue('Planning');
				
				// highlight column if needed
				if($planTotal < $eacTotal)
				{
					$row = $cellIterator->current()->getRow();
					$col = $cellIterator->current()->getColumn();
					$sheet->getStyle($firstFormatedColumn . $row . ':' . $col . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($warningColor);
				}
                $rowIterator->next();
                $cellIterator = $rowIterator->current()->getCellIterator();
            }
        }
        
		if(!is_null($this->uiFreezeToken))
			setcookie('fileDownloadToken', $this->uiFreezeToken);

        $objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
        if (!file_exists($this->report_storage_dir)) 
          mkdir($this->report_storage_dir, 0777, TRUE);
		
		if($this->year == date('Y'))
			$objWriter->save($this->report_storage_dir . 'PlanEACReport.xlsx');
		else
			$objWriter->save($this->report_storage_dir . 'PlanEACReport_' . $this->year . '.xlsx');
		
		$this->logger->log('Plan vs EAC report generation end: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		$lock->unlock();
    }
}

class EPlanEacReportServiceException extends Exception {}