<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing responsibilities from database using dibi.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class ResponsibilityDibiMapper extends DibiMapper implements IResponsibilityMapper
{
	/**
	 * Creates Responsibility instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	Responsibility 
	 */
    private function loadResponsibility(DibiRow $row)
    {
		$responsibility = new Responsibility();
		$responsibility->ID = $row->Responsibility_ID;
		$responsibility->Name = $row->Name;
		return $responsibility;
    }	

    /**
     * Saves responsibility.
     * 
     * @param Responsibility $responsibility responsibility
     * @return boolean Returns true if responsibility is successfully saved, otherwise false.
     */
    public function save(Responsibility $responsibility)
    {
        trigger_error("Not implemented yet.", E_USER_ERROR);
    }

    /**
     * Updates responsibility.
     * 
     * @param Responsibility $responsibility responsibility
     * @return boolean Returns true if responsibility is successfully updated, otherwise false.
     */
    public function update(Responsibility $responsibility)
    {
        trigger_error("Not implemented yet.", E_USER_ERROR);
    }
	
	/**
	 * Returns responsibility.
	 * 
	 * @param string $id Responsibility ID.
	 * @return Responsibility or NULL if not found.
	 */
	public function findResponsibility($id)
	{
		$responsibility_db_result = $this->conn->query('SELECT * FROM [Responsibility] WHERE [Responsibility_ID] = %s', $id)->fetch();
		if ($responsibility_db_result)
		{
			$responsibility = $this->loadResponsibility($responsibility_db_result);
			return $responsibility;
		}
		else
			return NULL;
	}

	/**
	 * Returns all responsibilities as an array sorted by name.
	 * 
	 * @return array associative array of Responsibility as value and its ID as a key
	 */
	public function findAllResponsibilities()
	{
		$responsibilities_db_result = $this->conn->query('SELECT * FROM [Responsibility] ORDER BY [Name] ASC')->fetchAll();
		$responsibilities = array();
		foreach ($responsibilities_db_result as $row)
		{
			$responsibility = $this->loadResponsibility($row);
			$responsibilities[$responsibility->ID] = $responsibility;
		}
		return $responsibilities;
	}
	
}
