<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * CustomerProgramCall definition.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class CustomerProgramCall extends Object
{
	/** @var int ID of customer call */
	private $callID;
	/** @var int ID of program */
	private $programID;
	/** @var string Name of call */
	private $call;
	/** @var string Remarks*/
	private $remarks;
	/** @var date Opening date of call */
	private $openingDate;
	/** @var date Closing date of call */
	private $closingDate;
	/** @var int Amount covered by customer */
	private $costShare = 0;
	/** @var string EID of responsible person */
	private $responsible;
	
	/**
     * Sets ID of call.
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
	public function setCallID($id)
	{
		if ($id > 0)
			$this->callID = $id;
		else
			throw new InvalidArgumentException('Call ID must be integer greater than 0. Wrong param: ' . $id);
	}
	
    /**
     * Gets ID of call.
     * @return int
     */
	public function getCallID()
	{
		return $this->callID;
	}
	
	/**
     * Sets ID of program.
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
	public function setProgramID($id)
	{
		if ($id > 0)
			$this->programID = $id;
		else
			throw new InvalidArgumentException('Program ID must be integer greater than 0. Wrong param: ' . $id);
	}
	
    /**
     * Gets ID of program.
     * @return int
     */
	public function getProgramID()
	{
		return $this->programID;
	}
	
	/**
	 * Sets call.
	 * @param string $name
	 */
	public function setCall($name)
	{
		$this->call = $name;
	}
	
	/**
	 * Gets call.
	 * @return string
	 */
	public function getCall()
	{
		return $this->call;
	}
	
	/**
	 * Sets remarks.
	 * @param string $text
	 */
	public function setRemarks($text)
	{
		$this->remarks = $text;
	}
	
	/**
	 * Gets remarks.
	 * @return string
	 */
	public function getRemarks()
	{
		return $this->remarks;
	}
	
	/**
	 * Sets opening date.
	 * @param string $date
	 */
	public function setOpeningDate($date)
	{
		$this->openingDate = $date;
	}
	
	/**
	 * Gets opening date.
	 * @return string
	 */
	public function getOpeningDate()
	{
		return $this->openingDate;
	}
	
	/**
	 * Sets closing date.
	 * @param string $date
	 */
	public function setClosingDate($date)
	{
		$this->closingDate = $date;
	}
	
	/**
	 * Gets closing date.
	 * @return string
	 */
	public function getClosingDate()
	{
		return $this->closingDate;
	}
	
	/**
     * Sets costs sharing percentage between customer and supplier.
	 * @param float $percent_as_float
     */
	public function setCostShare($percent_as_float)
	{
		if ((is_numeric($percent_as_float) && ($percent_as_float >= 0) && ($percent_as_float <= 1.0)) || is_null($percent_as_float))
			$this->costShare = $percent_as_float;
		else
			throw new InvalidArgumentException('Cost sharing must be float in range <0,1>.');
	}
    
    /**
     * Gets cost share.
     * 
     * @return float
     */
	public function getCostShare()
	{
		return $this->costShare;
	}
    
    /**
     * Gets cost share expressed in percentage.
     * 
     * @return float
     */
	public function getCostSharePercentage()
	{
	    return $this->costShare * 100;
	}
	
	/**
	 * Sets responsible.
	 * @param string $eid
	 */
	public function setResponsible($eid)
	{
		$this->responsible = Person::processEID($eid);
	}
	
	/**
	 * Gets responsible.
	 * @return string
	 */
	public function getResponsible()
	{
		return $this->responsible;
	}
}
