<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * CustomerProgramCall repository access class.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class CustomerProgramCallRepository extends Object
{
	/** @var ICustomerProgramCallMapper */
 	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(ICustomerProgramCallMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'Call-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Call"));
    }
    
	/**
	 * Saves program call.
	 * @param CustomerProgramCall $programCall
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(CustomerProgramCall $programCall)
	{
		if($this->mapper->save($programCall))
		{
			$this->multilog->log('Call with ID: ' . $programCall->CallID . ' and Name: ' . $programCall->Call . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Updates program call.
	 * @param CustomerProgramCall $programCall
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(CustomerProgramCall $programCall)
	{
		$oldObject = $this->find($programCall->CallID);
		if($this->mapper->update($programCall))
		{
			$this->multilog->log('Call with ID: ' . $programCall->CallID . ' and Name: ' . $programCall->Call . ' updated. ' . $this->multilog->differences($oldObject, $programCall), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes program call.
	 * @param int $id programCall ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		if($this->mapper->delete($id))
		{
			$this->multilog->log('Call with ID: ' . $id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Finds program call by ID.
	 * @param int $id programCall ID.
	 * @return	CustomerProgramCall or NULL if not found.
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}
	
	/** 
	 * Finds all program calls.
	 * @return array Array of CustomerProgramCall or empty array.	
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
	
	/** 
	 * Finds program calls by customer program ID.
	 * @param int $id customer program ID.
	 * @return array Array of CustomerProgramCall or empty array.	
	 */
	public function findByProgram($id)
	{
		return $this->mapper->findByProgram($id);
	}
	
	/** 
	 * Finds program calls by customer ID.
	 * @param int $id customer ID.
	 * @return array Array of CustomerProgramCall or empty array.	
	 */
	public function findByCustomer($id)
	{
		return $this->mapper->findByCustomer($id);
	}
}
