<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing STRAP Item.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
interface IItemMapper
{
	/**
	 * Saves new STRAP item.
	 * 
	 * @param Item $item
	 * @return boolean
	 */
	public function save(Item $item);
	
	/**
	 * Updates existing STRAP item.
	 * 
	 * @param Item $item
	 * @return boolean
	 */
	public function update(Item $item);
	
	/**
	 * Deletes existing STRAP Item.
	 * 
	 * @param int $item
	 * @return boolean
	 */
	public function delete($item_id);

	/**
	 * Gets STRAP Item by id.
	 * 
	 * @param int $item
	 * @return Item
	 */
	public function find($item_id);

	/**
	 * Gets all Strap Items.
	 * 
	 * @return array Array of Items where strapItem_ID is key.
	 */
	public function findAll();
	
	/**
	 * Gets unique number for new item where CustomID has passed prefix.
	 * Number expresses order of items with the same prefix.
	 * 
	 * @param string $prefix May be defined as '[department short]-[technical group short]-'
	 * @return string '1' if prefix does not exist, first free number for passed prefix otherwise.
	 */
	public function getNextCustomIDNumber($prefix);
}
