<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing STRAP ItemActivityYearlyBudget from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ItemActivityYearlyBudgetDibiMapper extends DibiMapper implements IItemActivityYearlyBudgetMapper
{
	/**
	 * Creates ItemActivityYearlyBudget instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	ItemActivityYearlyBudget
	 */
    private function load(DibiRow $row)
    {
		$itemActivityYearlyBudget = new ItemActivityYearlyBudget();
		$itemActivityYearlyBudget->ID = $row->Activity_ID;
		$itemActivityYearlyBudget->Year = $row->Year;
		$itemActivityYearlyBudget->LaborCosts = $row->LabourCosts;
		$itemActivityYearlyBudget->SubcontractingCosts = $row->SubcontractingCosts;
		$itemActivityYearlyBudget->TravelCosts = $row->TravelCosts;
		$itemActivityYearlyBudget->MaterialCosts = $row->MaterialCosts;
		return $itemActivityYearlyBudget;
    }
	
	/**
	 * Saves new STRAP item activity yearl budget.
	 * 
	 * @param ItemActivityYearlyBudget $item
	 * @return boolean
	 */
	public function save(ItemActivityYearlyBudget $item)
	{
        try
        {
            $array = array();
			$array['Activity_ID'] = $item->ID;
			$array['Year'] = $item->Year;
			$array['LabourCosts'] = $item->LaborCosts;
			$array['SubcontractingCosts'] = $item->SubcontractingCosts;
			$array['TravelCosts'] = $item->TravelCosts;
			$array['MaterialCosts'] = $item->MaterialCosts;
            $this->conn->insert('StrapItemActivityYearlyBudget', $array)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert new STRAP ItemActivityYearlyBudget with ID: " . $item->ID . " for year: " . $item->Year . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates existing STRAP ItemActivityYearlyBudget.
	 * 
	 * @param ItemActivityYearlyBudget $item
	 * @return boolean
	 */
	public function update(ItemActivityYearlyBudget $item)
	{
        try
        {
            $array = array();
			$array['LabourCosts'] = $item->LaborCosts;
			$array['SubcontractingCosts'] = $item->SubcontractingCosts;
			$array['TravelCosts'] = $item->TravelCosts;
			$array['MaterialCosts'] = $item->MaterialCosts;
            $this->conn->update('StrapItemActivityYearlyBudget', $array)->where('[Activity_ID] = %i AND [Year] = %i', $item->ID, $item->Year)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of STRAP ItemActivityYearlyBudget with ID: " . $item->ID . " for year: " . $item->Year . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes existing STRAP ItemActivityYearlyBudget.
	 * 
	 * @param int $item_id
	 * @param int $year
	 * @return boolean
	 */
	public function delete($item_id, $year)
	{
		try
		{
		    $result = $this->conn->delete('StrapItemActivityYearlyBudget')->where('[Activity_ID] = %i AND [Year] = %i', $item_id, $year)->execute();
			if ($result)
				Debugger::log("Strap Item Activity with ID: " . $item_id . " for year: " . $year . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of STRAP Item Activity with ID: " . $item_id . " for year: " . $year . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of STRAP Item Activity with ID: " . $item_id . " for year: " . $year . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Gets STRAP ItemActivityYearlyBudget by id and year.
	 * 
	 * @param int $item_id
	 * @param int $year
	 * @return ItemActivityYearlyBudget or null if not found
	 */
	public function find($item_id, $year)
	{
		$item_db_result = $this->conn->select('*')->from('StrapItemActivityYearlyBudget')->where('[Activity_ID] = %u AND [Year] = %i', $item_id, $year)->fetch();
		if ($item_db_result !== FALSE)
			return $this->load($item_db_result);
		else
			return NULL;
	}

	/**
	 * Gets all STRAP ItemActivityYearlyBudget by id.
	 * 
	 * @param int $item_id
	 * @return array Array of ItemActivityYearlyBudget where Activity_ID_Year is key or null if not found.
	 */
	public function findAllByID($item_id)
	{
		$items_array = array();
		$items_db_result = $this->conn->select('*')->from('StrapItemActivityYearlyBudget')->where('[Activity_ID] = %u', $item_id)->fetchAll();
		if ($items_db_result !== FALSE)
		{
				foreach ($items_db_result as $item)
				{
					$items_array[$item->Activity_ID . '_' . $item->Year] = $this->load($item);
				}
				return $items_array;
		}
		else
			return null;
	}
}
