<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of STRAP ItemActivityYearlyBudget.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ItemActivityYearlyBudgetRepository extends Object
{
	/** @var IItemActivityYearlyBudgetMapper */
    private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IItemActivityYearlyBudgetMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ActivtiyBudget-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "STRAP Activity Budget"));
    }
	
	/**
	 * Saves new STRAP item activity yearl budget.
	 * 
	 * @param ItemActivityYearlyBudget $item
	 * @return boolean
	 */
	public function save(ItemActivityYearlyBudget $item)
	{
		if($this->mapper->save($item))
		{
			$this->multilog->log('Strap Activity Budget for Activity with ID: ' . $item->ID . ' and Year: ' . $item->Year . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Updates existing STRAP ItemActivityYearlyBudget.
	 * 
	 * @param ItemActivityYearlyBudget $item
	 * @return boolean
	 */
	public function update(ItemActivityYearlyBudget $item)
	{
		$oldObject = $this->find($item->ID, $item->Year);
		if($this->mapper->update($item))
		{
			$this->multilog->log('Strap Activity Budget for Activity with ID: ' . $item->ID . ' and Year: ' . $item->Year . ' updated. ' . $this->multilog->differences($oldObject, $item), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing STRAP ItemActivityYearlyBudget.
	 * 
	 * @param int $item_id
	 * @param int $year
	 * @return boolean
	 */
	public function delete($item_id, $year)
	{
		if($this->mapper->delete($item_id, $year))
		{
			$this->multilog->log('Strap Activity Budget for Activity with ID: ' . $item_id . ' and Year: ' . $year . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Gets STRAP ItemActivityYearlyBudget by id and year.
	 * 
	 * @param int $item_id
	 * @param int $year
	 * @return ItemActivityYearlyBudget or null if not found
	 */
	public function find($item_id, $year)
	{
		return $this->mapper->find($item_id, $year);
	}

	/**
	 * Gets all STRAP ItemActivityYearlyBudget by id.
	 * 
	 * @param int $item_id
	 * @return array Array of ItemActivityYearlyBudget where Activity_ID_Year is key or null if not found.
	 */
	public function findAllByID($item_id)
	{
		return $this->mapper->findAllByID($item_id);
	}
}