<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of STRAP NeedToPursue.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class NeedToPursueRepository extends Object
{
	/** @var INeedToPursueMapper */
    private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(INeedToPursueMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'NeedToPursue-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "STRAP NeedToPursue"));
    }
    
	/**
	 * Saves new STRAP NeedToPursue.
	 * 
	 * @param NeedToPursue $needToPursue
	 * @return boolean
	 */
	public function save(NeedToPursue $needToPursue)
	{
		if($this->mapper->save($needToPursue))
		{
			$this->multilog->log('Strap NeedToPursue with ID: ' . $needToPursue->ID . ' and Name: ' . $needToPursue->Name . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Updates existing STRAP NeedToPursue.
	 * 
	 * @param NeedToPursue $needToPursue
	 * @return boolean
	 */
	public function update(NeedToPursue $needToPursue)
	{
		$oldObject = $this->find($needToPursue->ID);
		if($this->mapper->update($needToPursue))
		{
			$this->multilog->log('Strap NeedToPursue with ID: ' . $needToPursue->ID . ' and Name: ' . $needToPursue->Name . ' updated. ' . $this->multilog->differences($oldObject, $needToPursue), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing STRAP Item.
	 * 
	 * @param int $needToPursue_id
	 * @return boolean
	 */
	public function delete($needToPursue_id)
	{
		if($this->mapper->delete($needToPursue_id))
		{
			$this->multilog->log('Strap NeedToPursue with ID: ' . $needToPursue_id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Gets STRAP NeedToPursue by id.
	 * 
	 * @param int $needToPursue_id
	 * @return NeedToPursue or null if not found
	 */
	public function find($needToPursue_id)
	{
		return $this->mapper->find($needToPursue_id);
	}

	/**
	 * Gets all STRAP NeedToPursue.
	 * 
	 * @return array Array of NeedToPursue where NeedToPursue_ID is key or null if not found.
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
}