<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing STRAP ProductLine from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ProductLineDibiMapper extends DibiMapper implements IProductLineMapper
{
	/**
	 * Creates ProductLine instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	ProductLine 
	 */
    private function load(DibiRow $row)
    {
		$productLine = new ProductLine();
		$productLine->ProductName = $row->ProductName;
		return $productLine;
    }
	
	/**
	 * Saves new STRAP ProductLine.
	 * 
	 * @param ProductLine $productLine
	 * @return boolean
	 */
	public function save(ProductLine $productLine)
	{
        try
        {
            $array = array();
			$array['ProductName'] = $productLine->ProductName;
            $this->conn->insert('ProductLine', $array)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert new STRAP ProductLine with ProductName: " . $productLine->ProductName . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates existing STRAP ProductLine.
	 * 
	 * @param ProductLine $productLine
	 * @return boolean
	 */
	public function update(ProductLine $productLine)
	{
        try
        {
            $array = array();
			$array['ProductName'] = $productLine->ProductName;
            $this->conn->update('ProductLine', $array)->where('[ProductName] = %s', $productLine->ProductName)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of STRAP ProductLine with ProductName = " . $productLine->ProductName . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes existing STRAP ProductLine.
	 * 
	 * @param int $productLine
	 * @return boolean
	 */
	public function delete($productLine)
	{
		try
		{
		    $result = $this->conn->delete('ProductLine')->where('[ProductName] = %s', $productLine)->execute();
			if ($result)
				Debugger::log("Strap ProductLine with ProductName = " . $productLine . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of STRAP ProductLine with ProductName = " . $productLine . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of STRAP ProductLine with ProductName = " . $productLine . " failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Gets STRAP ProductLine.
	 * 
	 * @param int $productLine
	 * @return ProductLine or null if not found
	 */
	public function find($productLine)
	{
		$productLine_db_result = $this->conn->select('*')->from('ProductLine')->where('[ProductName] = %s', $productLine)->fetch();
		if ($productLine_db_result !== FALSE)
			return $this->load($productLine_db_result);
		else
			return NULL;
	}

	/**
	 * Gets all STRAP ProductLine.
	 * 
	 * @return array Array of ProductLine or null if not found.
	 */
	public function findAll()
	{
        $productLine_array = array();
		$productLine_db_result = $this->conn->select('*')->from('ProductLine')->fetchAll();        
		if ($productLine_db_result !== FALSE)			
		{
			foreach ($productLine_db_result as $row)
				$productLine_array[] = $this->load($row);
			return $productLine_array;
		}
		else
			return null;
	}
}
