<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of STRAP ProductLine.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ProductLineRepository extends Object
{
	/** @var IProductLineMapper */
    private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IProductLineMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ProductLine-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "STRAP ProductLine"));
    }
    
	/**
	 * Saves new STRAP ProductLine.
	 * 
	 * @param ProductLine $productLine
	 * @return boolean
	 */
	public function save(ProductLine $productLine)
	{
		if($this->mapper->save($productLine))
		{
			$this->multilog->log('Strap ProductLine with Name: ' . $productLine->ProductName . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Updates existing STRAP ProductLine.
	 * 
	 * @param ProductLine $productLine
	 * @return boolean
	 */
	public function update(ProductLine $productLine)
	{
		$oldObject = $this->find($productLine->ProductName);
		if($this->mapper->update($productLine))
		{
			$this->multilog->log('Strap ProductLine with Name: ' . $productLine->ProductName . ' updated. ' . $this->multilog->differences($oldObject, $productLine), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing STRAP ProductLine.
	 * 
	 * @param int $productLine
	 * @return boolean
	 */
	public function delete($productLine)
	{
		if($this->mapper->delete($productLine))
		{
			$this->multilog->log('Strap ProductLine with Name: ' . $productLine . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Gets STRAP ProductLine.
	 * 
	 * @param int $productLine
	 * @return ProductLine or null if not found
	 */
	public function find($productLine)
	{
		return $this->mapper->find($productLine);
	}

	/**
	 * Gets all STRAP ProductLine.
	 * 
	 * @return array Array of ProductLine or null if not found.
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
}