<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of PursuitPlan.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PursuitPlanRepository extends Object
{
	/** @var IPursuitPlanMapper */
    private $mapper;

	/** @var MultiLogger */
	private $multilog;
	
    public function __construct(IPursuitPlanMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'PursuitPlan-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Pursuit Plan"));
    }
    
	/**
	 * Saves new PursuitPlan.
	 * 
	 * @param PursuitPlan $pursuitPlan
	 * @return boolean
	 */
	public function save(PursuitPlan $pursuitPlan)
	{
		if($this->mapper->save($pursuitPlan))
		{
			$this->multilog->log('PursuitPlan milestone: ' . $pursuitPlan->Milestone . ' for pursuit: ' . $pursuitPlan->PursuitID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Updates PursuitPlan.
	 * 
	 * @param PursuitPlan $pursuitPlan
	 * @return boolean
	 */
	public function update(PursuitPlan $pursuitPlan)
	{
		$oldObject = $this->find($pursuitPlan->PursuitID, $pursuitPlan->Milestone);
		if($this->mapper->update($pursuitPlan))
		{
			$this->multilog->log('PursuitPlan milestone: ' . $pursuitPlan->Milestone . ' for pursuit: ' . $pursuitPlan->PursuitID . ' updated. ' . $this->multilog->differences($oldObject, $pursuitPlan), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing PursuitPlan.
	 * 
	 * @param int $pursuit Pursuit ID
	 * @param string $milestone Milestone
	 * @return boolean
	 */
	public function delete($pursuitID, $milestone)
	{
		if($this->mapper->delete($pursuitID, $milestone))
		{
			$this->multilog->log('PursuitPlan milestone: ' . $milestone . ' for pursuit: ' . $pursuitID . ' deleted. ', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Gets PursuitPlan by purusit ID and milestone.
	 * 
	 * @param int $pursuit Pursuit ID
	 * @param string $milestone Milestone
	 * @return PursuitPlan or null if not found
	 */
	public function find($pursuitID, $milestone)
	{
		return $this->mapper->find($pursuitID, $milestone);
	}
	
	/**
	 * Gets PursuitPlan by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return array Of PursuitPlan or empty array if not found
	 */
	public function findByPursuitID($pursuitID)
	{
		return $this->mapper->findByPursuitID($pursuitID);
	}
	
	/**
	 * Gets nearest PursuitPlan by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return PursuitPlan or null if not found
	 */
	public function findNearestByPursuitID($pursuitID)
	{
		return $this->mapper->findNearestByPursuitID($pursuitID);
	}
	
	/**
	 * Finds out if exists missed milestone.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return True or false
	 */
	public function existsMissedMilestone($pursuitID)
	{
		return $this->mapper->existsMissedMilestone($pursuitID);
	}	
	
	/**
	 * Gets missed PursuitPlan by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return array Of PursuitPlan or empty array if not found
	 */
	public function findMissedByPursuitID($pursuitID)
	{
		return $this->mapper->findMissedByPursuitID($pursuitID);
	}
}