<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of pursuit risk.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PursuitRiskRepository extends Object
{
	/** @var IPursuitRiskMapper */
    private $mapper;

	/** @var MultiLogger */
	private $multilog;
	
    public function __construct(IPursuitRiskMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'PursuitRisk-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Pursuit Risk"));
    }
    
	/**
	 * Saves new PursuitRisk.
	 * 
	 * @param PursuitRisk $pursuitRisk
	 * @return boolean
	 */
	public function save(PursuitRisk $pursuitRisk)
	{
		if($this->mapper->save($pursuitRisk))
		{
			$this->multilog->log('PursuitRisk with risk: ' . $pursuitRisk->Risk . ' for pursuit: ' . $pursuitRisk->PursuitID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Updates PursuitRisk.
	 * 
	 * @param PursuitRisk $pursuitRisk
	 * @return boolean
	 */
	public function update(PursuitRisk $pursuitRisk)
	{
		$oldObject = $this->find($pursuitRisk->ID, $pursuitRisk->PursuitID);
		if($this->mapper->update($pursuitRisk))
		{
			$this->multilog->log('PursuitRisk with risk: ' . $pursuitRisk->Risk . ' for pursuit: ' . $pursuitRisk->PursuitID . ' updated. ' . $this->multilog->differences($oldObject, $pursuitRisk), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing PursuitRisk.
	 * 
	 * @param int $id PursuitRisk ID 
	 * @return boolean
	 */
	public function delete($id)
	{
		if($this->mapper->delete($id))
		{
			$this->multilog->log('PursuitRisk with ID: ' . $id . ' deleted. ', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Gets PursuitRisk.
	 * 
	 * @param int $id PursuitRisk id
	 * @return PursuitRisk or null if not found
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}
	
	/**
	 * Gets PursuitRisk by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return array Of PursuitRisk or empty array if not found
	 */
	public function findByPursuitID($pursuitID)
	{
		return $this->mapper->findByPursuitID($pursuitID);
	}
	
	/**
	 * Gets nearest PursuitRisk by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return PursuitPlan or null if not found
	 */
	public function findNearestByPursuitID($pursuitID)
	{
		return $this->mapper->findNearestByPursuitID($pursuitID);
	}
	
	/**
	 * Finds out if exists missed milestone.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return True or false
	 */
	public function existsMissedRisk($pursuitID)
	{
		return $this->mapper->existsMissedRisk($pursuitID);
	}
	
	/**
	 * Gets missed PursuitRisk by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return array Of PursuitRisk or empty array if not found
	 */
	public function findMissedByPursuitID($pursuitID)
	{
		return $this->mapper->findMissedByPursuitID($pursuitID);
	}
}