<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Security
 */

/**
 * Authentication via testing valid user account in database. No password check.
 */
class ValidUserAuthenticator extends Object implements IAuthenticator
{

	/** @var PersonRepository User repository */
	private $userRepository = NULL;
	
	/**
	 * Options injection from application configuration.
	 */
	public function setOptions($options)
	{
	}

	/**
	 * Options injection from application configuration.
	 */
	public function setUserRepository(PersonRepository $user_repository)
	{
		$this->userRepository = $user_repository;
	}

	/**
	 * Returns used user repository
	 * @return PersonRepository
	 */
	public function getUserRepository()
	{
		return $this->userRepository;
	}
	
	/**
	 * Test validity of user account.
	 * @throws AuthenticationException on invalid user account name.
	 */
	protected function validateUserAccount($username)
	{
		if ($this->userRepository === NULL)
             throw new AuthenticationException("Cannot authenticate user. Repository not setup.", self::FAILURE);
		$person = $this->userRepository->find($username);
        if ($person === NULL)
             throw new PersonNotFoundAuthenticationException("User '".$username."' not found in the database.", self::FAILURE);
		return $person;
	}
	
	/**
     * Performs authentication.
     *
     * @param  array
     * @return IIdentity
     * @throws AuthenticationException
     */
    public function authenticate(array $credentials)
	{
        $username = strtolower($credentials[self::USERNAME]);
		$person = $this->validateUserAccount($username);
		return new ValidPersonIdentity($username);
	}

}
