<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * SettingDisplayedContent repository access class.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class SettingDisplayedContentRepository extends Object
{
	/** @var ISettingDisplayedContentMapper */
 	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(ISettingDisplayedContentMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'SettingDisplayedContent-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Person setting"));
    }
    
	/**
	 * Saves setting.
	 * @param SettingDisplayedContent $setting
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(SettingDisplayedContent $setting)
	{
		if($this->mapper->save($setting))
		{
			$this->multilog->log('Setting of displayed content for EID: ' . $setting->EID . ' and OrganizationUnitID: ' . $setting->OrganizationUnitID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Updates setting.
	 * @param SettingDisplayedContent $setting
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(SettingDisplayedContent $setting)
	{
		$oldObject = $this->find($setting->EID, $setting->OrganizationUnitID);
		if($this->mapper->update($setting))
		{
			$this->multilog->log('Setting of displayed content for EID: ' . $setting->EID . ' and OrganizationUnitID: ' . $setting->OrganizationUnitID . ' updated. ' . $this->multilog->differences($oldObject, $setting), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes setting.
	 * @param string $eid User EID.
	 * @param int $orgUnitID Organization unit ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($eid, $orgUnitID)
	{
		if($this->mapper->delete($eid, $orgUnitID))
		{
			$this->multilog->log('Setting of displayed content for EID: ' . $eid . ' and OrganizationUnitID: ' . $orgUnitID . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Finds setting.
	 * @param string $eid User EID.
	 * @param int $orgUnitID Organization unit ID.
	 * @return	SettingDisplayedContent or NULL if not found.
	 */
	public function find($eid, $orgUnitID)
	{
		return $this->mapper->find($eid, $orgUnitID);
	}
	
	/**
	 * Finds setting by EID.
	 * @param string $eid User EID.
	 * @return array Array of SettingDisplayedContent or empty array.
	 */
	public function findByEID($eid)
	{
		return $this->mapper->findByEID($eid);
	}
	
}
