<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing subcontractor from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 *
 */
class SubcontractorDibiMapper extends DibiMapper implements ISubcontractorMapper
{
    /**
	 * Creates Subcontractor instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	Subcontractor
	 */
    private function load(DibiRow $row)
    {
		$subcontractor = new Subcontractor();
		$subcontractor->ID = $row->Subcontractor_ID;
		$subcontractor->name = $row->Name;
		$subcontractor->short = $row->Short;
		$subcontractor->status = $row->Status;
		return $subcontractor;
    }	
    
	/**
	 * Finds all subcontractors.
	 * 
	 * @return array Subcontractor or NULL if not found.
	 */
	public function findAll()
	{
            $subcontractors_array = array();
            $subcontractors_db_result = $this->conn->select('*')->from('Subcontractor')->orderBy('Name')->fetchAll();
            if ($subcontractors_db_result !== FALSE)
            {
                foreach ($subcontractors_db_result as $subcontractor)
                    $subcontractors_array[$subcontractor->Subcontractor_ID] = $this->load($subcontractor);
                return $subcontractors_array;
            }
            else 
                return NULL;
	}
    
	/**
	 * Finds subcontractor by id.
	 * 
	 * @param int $id subcontractor ID.
	 * @return	Subcontractor or NULL if not found.
	 */
	public function find($id)
	{
		$subcontractor_db_result = $this->conn->select('*')->from('Subcontractor')->where('[Subcontractor_ID] = %i', $id)->fetch();
		if ($subcontractor_db_result !== FALSE)
			return $this->load($subcontractor_db_result);
		else
			return NULL;
	}
    
	/**
	 * Finds all subcontractors working on given project charter.
	 * 
	 * @param int $charter_ID ProjectCharter ID.
	 * @return array Array of Subcontractors where key is subcontractor_ID or NULL if not found.
	 */
	public function findAllSubcontractorsOnCharter($charter_ID)
	{
		$subcontractor_db_result = $this->conn->query('SELECT [Subcontractor].* FROM [ProjectCharterBudget] 
                                                            JOIN [Subcontractor] ON [Subcontractor].[Subcontractor_ID] = [ProjectCharterBudget].[Subcontractor]
                                                                WHERE [ProjectCharterBudget].[ProjectCharter_ID] = %i', $charter_ID)->fetchAll();
        $subcontractor_array = array();
        if ($subcontractor_db_result !== FALSE)
        {
            foreach ($subcontractor_db_result as $subcontractor)
                $subcontractor_array[$subcontractor->Subcontractor_ID] = $this->load($subcontractor);
            return $subcontractor_array;
        }
        else
            return null;
	}
}
