<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Subcontractor repository access class.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 *
 */
class SubcontractorRepository  extends Object
{
    /** @var ISubcontractorMapper */
 	private $mapper;

    public function __construct(ISubcontractorMapper $mapper)
    {
        $this->mapper = $mapper;
    }
    
	/**
	 * Finds all subcontractors.
	 * 
	 * @return array Subcontractors or NULL if not found.
	 */
	public function findAll()
	{
            return $this->mapper->findAll();
	}
    
	/**
	 * Finds subcontractor by id.
	 * 
	 * @param int $id subcontractor ID.
	 * @return array Subcontractor or NULL if not found.
	 */
	public function find($id)
	{
            return $this->mapper->find($id);
	}
    
	/**
	 * Finds all subcontractors working on given project charter.
	 * 
	 * @param int $charter_ID ProjectCharter ID.
	 * @return array Array of Subcontractors where key is subcontractor_ID or NULL if not found.
	 */
	public function findAllSubcontractorsOnCharter($charter_ID)
	{
            return $this->mapper->findAllSubcontractorsOnCharter($charter_ID);
        }
}
