<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing system checks using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class SystemCheckDibiMapper extends DibiMapper implements ISystemCheckMapper
{
	/**
	 * Creates Responsibility instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	SystemCheck 
	 */
    private function loadSystemCheckArray(DibiRow $row)
    {
		$array = array();
		$systemCheck = new SystemCheck();
		foreach($row as $key => $value)
		{
			$array[$key] = $value;
		}
		$systemCheck->Values = $array;
		return $systemCheck;
    }	

	/**
	 * Check if people are assigned only on the task with the same work center.
	 * Returns wrong assigned persons.
	 * @param Year int
	 * 
	 * @return array of SytemCheck or null of not found.
	 */
	public function findAssignedPersonCheck($year)
	{
		$attributes = array();
		$systemCheck_db_result = $this->conn->query('SELECT DISTINCT `Person`.`EID`,`Person`.`Surname`,`Person`.`CostCenter` AS `PersonWorkCenter`,
														`Task`.`SAP_ID`, `Task`.`CostCenter_ID` AS `TaskWorkCenter` 
														FROM `Person`, `Person_Task`, `Task` 
														WHERE `Person`.`EID`=`Person_Task`.`EID` 
														AND `Task`.`Task_ID`=`Person_Task`.`Task_ID` 
														AND `Person`.`CostCenter`<>`Task`.`CostCenter_ID`')->fetchAll();
		if ($systemCheck_db_result)
		{
			foreach ($systemCheck_db_result[0] as $attribute => $result)
			{
				$attributes[] = $attribute;
			}
			$systemCheck[0] = $attributes;
			foreach($systemCheck_db_result as $result)
			{
				$systemCheck[] = $this->loadSystemCheckArray($result);
			}
			return $systemCheck;
		}
		else
			return NULL;
	}

	/**
	 * Check if there are tasks with zero hours effort and zero budget having task plan for current year in the database.
	 * Returns wrong tasks.
	 * @param Year int
	 * 
	 * @return array of SytemCheck or null of not found.
	 */
	public function findZeroTaskCheck($year)
	{
		$attributes = array();
		$systemCheck_db_result = $this->conn->query("SELECT `Project`.`Name`,`Project`.`SAP_ID` AS `Project_SAP_ID`,
														`Task`.`Name`,`Task`.`SAP_ID` AS `Task_SAP_ID`,`Task`.`LabourTask`,`TaskPlan`.`Year`,
														`TaskPlan`.`TaskPlan_ID`
														FROM `TaskPlan`,`Task`,`Project`
														WHERE `TaskPlan`.`Task_ID`=`Task`.`Task_ID` AND `Project`.`Project_ID`=`Task`.`Project_ID` 
														AND `TaskPlan`.`January`='0' AND `TaskPlan`.`February`='0' AND `TaskPlan`.`March`='0'
														AND `TaskPlan`.`April`='0' AND `TaskPlan`.`May`='0' AND `TaskPlan`.`June`='0' 
														AND `TaskPlan`.`July`='0' AND `TaskPlan`.`August`='0' AND `TaskPlan`.`September`='0' 
														AND `TaskPlan`.`October`='0' AND `TaskPlan`.`November`='0' AND `TaskPlan`.`December`='0'
														AND `TaskPlan`.`JanHrs`='0' AND `TaskPlan`.`FebHrs`='0' AND `TaskPlan`.`MarHrs`='0'
														AND `TaskPlan`.`AprHrs`='0' AND `TaskPlan`.`MayHrs`='0' AND `TaskPlan`.`JunHrs`='0' 
														AND `TaskPlan`.`JulHrs`='0' AND `TaskPlan`.`AugHrs`='0' AND `TaskPlan`.`SepHrs`='0' 
														AND `TaskPlan`.`OctHrs`='0' AND `TaskPlan`.`NovHrs`='0' AND `TaskPlan`.`DecHrs`='0'
														AND `TaskPlan`.`Year`= %i 
														ORDER BY `Project`.`Name`,`Task`.`Name`", $year)->fetchAll();
		if ($systemCheck_db_result)
		{
			foreach ($systemCheck_db_result[0] as $attribute => $result)
			{
				$attributes[] = $attribute;
			}
			$systemCheck[0] = $attributes;
			foreach($systemCheck_db_result as $result)
			{
				$systemCheck[] = $this->loadSystemCheckArray($result);
			}
			return $systemCheck;
		}
		else
			return NULL;
	}
}
