<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * UserRole repository access class.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class UserRoleRepository extends Object
{
    /** @var IUserRoleMapper */
 	private $mapper;
	/** @var MultiLogger */
	private $multilog;	

    public function __construct(IUserRoleMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'UserRole-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "User Role"));
    }	
    
    /**
	 * Finds roles by person id.
	 * 
	 * @param string $id person ID.
	 * @return	array of UserRole.
	 */    
    public function findUserRoles($id)
    {
        return $this->mapper->findUserRoles($id);
    }
    
     /**
     * Saves UserRole.
     * 
     * @param UserRole $userRole
     * @return boolean Returns true if create successfull, otherwiste returns false.
     */
    public function save(UserRole $userRole)
    {
        if($this->mapper->save($userRole))
		{
			$this->multilog->log('Role: ' . $userRole->Role . ' saved for person: ' . $userRole->ID, ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
    }
    
    /**
     * Deletes UserRole.
     * 
     * @param string $role
     * @param string $person_id
     * @return boolean Return true if delete successfull, otherwise returns false.
     */
    public function delete($person_id, $role)
    {
		if($this->mapper->delete($person_id, $role))
		{
			$this->multilog->log('Role: ' . $role . ' deleted for person: ' . $person_id, ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
    }   
}
