<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of person/employee's work effort.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class WorkRepository extends Object
{

    /** @var IWorkMapper */
 	private $mapper;

    public function __construct(IWorkMapper $mapper)
    {
        $this->mapper = $mapper;
    }	
	
	/**
	 * Returns hours a worker spent on a given task in given month and year.
	 * 
	 * @param string $id person ID.
	 * @param int $id task ID.
	 * @param int $month month in a year (1-12)
	 * @param int $year year
	 * @return	float
	 */
	public function findPersonTaskMonthWork($person_id, $task_id, $month, $year)
	{
		return $this->mapper->findPersonTaskMonthWork($person_id, $task_id, $month, $year);
	}
	
	/**
	 * Returns array of hours a worker spent on year.
	 *   
	 * @param string $id person ID.
	 * @param int $year year.
	 * @return	MonthArray associative array of twelve floats where key is month.
	 */
	public function findPersonYearWork($person_id, $year)
	{
		return $this->mapper->findPersonYearWork($person_id, $year);
	}

	/**
	 * Returns array of hours a worker spent on a given task in given year.
	 *   
	 * @param string $id person ID.
	 * @param int $id task ID.
	 * @param int $year year.
	 * @return	MonthArray associative array of twelve floats where key is month.
	 */
	public function findPersonTaskYearWork($person_id, $task_id, $year)
	{
		return $this->mapper->findPersonTaskYearWork($person_id, $task_id, $year);
	}

	/**
	 * Returns hours a worker spent on a given task in given year.
	 *   
	 * @param string $id person ID.
	 * @param int $id task ID.
	 * @param int $year year.
	 * @return	float
	 */
	public function findPersonTaskYearWorkSum($person_id, $task_id, $year)
	{
		$work = $this->mapper->findPersonTaskYearWork($person_id, $task_id, $year);
		$sum = 0.0;
		foreach ($work as $hours)
			$sum += $hours;
		return $sum;
	}
	
	/**
	 * Returns array of hours a worker spent on a given task in given year for all worker.
	 * 
	 * @param int $task_id
	 * @param int $year
	 * @return array Associative array hours[EID of worker][index of month (1..12)] => number of hours a worker spent on task in selected month
	 */
	public function findPeopleTaskYearWork($task_id, $year)
	{
		return $this->mapper->findPeopleTaskYearWork($task_id, $year);
	}

	/**
	 * Returns array of sums of hours that workers spent on a given task in given year.
	 *   
	 * @param int $id task ID.
	 * @param int $year year.
	 * @return	array associative array of twelve floats where key is month (indexes 1 to 12).
	 */
	public function findTaskYearWork($task_id, $year)
	{
		return $this->mapper->findTaskYearWork($task_id, $year);
	}

	/**
	 * Returns array of ids of workers that were working on a task in given year.
	 *   
	 * @param int $id task ID.
	 * @param int $year year.
	 * @return	array array of ids of workers (EIDs).
	 */
	public function findPeopleWorkingOnTaskInYear($task_id, $year)
	{
		return $this->mapper->findPeopleWorkingOnTaskInYear($task_id, $year);
	}
    
     /**
     * Returns array of ids of workers that are scheduled to work on a task in given year (Person_Task table).
     * 
     * @param int $task_id
     * @param int $year
     * @return  array Array of ids of workers (EIDs).
     */
    public function findPeopleScheduledToWorkOnTaskInYear($task_id, $year)
    {
        return $this->mapper->findPeopleScheduledToWorkOnTaskInYear($task_id, $year);
    }

     /**
     * Returns associative array of effort/hours/actuals spent on tasks in month.
	 * Key in the array is Task ID and value is spent hours / actuals.
     * 
     * @param int $task_id
     * @param int $month Month number (1-12).
     * @param int $year
     * @return  array Associative array.
     */
    public function findTasksPersonWorkedOnInMonth($person_id, $month, $year)
    {
        return $this->mapper->findTasksPersonWorkedOnInMonth($person_id, $month, $year);
    }
	
	/**
	 * Returns array of hours a worker spent on a given project per month in given year for all worker.
	 * 
	 * @param string $person_id
	 * @param int $project_id
	 * @param int $year
	 * @return array Associative array hours[index of month (1..12)] => number of hours a worker spent on project in selected month
	 * @throws InvalidArgumentException
	 */
	public function findPersonProjectMonthWork($person_id, $project_id, $year)
	{
		return $this->mapper->findPersonProjectMonthWork($person_id, $project_id, $year);
	}
	
}

