<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * iASAP access class for person import
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PersonReader extends IASAPWrapper
{
	/**
	 * @param array $tasks Array of Tasks
	 * @param ILogger $logger
	 */
	public function __construct(ILogger $logger = NULL)
	{
		parent::__construct($logger);
	}
	
	/**
	 * Reads Persons from iASAP.
	 * Quantity of readed data is determined by start.
	 * @return array Contains for each task SAP ID set of people.
	 *     And for each person it contains sum of actual hours in defined fiscal months.
	 *     $table[SAP_ID][Person_EID] = PersonData
	 *     For empty input set of tasks or in case of error returns empty array.
	 */
	public function readPerson()
	{
		$date = strtotime("-200 day");
		$start_date = Date('Y-m-d', $date);
		
		$table = array();
		$db_result = $this->conn
			->query("
				SELECT DISTINCT [Emp ID] AS [EID], [Network], [Employee Name] AS [Name], [Employee Work Center] AS [WCID], [Supervisor]
				FROM [input_asap_yhr_taskgrp]
				WHERE [Date] >= %s
			", $start_date)
			->fetchAll();
		if ($db_result)
		{
			foreach ($db_result as $row)
			{
				$eid = $this->repairEID($row->EID);
				$supervisorEID = $this->repairEID($row->Supervisor);
				$table[$row->Network][$eid]['EID'] = $eid;
				$table[$row->Network][$eid]['Name'] = $row->Name;
				$table[$row->Network][$eid]['WorkCenterID'] = $row->WCID;
				$table[$row->Network][$eid]['SupervisorEID'] = $supervisorEID;
			}
		}
		return $table;
	}
}
