<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package utils\Logging
 */

/**
 * ILogger is is generic inteface to output logging.
 * 
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
interface ILogger
{
	/** Default log levels */
	const 
		DEBUG = 'debug', // debug messages
		INFO = 'info', // general messages or formatting purposes
		NOTICE = 'notice', // information to take into account
		WARNING = 'warning', // needs attention
		ERROR = 'error', // error occurred
		FATAL = 'fatal'; // serious which could not be handled - application should/could fail

	/**
	 * Logs message into log file (if opened).
	 * For $level only these values are processed:
	 * ILogger::[INFO|NOTICE|WARNING|ERROR|FATAL]
	 * 
	 * @param string $message Message text.
	 * @param mixed $level message level. Default INFO.
	 * @param string $domain domain of logged message (can be used for grouping or filtering later).
	 * @param mixed $context variable logging context (useful for special log parameters).
	 * @return boolean Returns TRUE if message was written.
	 */
    public function log($message, $level = self::INFO, $domain = NULL, $context = NULL);
}


