<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * NumericUtils is a library for numbers.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @static
 */
class NumericUtils
{

	/**
	 * Test whether given value holds integer number.
	 * 
	 * Does not test data type but content of data. Correct value
	 * may be string, integer or float containing positive integer value.
	 *
	 * @param mixed $value tested value (string, int, float).
	 * @static
	 * @return boolean
	 */
	public static function isInteger($value)
	{
		if (is_numeric($value) 
			&& ((0 + $value) == $value) // typing to integer
			&& (round(0 + $value) == (0 + $value)))
			return true;
		else
			return false;
	}	
	/**
	 * Test whether given value holds positive integer number.
	 * 
	 * Does not test data type but content of data. Correct value
	 * may be string, integer or float containing positive integer value.
	 *
	 * @param mixed $value tested value (string, int, float).
	 * @static
	 * @return boolean
	 */
	public static function isNonNegativeInteger($value)
	{
		if (NumericUtils::isInteger($value)
			&& ((0 + $value) >= 0))
			return true;
		else
			return false;
	}

	/**
	 * Test whether given array holds non-negative integer numbers.
	 * 
	 * Does not test data type but content of data. Correct value
	 * may be string, integer or float containing positive integer value.
	 *
	 * @param array $array tested array.
	 * @static
	 * @return boolean
	 */
	public static function isNonNegativeIntegerArray($array)
	{
		if (!is_array($array) && (!($array instanceof Traversable)))
			return false;
		else
			foreach ($array as $value)
				if (!NumericUtils::isNonNegativeInteger($value))
					return false;
		return true;
	}		
	/**
	 * Test whether given value holds positive integer number.
	 * 
	 * Does not test data type but content of data. Correct value
	 * may be string, integer or float containing positive integer value.
	 *
	 * @param mixed $value tested value (string, int, float).
	 * @static
	 * @return boolean
	 */
	public static function isPositiveInteger($value)
	{
		if (NumericUtils::isNonNegativeInteger($value)
			&& ((0 + $value) > 0))
			return true;
		else
			return false;
	}	

	/**
	 * Test whether given array holds positive integer numbers.
	 * 
	 * Does not test data type but content of data. Correct value
	 * may be string, integer or float containing positive integer value.
	 *
	 * @param array $array tested array.
	 * @static
	 * @return boolean
	 */
	public static function isPositiveIntegerArray($array)
	{
		if (!is_array($array) && (!($array instanceof Traversable)))
			return false;
		else
			foreach ($array as $value)
				if (!NumericUtils::isPositiveInteger($value))
					return false;
		return true;
	}	
	
}


