/**
 * This file is part of the HON Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * 
 * Definition of behavior of block for separating sections of page
 */

/**
 * Tab visibility change according to current state
 */
$(document).on('click', '.ddt-header', function(){
    var tab_name = $(this).parent().attr('data-name');
    var tab = new DropDownTab(tab_name);
    tab.switchState();
});

/**
 * Initialization of tabs according to previous setting
 */
$(document).ready(function(){
    $('.drop-down-tab').each(function(){
        var tab_name = $(this).attr('data-name');
        var tab = new DropDownTab(tab_name);
        tab.init();
    });
});    

/**
 * Definition of block for separating sections of page
 * @param string name Unique name which is used to distinguish tabs
 */
var DropDownTab = function (name)
{
    /** @var string Tab identification key */
    this.name = name;
    /** @var object Header section of tab */
    this.header = $(document).find('.drop-down-tab[data-name="' + name + '"]').find('.ddt-header');
    /** @var object Info section of tab */
    this.info = this.header.find('.ddt-info');
    /** @var object Content section of tab */
    this.content = $(document).find('.drop-down-tab[data-name="' + name + '"]').find('.ddt-content');
}

/**
 * Sets tab visibility state to local storage if exists
 * @param boolean visibility True for visible tab, false otherwise
 */
DropDownTab.prototype.setVisibility = function(visibility) {
    if (typeof(Storage) !== 'undefined')
    {
        if (visibility)
            localStorage.setItem(this.name, '1');
        else
            localStorage.setItem(this.name, '');
    }
};

/**
 * Returns true if tab is currently visible, false otherwise
 * @return boolean
 */
DropDownTab.prototype.isVisible = function() {
    if (typeof(Storage) !== 'undefined')
        return (localStorage.getItem(this.name)) ? true : false;
    else
        return (this.header.hasClass('ddt-open')) ? true : false;
};

/**
 * Opens tab
 */
DropDownTab.prototype.open = function() {
    this.header.removeClass('ddt-closed').addClass('ddt-open');
    this.info.addClass('invisible');
    this.content.removeClass('invisible');
    this.setVisibility(true);
};

/**
 * Closes tab
 */
DropDownTab.prototype.close = function() {
    this.header.removeClass('ddt-open').addClass('ddt-closed');
    this.info.removeClass('invisible');
    this.content.addClass('invisible');
    this.setVisibility(false);
};

/**
 * Sets tab state according to previous setting
 */
DropDownTab.prototype.init = function() {
	//TEMP HARCODED FOR Project EAC
	if(this.name === 'projectEAC' || this.name === 'workpackageEAC' || this.name === 'projectPlanningCharter')
	{
		this.close();
	}
	else
	{
		if (this.isVisible())
			this.open();
		else
			this.close();
	}
};

/**
 * Sets opposite tab state
 */
DropDownTab.prototype.switchState = function() {
    if (this.isVisible())
        this.close();
    else
        this.open();
};


